package org.objectweb.telosys.auth;

/**
 * @author Laurent GUERIN
 * 
 * This class contains static methods to managed http authentication.
 *  
 */

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.objectweb.telosys.common.Telosys;
import org.objectweb.telosys.common.TelosysClassLogger;
import org.objectweb.telosys.common.TelosysFactory;
import org.objectweb.telosys.screen.env.ScreenSessionManager;
import org.objectweb.telosys.util.web.WebUtil;

public final class Auth 
{
    private static final TelosysClassLogger log = new TelosysClassLogger(Auth.class);

    private static IUserValidator getApplicationUserValidator()
    {
        //TelosysFactory factory = TelosysScreen.getFactory();
        TelosysFactory factory = Telosys.getFactory();
        if (factory != null)
        {
            return factory.getApplicationUserValidator();
        }
        log.error("Cannot get Telosys factory ! ");
        return null;
    }

    private static IUserValidator getDefaultUserValidator()
    {
        return new DefaultUserValidator();
    }

    //------------------------------------------------------------------------
    /**
     * Build a http response with error code "401-Unauthorized"<br>
     * and header "WWW-Authenticate"
     * @param request
     * @param response
     */
    private static void buildAuthenticateResponse(HttpServletRequest request, HttpServletResponse response)
    {
        log.trace("buildAuthenticateResponse()...");
        //--- Set the "real name" (printed in the navigator dialog-box)
        String sRealmName = TelosysAUTH.getRealmName();
        if (sRealmName == null)
        {
            sRealmName = "";
        }
        response.setHeader("WWW-Authenticate", "Basic realm=\"" + sRealmName + "\"");
        //--- Code Retour HTTP "401-Unauthorized"
        response.setStatus(HttpServletResponse.SC_UNAUTHORIZED);
    }

    //------------------------------------------------------------------------
    /**
     * Return the LoginUser object from the http request
     * @param request
     * @return
     */
    private static LoginUser getUserFromAuthorizationRequest(HttpServletRequest request)
    {
        //--- Get the "Authorization" HTTP header
        String sAuthorization = request.getHeader("Authorization");
        if (sAuthorization == null)
        {
            return null;
        }
        //--- Check the authorization type : "scheme BASIC"
        if (!sAuthorization.toLowerCase().startsWith("basic "))
        {
            return null;
        }
        //--- Get the Base64 string
        String sEncoded = sAuthorization.substring(6).trim();
        //--- Decode the Base64 string and parse it 
        String sDecoded = new String(Base64.decode(sEncoded.getBytes()));
        int iPosiColon = sDecoded.indexOf(':');
        if (iPosiColon < 0)
        {
            return null;
        }
        //--- Get the user login and password
        String sUserLogin = sDecoded.substring(0, iPosiColon).trim();
        String sUserPassword = sDecoded.substring(iPosiColon + 1).trim();

        //--- Retun the LoginUser instance
		LoginUser user = new LoginUser(sUserLogin, sUserPassword) ;        
        return user;
    }

    /**
     * Validate the login user (delegate to the application)
     * @param user :
     *            LoginUser to validate
     * @return : Application user if validate, null if not validate
     */
    private static IAppUser validUser(final LoginUser user)
    {
        IUserValidator applicationUserValidator = getApplicationUserValidator();
        if (applicationUserValidator != null)
        {
            return applicationUserValidator.validUser(user.getLogin(), user.getPassword());
        }
        else
        {
            IUserValidator defaultUserValidator = getDefaultUserValidator();
            if (defaultUserValidator != null)
            {
                return defaultUserValidator.validUser(user.getLogin(), user.getPassword());
            }
            else
            {
                return null;
            }
        }
    }
    
    /**
     * Check if the given http request contains an authenticated user
     * @param request
     * @return true if authenticated (else false)
     */
    public static boolean isAuthenticated(HttpServletRequest request)
    {        
//        //--- Is there a ScreenSession ?
//        ScreenSession screenSession = ScreenSessionManager.getScreenSession(request);
//        if (screenSession == null)
//        {
//            // No ScreenSession => Not Authenticated
//            log.trace("isAuthenticated(request) : FALSE ( No ScreenSession ).");
//            return false ;
//        }
        
        //--- Is there an authenticated user in the ScreenSession ?
        LoginUser user = ScreenSessionManager.findScreenSessionUser(request);
        if (user == null)
        {
            // No User in the ScreenSession => Not Authenticated
            log.trace("isAuthenticated(request) : FALSE ( No user in the ScreenSession ).");
            return false ;
        }
        //--- OK ; User authenticated.
        log.trace("isAuthenticated(request) : TRUE.");
        return true ;        
    }
    //---------------------------------------------------------------------------
    /**
     * Check the "login/password" authentication.
     * If the authentication is invalid : build a "401-Unauthorized" response
     * If the authentication is valid : send a "Redirect" response to "First Page URL"
     * @param request
     * @param response
     */
    public static void checkAuthentication(HttpServletRequest request, HttpServletResponse response)
    {
        String sUrlPage = null;
        //--- Is there a login/password in the  Http Request Header  ?
        LoginUser loginUser = Auth.getUserFromAuthorizationRequest(request);
        if (loginUser == null) // No "login/password" in the Request Header 
        {
            log.trace("login/password not found in request");
            //--- Response = Authentication ( "401-Unauthorized" )
            Auth.buildAuthenticateResponse(request, response);
        }
        else
        {
            log.trace("login/password in request ( " + loginUser.toString() + " )");
            //--- Delegate the user validation to the application validator instance
            IAppUser appUser = Auth.validUser(loginUser);
            if (appUser != null) // "login/password" is OK
            {
                log.trace("login/password OK");

                //--- OPEN a SCREEN SESSION for this user
                ScreenSessionManager.openScreenSession(request, loginUser, appUser);

                //--- Try to get a specific First Page for the user
                sUrlPage = appUser.getFirstPage();
                if (sUrlPage == null)
                {
                    //--- Try to get the Application's First Page
                    sUrlPage = TelosysAUTH.getFirstPage();
                    if (sUrlPage == null)
                    {
                        //--- Default first page : WebApp root
                        sUrlPage = WebUtil.getRootUrl(request);
                    }
                }
                //--- "redirect" to transmit the good URL
                WebUtil.redirect(sUrlPage, response);
            }
            else
            {
                log.trace("invalid login/password");
                //--- Response = Authentication ( "401-Unauthorized" )
                Auth.buildAuthenticateResponse(request, response);
            }
        }
    }
}