package org.objectweb.telosys.common;

import org.objectweb.telosys.auth.IUserValidator;
import org.objectweb.telosys.rpl.xml.IXmlWrapperProvider;
import org.objectweb.telosys.screen.core.IScreenDataProvider;
import org.objectweb.telosys.screen.core.IScreenProvider;
import org.objectweb.telosys.screen.core.ScreenContext;
import org.objectweb.telosys.screen.core.ScreenData;
import org.objectweb.telosys.screen.env.ScreenSession;
import org.objectweb.telosys.service.IServiceProvider;

/**
 * Global TELOSYS factory. <br>
 * It encapsulates and uses the "external factory" provided by the application
 * 
 * @author Laurent GUERIN
 *  
 */
public class TelosysFactory extends TelosysObject
{
    private IExternalMainFactory _externalFactory = null;

    /**
     *  
     */
    public TelosysFactory(IExternalMainFactory extFactory)
    {
        trace("TelosysFactory constructor ...");
        if (extFactory == null)
        {
            error("TelosysFactory constructor : external factory is NULL !");
        }
        _externalFactory = extFactory;
    }

    /**
     * Create a new screen context using the "external factory"
     * 
     * @param sScreenName
     * @param screenSession
     * @return
     * @throws TelosysException
     */
    public ScreenContext createScreen(ScreenSession screenSession, String sScreenName, int iScreenId ) throws TelosysException
    {
        trace("createScreen(" + sScreenName + ") ...");
        ScreenContext screen = null;
        if (sScreenName != null)
        {
            //--- Use the external main factory
            if (_externalFactory != null)
            {
                IScreenProvider screenProvider = _externalFactory.getScreenProvider();
                if (screenProvider != null)
                {
                    //--- Use the screen provider to get a screen instance
                    //screen = screenProvider.getScreen(sScreenName, screenSession);
                    screen = screenProvider.getScreen(screenSession, sScreenName, iScreenId );
                    if (screen != null)
                    {
                        trace("createScreen() : object created ");
                    }
                    else
                    {
                        error("createScreen() : cannot create object ");
                    }
                }
                else
                {
                    error("createScreen() : screenProvider is null");
                }
            }
            else
            {
                error("createScreen() : external factory is null");
            }
        }
        else
        {
            error(" createScreen() : ScreenName is null !");
        }
        return screen;
    }

    /**
     * Create a new ScreenData using the "external factory"
     * 
     * @param sScreenName
     * @return
     * @throws TelosysException
     */
    public ScreenData createScreenData(String sScreenName) throws TelosysException
    {
        trace("createScreenData(" + sScreenName + ") ...");
        ScreenData screenData = null;
        if (sScreenName != null)
        {
            //--- Use the external main factory
            if (_externalFactory != null)
            {
                //--- Get the external "Screen Data Provider"
                IScreenDataProvider screenDataProvider = _externalFactory.getScreenDataProvider();
                if (screenDataProvider != null)
                {
                    screenData = screenDataProvider.getScreenData(sScreenName);
                    if (screenData != null)
                    {
                        trace("createScreenData() : object created ");
                    }
                    else
                    {
                        error("createScreenData() : cannot create object ");
                    }
                }
                else
                {
                    error("createScreenData() : screenDataProvider is null");
                }
            }
            else
            {
                error("createScreenData() : external factory is null");
            }
        }
        else
        {
            error(" createData() : ScreenName is null !");
        }
        return screenData;
    }

    //-----------------------------------------------------------------------------------------------
    /**
     * Return a UserValidator from the "external factory"
     * 
     * @return
     */
    public IUserValidator getApplicationUserValidator()
    {
        IUserValidator validator = null;
        //--- Use the external main factory
        if (_externalFactory != null)
        {
            //--- Get the external "Screen Data Provider"
            validator = _externalFactory.getUserValidator();
            if (validator != null)
            {
                trace("getApplicationUserValidator() : OK");
            }
            else
            {
                error("getApplicationUserValidator() : cannot get validator ");
            }
        }
        else
        {
            error("getApplicationUserValidator() : external factory is null");
        }
        return validator;
    }

    //-----------------------------------------------------------------------------------------------
    /**
     * Return a XmlWrapperProvider from the "external factory"
     * 
     * @return
     */
    public IXmlWrapperProvider getXmlWrapperProvider()
    {
        IXmlWrapperProvider wrapperProvider = null;
        //--- Use the external main factory
        if (_externalFactory != null)
        {
            //--- Get the external "Screen Data Provider"
            wrapperProvider = _externalFactory.getXmlWrapperProvider();
            if (wrapperProvider != null)
            {
                trace("getXmlWrapperProvider() : OK");
            }
            else
            {
                error("getXmlWrapperProvider() : cannot get provider ");
            }
        }
        else
        {
            error("getXmlWrapperProvider() : external factory is null");
        }
        return wrapperProvider;
    }

    //-----------------------------------------------------------------------------------------------
    /**
     * Return a Service provider from the "external factory"
     * 
     * @return
     */
    public IServiceProvider getServiceProvider()
    {
        IServiceProvider provider = null;
        //--- Use the external main factory
        if (_externalFactory != null)
        {
            //--- Get the external "Screen Data Provider"
            provider = _externalFactory.getServiceProvider();
            if (provider != null)
            {
                trace("getServiceProvider() : OK");
            }
            else
            {
                error("getServiceProvider() : cannot get provider ");
            }
        }
        else
        {
            error("getServiceProvider() : external factory is null");
        }
        return provider;
    }
}