package org.objectweb.telosys.common;

/**
 * Global "tool" methods.
 * 
 * @author Laurent GUERIN
 *  
 */

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.PrintStream;
import java.net.URL;
import java.util.Enumeration;
import java.util.Properties;

import javax.servlet.ServletConfig;
import javax.servlet.ServletContext;

public class Tool
{
    private static final TelosysClassLogger log                = new TelosysClassLogger(Tool.class);

    private static final String             TELOSYS_PROPERTIES = "telosys.properties";

    //-----------------------------------------------------------------------------
    // FILE & IO
    //-----------------------------------------------------------------------------
    /**
     * Close an input stream without exception
     * 
     * @param in
     */
    public static void closeInputStream(InputStream in)
    {
        try
        {
            if (in != null)
            {
                in.close();
            }
        } catch (IOException ex)
        {
        }
    }

    //-----------------------------------------------------------------------------
    /**
     * Check the existence of a file
     * 
     * @param sFileName
     * @return
     */
    private static boolean checkFileExistence(String sFileName)
    {
        File file = new File(sFileName);
        if (file.exists())
        {
            log.trace("The file '" + sFileName + "' exists.");
            return true;
        }
        else
        {
            log.trace("The file '" + sFileName + "' does not exists !");
            return false;
        }
    }

    //-----------------------------------------------------------------------------
    /**
     * Check the existence of a file
     * 
     * @param sFileName
     * @return
     */
    public static boolean fileExists(String sFileName)
    {
        File file = new File(sFileName);
        return file.exists();
    }

    /**
     * Find a file using the CLASSPATH
     * 
     * @param sFileName
     * @return
     */
    public static String findFileByClassPath(String sFileName)
    {
        log.trace("Try to find file '" + sFileName + "' using the CLASSPATH... ");
        log.trace("CLASSPATH = " + System.getProperty("java.class.path"));
        URL url = ClassLoader.getSystemResource(sFileName);
        if (url != null)
        {
            log.trace("File found by CLASSPATH : URL = " + url);
            File file = new File(url.getFile());
            return file.getPath();
        }
        else
        {
            log.trace("File not found in CLASSPATH ");
        }
        return null;
    }

    //-----------------------------------------------------------------------------
    /**
     * Try to find the standard TELOSYS properties file. <br>
     * Step 1 - From init parameter of the "web.xml" <br>
     * Step 2 - From the "WEB-INF/conf" directory <br>
     * Step 3 - From the CLASSPATH
     * 
     * @param config
     *            the servlet config
     * @return
     */
    public static String findTelosysPropertiesFile(ServletConfig config)
    {
        String sTelosysPropFile = null;
        if (config != null)
        {
            //---------------------------------------------------------------------------------------
            //--- (1) Try to find the Prop File name in the "web.xml" ( in servlet init parameters )
            //---------------------------------------------------------------------------------------
            log.info("[1] Try to find telosys.properties with init parameter of the 'web.xml' ... ");
            sTelosysPropFile = config.getInitParameter("TelosysProperties");
            if (sTelosysPropFile != null)
            {
                log.info("[1] telosys.properties parameter found in web.xml : Param = " + sTelosysPropFile);
                if (checkFileExistence(sTelosysPropFile))
                {
                    log.info("[1] OK : telosys.properties found : File = " + sTelosysPropFile);
                }
                else
                {
                    log.info("[1] The proterties file '" + sTelosysPropFile + "' does not exists");
                    sTelosysPropFile = null;
                }
            }
            else
            {
                log.info("[1] telosys.properties parameter not found in web.xml ");
            }

            //---------------------------------------------------------------------------------------
            //--- (2) Try to find the Prop File in the "WEB-INF/conf" directory
            //---------------------------------------------------------------------------------------
            if (sTelosysPropFile == null)
            {
                log.info("[2] Try to find telosys.properties in the 'WEB-INF/conf' directory ... ");
                ServletContext context = config.getServletContext();
                if (context != null)
                {
                    sTelosysPropFile = context.getRealPath("/WEB-INF/conf/" + TELOSYS_PROPERTIES);
                    if (sTelosysPropFile != null)
                    {
                        if (checkFileExistence(sTelosysPropFile))
                        {
                            log.info("[2] OK : telosys.properties found in the default 'conf' directory : File = " + sTelosysPropFile);
                        }
                        else
                        {
                            log.info("[2] telosys.properties is not in the default 'conf' directory ");
                            sTelosysPropFile = null;
                        }
                    }
                    else
                    {
                        log.error("[2] Cannot find WebApp context root : context.getRealPath() return null !");
                    }
                }
                else
                {
                    log.error("[2] ServletContext not found");
                }
            }
        }
        else
        {
            log.error("ServletConfig not found : getServletConfig() return null !");
        }

        //---------------------------------------------------------------------------------------
        //--- (3) Try to find the Prop File using the classpath
        //---------------------------------------------------------------------------------------
        if (sTelosysPropFile == null)
        {
            log.info("[3] Try to find telosys.properties using the CLASSPATH... ");
            log.info("[3] CLASSPATH = " + System.getProperty("java.class.path"));
            sTelosysPropFile = findFileByClassPath(TELOSYS_PROPERTIES);
            if (sTelosysPropFile != null)
            {
                log.info("[3] telosys.properties found by CLASSPATH : " + sTelosysPropFile);
            }
            else
            {
                log.info("[3] telosys.properties not found in CLASSPATH ");
            }
        }
        return sTelosysPropFile;
    }

    /**
     * Try to find the standard telosys properties using the CLASSPATH
     * @return
     */
    public static String findTelosysPropertiesFile()
    {
        log.info("Try to find telosys.properties using the CLASSPATH... ");
        String sTelosysPropFile = findFileByClassPath(TELOSYS_PROPERTIES);
        if (sTelosysPropFile != null)
        {
            log.info("telosys.properties found by CLASSPATH : " + sTelosysPropFile);
        }
        else
        {
            log.info("telosys.properties not found in CLASSPATH ");
        }
        return sTelosysPropFile;
    }

    //-----------------------------------------------------------------------------
    /**
     * Try to find a properties file with classpath and load it (if found)
     * 
     * @param sFileName
     * @return
     */
    public static Properties findAndLoadProperties(String sFileName)
    {
        log.trace("findAndLoadProperties() : Try to find a properties file with CLASSPATH and load it");
        Properties prop = null;
        try
        {
            URL url = ClassLoader.getSystemResource(TELOSYS_PROPERTIES);
            if (url != null)
            {
                prop = new Properties();
                InputStream in = url.openStream();
                prop.load(in);
                closeInputStream(in);
            }
        } catch (IOException ex)
        {
            log.error("findAndLoadProperties() : IOException : " + ex.getMessage());
        }
        return prop;
    }

    //-----------------------------------------------------------------------------
    /**
     * Load properties from a specific file name and return them
     * 
     * @param sFileName
     * @return
     */
    public static Properties loadProperties(String sFileName)
    {
        log.trace("loadProperties() : FileName = " + sFileName);
        Properties prop = null;
        FileInputStream in = null;
        try
        {
            in = new FileInputStream(sFileName);
            prop = new Properties();
            prop.load(in);
            //prop.list(System.out); // Affichage du contenu
        } catch (IOException ex)
        {
            log.error("loadProperties() : IOException : " + ex.getMessage());
        } finally
        {
            if (in != null)
                Tool.closeInputStream(in);
        }

        return prop;
    }

    //-----------------------------------------------------------------------------
    /**
     * Print the content of the properties on the standard output
     * 
     * @param prop
     */
    public static void showProperties(Properties prop)
    {
        showProperties(prop, System.out);
    }

    //-----------------------------------------------------------------------------
    /**
     * Print the content of the properties on a specific output
     * 
     * @param prop
     * @param out
     */
    public static void showProperties(Properties prop, PrintStream out)
    {
        Enumeration enumKeys = prop.keys();
        String sKey = null;
        Object obj = null;
        out.println(" ---------- ");
        while (enumKeys.hasMoreElements()) // Pour chaque "cl" dans les properties
        {
            sKey = (String) enumKeys.nextElement();
            obj = prop.get(sKey);
            if (obj != null)
            {
                out.println(" . " + sKey + " : " + obj.toString());
            }
        }
        out.println(" ---------- ");
    }

}