package org.objectweb.telosys.common.data;

import org.objectweb.telosys.common.TelosysObject;
import org.objectweb.telosys.common.TelosysRuntimeException;

/**
 * This class is a row composed of 1 to N cells of data <br>
 * Each cell is indexed from 1 to N.
 * 
 * @author Laurent GUERIN
 */
public class DataRow extends TelosysObject
{
    //-----------------------------------------------------------------------------
    //--- PRIVATE ATTRIBUTES
    //-----------------------------------------------------------------------------
    private Object _ColValues[] = null; // Values ( Columns )

    private int    _iSize       = 0;   // Number of usable cells

    //-----------------------------------------------------------------------------
    //--- CONSTRUCTORS
    //-----------------------------------------------------------------------------
    private DataRow()
    {
    }

    /**
     * Create a DataRow with N columns
     * 
     * @param iSize
     *            the row size (the number of columns)
     */
    public DataRow(int iSize)
    {
        trace("DataRow(" + iSize + ") : Constructor ");
        //--- Create the row cells ( array )
        _ColValues = new Object[iSize + 1]; // +1 for index from 1 to N
        _iSize = iSize;
    }

    /**
     * Create a DataRow with N columns and populate it with the given values
     * 
     * @param iSize
     *            the row size (the number of columns)
     * @param values
     *            the values to populate the row cells ( standard Array : values from 0 to length-1 )
     */
    public DataRow(int iSize, Object[] values)
    {
        trace("DataRow(" + iSize + ", values ) : Constructor ... ");
        //--- Create the row cells ( array )
        _ColValues = new Object[iSize + 1]; // +1 for index from 1 to N
        _iSize = iSize;
        if (values != null)
        {
            trace("DataRow(" + iSize + ", values ) : values.length = " + values.length);
            int iValuesLength = values.length;
            for ( int iCol = 0 ; iCol < iValuesLength ; iCol++ ) // 0 to N-1 ( ARRAY )
            {
                if (iCol < _iSize)
                {
                    trace("DataRow(" + iSize + ", values ) : " + (iCol + 1) + " <- " + iCol);
                    _ColValues[iCol + 1] = values[iCol];
                }
            }
        }
    }

    //-----------------------------------------------------------------------------
    //--- PRIVATE/PROTECTED METHODS
    //-----------------------------------------------------------------------------
    private void checkIndex(int i)
    {
        if (i < 1)
        {
            throw new TelosysRuntimeException("DataRow : column index < 1");
        }
        if (i > _iSize)
        {
            throw new TelosysRuntimeException("DataRow : column index > " + _iSize);
        }
    }

    /**
     * Set a new object at the given column position ( 1 to N )
     * 
     * @param i
     *            the colum
     * @param obj
     */
    protected void setObject(int i, Object obj)
    {
        checkIndex(i);
        _ColValues[i] = obj;
    }

    //-----------------------------------------------------------------------------
    //--- PUBLIC METHODS
    //-----------------------------------------------------------------------------
    //--------------------------------------------------------------------------
    public String toString()
    {
        StringBuffer sb = new StringBuffer(200);
        for ( int i = 1 ; i < _ColValues.length ; i++ )
        {
            if (i > 1)
            {
                sb.append("|");
            }
            sb.append(_ColValues[i]);
        }
        return sb.toString();
    }

    //--------------------------------------------------------------------------
    /**
     * Return the size of the row (the number of cells) <br>
     * ie : size = 3 for 3 cells (1 to 3)
     * 
     * @return
     */
    public int getSize()
    {
        return _iSize;
    }

    //--------------------------------------------------------------------------
    // Rcupration des valeurs du Record ( par No de colonne )
    // Mthodes identiques  celles de "ResultSet"
    //--------------------------------------------------------------------------
    /**
     * Return the object stored at the given position
     * 
     * @param i
     *            position (1 to N)
     * @return
     */
    public Object getObject(int i)
    {
        checkIndex(i);
        return _ColValues[i];
    }

    //--------------------------------------------------------------------------
    // STRING
    //--------------------------------------------------------------------------
    /**
     * Return the String object stored at the given position
     * 
     * @param i
     *            position (1 to N)
     * @return string instance (or null)
     */
    public String getString(int i)
    {
        checkIndex(i);
        return getString(i, null);
    }

    //--------------------------------------------------------------------------
    /**
     * Return the String object stored at the given position
     * 
     * @param i
     *            position (1 to N)
     * @param valueIfNull
     *            the string to return if there's no object at the given position
     * @return string instance
     */
    public String getString(int i, String valueIfNull)
    {
        checkIndex(i);
        if (_ColValues[i] == null)
        {
            return valueIfNull;
        }
        if (_ColValues[i] instanceof String)
        {
            return (String) _ColValues[i];
        }
        else
        {
            return _ColValues[i].toString();
        }
    }

    //--------------------------------------------------------------------------
    // BOOLEAN
    //--------------------------------------------------------------------------
    /**
     * Return the boolean value of the given position <br>
     * return false if null
     * 
     * @param i
     *            position in the row (1 to N)
     * @return
     */
    public boolean getBoolean(int i) 
    {
        return getBoolean(i, false);
    }

    /**
     * Return the boolean value of the given position
     * 
     * @param i
     *            position in the row (1 to N)
     * @param valueIfNull
     *            the value to return if there's no object at the given position
     * @return
     */
    public boolean getBoolean(int i, boolean valueIfNull) 
    {
        checkIndex(i);
        if (_ColValues[i] == null)
        {
            return valueIfNull;
        }
        if (_ColValues[i] instanceof Boolean)
        {
            return ((Boolean) _ColValues[i]).booleanValue();
        }
        else
        {
            throw new TelosysRuntimeException("DataRow : getBoolean(" + i + "): The column is not a boolean");
        }
    }

    //--------------------------------------------------------------------------
    // NUMBERS : cf "java.lang.Number" super class for
    //     BigDecimal, BigInteger, Byte, Double, Float, Integer, Long, Short
    //--------------------------------------------------------------------------
    /**
     * Return the integer value of the given position (return 0 if null)
     * 
     * @param i
     *            position in the row (1 to N)
     * @return
     */
    public int getInt(int i) 
    {
        return getInt(i, 0);
    }

    /**
     * Return the integer value of the given position
     * 
     * @param i
     *            position in the row (1 to N)
     * @param valueIfNull
     *            the value to return if there's no object at the given position
     * @return
     */
    public int getInt(int i, int valueIfNull)
    {
        checkIndex(i);
        if (_ColValues[i] == null)
        {
            return valueIfNull;
        }
        if (_ColValues[i] instanceof Number)
        {
            return ((Number) _ColValues[i]).intValue();
        }
        else
        {
            throw new TelosysRuntimeException("DataRow : getInt(" + i + "): The column is not a number");
        }
    }

    //--------------------------------------------------------------------------
    /**
     * Return the long value of the given position
     * 
     * @param i
     *            position in the row (1 to N)
     * @return the value ( 0 if null)
     */
    public long getLong(int i) 
    {
        return getLong(i, 0);
    }

    /**
     * Return the long value of the given position
     * 
     * @param i
     *            position in the row (1 to N)
     * @param valueIfNull
     *            the value to return if there's no object at the given position
     * @return
     */
    public long getLong(int i, long valueIfNull) 
    {
        checkIndex(i);
        if (_ColValues[i] == null)
        {
            return valueIfNull;
        }
        if (_ColValues[i] instanceof Number)
        {
            return ((Number) _ColValues[i]).longValue();
        }
        else
        {
            throw new TelosysRuntimeException("DataRow : getLong(" + i + "): The column is not a number");
        }
    }

    //--------------------------------------------------------------------------
    /**
     * Return the short value of the given position (return 0 if null)
     * 
     * @param i
     *            position in the row (1 to N)
     * @return
     */
    public short getShort(int i) 
    {
        return getShort(i, (short) 0);
    }

    /**
     * Return the short value of the given position
     * 
     * @param i
     *            position in the row (1 to N)
     * @param valueIfNull
     *            the value to return if there's no object at the given position
     * @return
     */
    public short getShort(int i, short valueIfNull) 
    {
        checkIndex(i);
        if (_ColValues[i] == null)
        {
            return valueIfNull;
        }
        if (_ColValues[i] instanceof Number)
        {
            return ((Number) _ColValues[i]).shortValue();
        }
        else
        {
            throw new TelosysRuntimeException("DataRow : getShort(" + i + "): The column is not a number");
        }
    }

    //--------------------------------------------------------------------------
    /**
     * Return the byte value of the given position (return 0 if null)
     * 
     * @param i
     *            position in the row (1 to N)
     * @return
     */
    public byte getByte(int i) 
    {
        return getByte(i, (byte) 0);
    }

    /**
     * Return the byte value of the given position
     * 
     * @param i
     *            position in the row (1 to N)
     * @param valueIfNull
     *            the value to return if there's no object at the given position
     * @return
     */
    public byte getByte(int i, byte valueIfNull) 
    {
        checkIndex(i);
        if (_ColValues[i] == null)
        {
            return valueIfNull;
        }
        if (_ColValues[i] instanceof Number)
        {
            return ((Number) _ColValues[i]).byteValue();
        }
        else
        {
            throw new TelosysRuntimeException("DataRow : getByte(" + i + "): The column is not a number");
        }
    }

    //--------------------------------------------------------------------------
    /**
     * Return the float value of the given position (return 0 if null)
     * 
     * @param i
     *            position in the row (1 to N)
     * @return
     */
    public float getFloat(int i) //throws TelosysException
    {
        return getFloat(i, 0);
    }

    /**
     * Return the float value of the given position
     * 
     * @param i
     *            position in the row (1 to N)
     * @param valueIfNull
     *            the value to return if there's no object at the given position
     * @return
     */
    public float getFloat(int i, float valueIfNull) //throws TelosysException
    {
        checkIndex(i);
        if (_ColValues[i] == null)
        {
            return valueIfNull;
        }
        if (_ColValues[i] instanceof Number)
        {
            return ((Number) _ColValues[i]).floatValue();
        }
        else
        {
            throw new TelosysRuntimeException("DataRow : getFloat(" + i + "): The column is not a number");
        }
    }

    //--------------------------------------------------------------------------
    /**
     * Return the double value of the given position (return 0 if null)
     * 
     * @param i
     *            position in the row (1 to N)
     * @return
     */
    public double getDouble(int i) //throws TelosysException
    {
        return getDouble(i, 0.0);
    }

    /**
     * Return the double value of the given position
     * 
     * @param i
     *            position in the row (1 to N)
     * @param valueIfNull
     *            the value to return if there's no object at the given position
     * @return
     */
    public double getDouble(int i, double valueIfNull) //throws TelosysException
    {
        checkIndex(i);
        if (_ColValues[i] == null)
        {
            return valueIfNull;
        }
        if (_ColValues[i] instanceof Number)
        {
            return ((Number) _ColValues[i]).doubleValue();
        }
        else
        {
            throw new TelosysRuntimeException("DataRow : getDouble(" + i + "): The column is not a number");
        }
    }

    //--------------------------------------------------------------------------
    // DATES
    //--------------------------------------------------------------------------
    /**
     * Return the Date object of the given position (java.util.Date instance)
     * 
     * @param i
     *            position in the row (1 to N)
     * @return
     */
    public java.util.Date getDate(int i) //throws TelosysException
    {
        checkIndex(i);
        if (_ColValues[i] == null)
        {
            return null;
        }
        if (_ColValues[i] instanceof java.sql.Date) // 1) the more specialized class
        {
            //--- java.sql.Date extends java.util.Date => can convert as is
            return (java.util.Date) _ColValues[i];
        }
        else if (_ColValues[i] instanceof java.util.Date) // 2) the standard class
        {
            return (java.util.Date) _ColValues[i];
        }
        else
        {
            throw new TelosysRuntimeException("DataRow : getDate(" + i + "): The column is not a Date");
        }
    }

    //--------------------------------------------------------------------------
    /**
     * Return the Date object of the given position (java.sql.Date instance)
     * 
     * @param i
     *            position in the row (1 to N)
     * @return
     */
    public java.sql.Date getSqlDate(int i) //throws TelosysException
    {
        checkIndex(i);
        if (_ColValues[i] == null)
        {
            return null;
        }

        if (_ColValues[i] instanceof java.sql.Date) // 1) the more specialized class
        {
            return (java.sql.Date) _ColValues[i];
        }
        else if (_ColValues[i] instanceof java.util.Date) // 2) the standard class
        {
            java.util.Date utilDate = (java.util.Date) _ColValues[i];
            //--- Create a "sql.date" from "util.date"
            return new java.sql.Date(utilDate.getTime());
        }
        else
        {
            throw new TelosysRuntimeException("DataRow : getSqlDate(" + i + "): The column is not a Date");
        }
    }
    //--------------------------------------------------------------------------
}