package org.objectweb.telosys.dal.dao;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;

import org.objectweb.telosys.common.TelosysException;
import org.objectweb.telosys.common.TelosysObject;

/**
 * @author Laurent GUERIN
 * 
 */
public class QueryContext extends TelosysObject //implements IConstantsDAO
{
    //============================================================================
    // Constants
    //============================================================================
    private static final int  STORE_BOOLEAN_DEFAULT   = 0;

    private static final int  STORE_BOOLEAN_AS_INT    = 1;

    private static final int  STORE_BOOLEAN_AS_STRING = 2;

    //============================================================================
    // Instance Attributes
    //============================================================================

    private PreparedStatement _ps                     = null;

    private ResultSet         _rs                     = null;

    private int               _iBooleanStorage        = STORE_BOOLEAN_DEFAULT;

    //============================================================================
    // Constructor(s)
    //============================================================================
    public QueryContext(Connection con, String sSql) throws SQLException
    {
        super();
        this._ps = con.prepareStatement(sSql);
        this._rs = null;
        this._iBooleanStorage = STORE_BOOLEAN_DEFAULT;
    }

    public QueryContext(Connection con, String sSql, int iBooleanStorage) throws SQLException
    {
        super();
        this._ps = con.prepareStatement(sSql);
        this._rs = null;
        this._iBooleanStorage = iBooleanStorage;
    }

    //============================================================================
    // Methods
    //============================================================================
    protected int executeUpdate() throws SQLException
    {
        // _rs = null ;
        return _ps.executeUpdate();
    }

    //----------------------------------------------------------------------------
    protected ResultSet executeQuery() throws SQLException
    {
        _rs = _ps.executeQuery();
        return _rs;
    }

    //----------------------------------------------------------------------------
    // Type of storage for Boolean
    //----------------------------------------------------------------------------
    public void storeBooleanAsInt()
    {
        _iBooleanStorage = STORE_BOOLEAN_AS_INT;
    }

    //----------------------------------------------------------------------------
    public void storeBooleanAsString()
    {
        _iBooleanStorage = STORE_BOOLEAN_AS_STRING;
    }

    //---------------------------------------------------------------------------
    // Close the ResultSet
    // Doesn't throw Exception (just trace the error)
    //---------------------------------------------------------------------------
    private void closeResultSet()
    {
        if (_rs != null)
        {
            try
            {
                _rs.close();
            } catch (SQLException e)
            {
                error("Cannot close the ResultSet ! SQLException : " + e.getMessage());
            }
        }
    }

    //---------------------------------------------------------------------------
    // Close the PreparedStatement
    // Doesn't throw Exception (just trace the error)
    //---------------------------------------------------------------------------
    private void closePreparedStatement()
    {
        if (_ps != null)
        {
            try
            {
                _ps.close();
            } catch (SQLException e)
            {
                error("Cannot close the PreparedStatement ! SQLException : " + e.getMessage());
            }
        }
    }

    //---------------------------------------------------------------------------
    // Close the PreparedStatement and the ResultSet
    // Doesn't throw Exception (just trace the error)
    //---------------------------------------------------------------------------
    protected void close()
    {
        closeResultSet();
        closePreparedStatement();
    }

    //============================================================================
    // METHODES DE VALORISATION DES PARAMETRES DU "PREPARED STATEMENT"
    // Les exceptions sont gres au niveau des mthodes "executeSqlXxxx"
    //============================================================================
    /* package */void setParam(int i, Object oParam) throws TelosysException
    {
        if (_ps == null)
        {
            throw new TelosysException("setParamInt: _ps is null !");
        }
        else
        {
            try
            {
                if (oParam instanceof Boolean)
                {
                    setParamBoolean(i, ((Boolean) oParam).booleanValue());
                }
                //--- Integer Wrappers
                else if (oParam instanceof Integer)
                {
                    setParamInt(i, ((Integer) oParam).intValue());
                }
                else if (oParam instanceof Byte)
                {
                    setParamByte(i, ((Byte) oParam).byteValue());
                }
                else if (oParam instanceof Short)
                {
                    setParamShort(i, ((Short) oParam).shortValue());
                }
                else if (oParam instanceof Long)
                {
                    setParamLong(i, ((Long) oParam).longValue());
                }
                //--- Float/Double Wrappers
                else if (oParam instanceof Float)
                {
                    setParamFloat(i, ((Float) oParam).floatValue());
                }
                else if (oParam instanceof Double)
                {
                    setParamDouble(i, ((Double) oParam).doubleValue());
                }
                //--- Standards Objects
                else if (oParam instanceof String)
                {
                    setParamString(i, (String) oParam);
                }
                else if (oParam instanceof java.util.Date)
                {
                    setParamDate(i, (java.util.Date) oParam);
                }
                //--- Others
                else
                {
                    _ps.setObject(i, oParam);
                }
            } catch (SQLException e)
            {
                throw new TelosysException("setParam", e);
            }
        }
    }

    //-------------------------------------------------------------
    private void checkPreparedStatement() throws TelosysException
    {
        if (_ps == null)
        {
            throw new TelosysException("PreparedStatement '_ps' is null !");
        }
    }

    //-------------------------------------------------------------
    public void setParamInt(int i, int iValue) throws TelosysException
    {
        try
        {
            checkPreparedStatement();
            _ps.setInt(i, iValue);
        } catch (SQLException e)
        {
            throw new TelosysException("setParamInt", e);
        }
    }

    //-------------------------------------------------------------
    public void setParamByte(int i, byte value) throws TelosysException
    {
        try
        {
            checkPreparedStatement();
            _ps.setByte(i, value);
        } catch (SQLException e)
        {
            throw new TelosysException("setParamByte", e);
        }
    }

    //-------------------------------------------------------------
    public void setParamShort(int i, short value) throws TelosysException
    {
        try
        {
            checkPreparedStatement();
            _ps.setShort(i, value);
        } catch (SQLException e)
        {
            throw new TelosysException("setParamShort", e);
        }
    }

    //-------------------------------------------------------------
    public void setParamLong(int i, long value) throws TelosysException
    {
        try
        {
            checkPreparedStatement();
            _ps.setLong(i, value);
        } catch (SQLException e)
        {
            throw new TelosysException("setParamLong", e);
        }
    }

    //-------------------------------------------------------------
    public void setParamFloat(int i, float fValue) throws TelosysException
    {
        try
        {
            checkPreparedStatement();
            _ps.setFloat(i, fValue);
        } catch (SQLException e)
        {
            throw new TelosysException("setParamFloat", e);
        }
    }

    //-------------------------------------------------------------
    public void setParamDouble(int i, double dValue) throws TelosysException
    {
        try
        {
            checkPreparedStatement();
            _ps.setDouble(i, dValue);
        } catch (SQLException e)
        {
            throw new TelosysException("setParamDouble", e);
        }
    }

    //-------------------------------------------------------------
    public void setParamString(int i, String sValue) throws TelosysException
    {
        try
        {
            checkPreparedStatement();
            _ps.setString(i, sValue);
        } catch (SQLException e)
        {
            throw new TelosysException("setParamString", e);
        }
    }

    //-------------------------------------------------------------
    public void setParamDate(int i, java.sql.Date dateValue) throws TelosysException
    {
        try
        {
            checkPreparedStatement();
            _ps.setDate(i, dateValue);
        } catch (SQLException e)
        {
            throw new TelosysException("setParamDate", e);
        }
    }

    //-------------------------------------------------------------
    public void setParamDate(int i, java.util.Date dateValue) throws TelosysException
    {
        try
        {
            checkPreparedStatement();
            if (dateValue != null)
            {
                _ps.setDate(i, new java.sql.Date(dateValue.getTime()));
            }
            else
            {
                _ps.setDate(i, null);
            }
        } catch (SQLException e)
        {
            throw new TelosysException("setParamDate", e);
        }
    }

    //-------------------------------------------------------------
    public void setParamBoolean(int i, boolean bValue) throws TelosysException
    {
        try
        {
            checkPreparedStatement();
            switch (_iBooleanStorage)
            {
            case STORE_BOOLEAN_AS_INT :
                _ps.setInt(i, (bValue ? 1 : 0));
                break;
            case STORE_BOOLEAN_AS_STRING :
                _ps.setString(i, (bValue ? "1" : "0"));
                break;
            default :
                _ps.setBoolean(i, bValue);
                break;
            }
        } catch (SQLException e)
        {
            throw new TelosysException("setParamBoolean", e);
        }
    }

    //============================================================================
    // Mthodes de rcupration des donnes  partir du "RESULT SET"
    //============================================================================
    private void checkResultSet() throws TelosysException
    {
        if (_rs == null)
        {
            throw new TelosysException("ResultSet '_rs' is null !");
        }
    }

    //-------------------------------------------------------------
    public String getResultString(int iPosition) throws TelosysException
    {
        try
        {
            checkResultSet();
            return _rs.getString(iPosition);
        } catch (SQLException e)
        {
            throw new TelosysException("getResultString", e);
        }
    }

    //-------------------------------------------------------------
    public int getResultInt(int iPosition) throws TelosysException
    {
        try
        {
            checkResultSet();
            return _rs.getInt(iPosition);
        } catch (SQLException e)
        {
            throw new TelosysException("getResultInt", e);
        }
    }
    //-------------------------------------------------------------
    public byte getResultByte(int iPosition) throws TelosysException
    {
        try
        {
            checkResultSet();
            return _rs.getByte(iPosition);
        } catch (SQLException e)
        {
            throw new TelosysException("getResultByte", e);
        }
    }
    //-------------------------------------------------------------
    public short getResultShort(int iPosition) throws TelosysException
    {
        try
        {
            checkResultSet();
            return _rs.getShort(iPosition);
        } catch (SQLException e)
        {
            throw new TelosysException("getResultShort", e);
        }
    }
    //-------------------------------------------------------------
    public long getResultLong(int iPosition) throws TelosysException
    {
        try
        {
            checkResultSet();
            return _rs.getLong(iPosition);
        } catch (SQLException e)
        {
            throw new TelosysException("getResultLong", e);
        }
    }

    //-------------------------------------------------------------
    public float getResultFloat(int iPosition) throws TelosysException
    {
        try
        {
            checkResultSet();
            return _rs.getFloat(iPosition);
        } catch (SQLException e)
        {
            throw new TelosysException("getResultFloat", e);
        }
    }
    //-------------------------------------------------------------
    public double getResultDouble(int iPosition) throws TelosysException
    {
        try
        {
            checkResultSet();
            return _rs.getDouble(iPosition);
        } catch (SQLException e)
        {
            throw new TelosysException("getResultDouble", e);
        }
    }

    //-------------------------------------------------------------
    public java.sql.Date getResultDate(int iPosition) throws TelosysException
    {
        try
        {
            checkResultSet();
            return _rs.getDate(iPosition);
        } catch (SQLException e)
        {
            throw new TelosysException("getResultDate", e);
        }
    }

    //-------------------------------------------------------------
    public boolean getResultBoolean(int iPosition) throws TelosysException
    {
        boolean bRet = false;
        try
        {
            checkResultSet();
            switch (_iBooleanStorage)
            {
            case STORE_BOOLEAN_AS_INT :
                bRet = (_rs.getInt(iPosition) != 0 ? true : false);
                break;
            case STORE_BOOLEAN_AS_STRING :
                String s = _rs.getString(iPosition);
                if (s == null)
                {
                    bRet = false;
                }
                else
                {
                    bRet = (s.trim().equals("0") ? true : false);
                }
                break;
            default :
                bRet = _rs.getBoolean(iPosition);
                break;
            }
        } catch (SQLException e)
        {
            throw new TelosysException("getResultBoolean", e);
        }
        return bRet;
    }

    //-------------------------------------------------------------
    protected boolean nextResult() throws TelosysException
    {
        if (_rs == null)
        {
            throw new TelosysException("nextResult: _rs is null !");
        }
        else
        {
            try
            {
                return _rs.next();
            } catch (SQLException e)
            {
                throw new TelosysException("nextResult", e);
            }
        }
    }

    //-------------------------------------------------------------
}