package org.objectweb.telosys.dal.dao;


/**
 * @author Laurent GUERIN
 *  
 */
public class SqlRequests
{
    private String  _table           = null;

    private String  _keyColumns[]    = null;

    private String  _dataColumns[]   = null;

    //--- SQL requests
    private String  _sqlSelect       = null;

    private String  _sqlInsert       = null;

    private String  _sqlUpdate       = null;

    private String  _sqlDelete       = null;

    //--- Pre-built clauses
    private String  _allColumnsList  = null;

    private String  _whereCriteria   = null;

    //--- Join management
    private boolean _bSelectWithJoin = false;

    private String  _sJoinTables     = null;

    private String  _sJoinColumns    = null;

    private String  _sJoinCriteria   = null;

    //============================================================================
    //  CONSTRUCTORS
    //============================================================================
    public SqlRequests(String table, String keyColumns[], String dataColumns[])
    {
        if (table == null || keyColumns == null || dataColumns == null)
        {
            return;
        }

        //--- Init the input parameters
        _table = table.trim();
        _keyColumns = (String[]) keyColumns.clone(); // clone to avoid exposing mutable object
        _dataColumns = (String[]) dataColumns.clone(); // clone to avoid exposing mutable object

        //--- Build the 4 SQL Request
        _sqlSelect = buildSqlSelect();
        _sqlInsert = buildSqlInsert();
        _sqlUpdate = buildSqlUpdate();
        _sqlDelete = buildSqlDelete();

        //--- Pre-built clauses parts
        _allColumnsList = allColumns(true);
        _whereCriteria = whereCriteria(true);
    }

    //============================================================================
    //  PRIVATE METHODS
    //============================================================================

    /**
     * Build a list of coma separated colums (for ALL the columns) ie : "tab.col1, tab.col2, tab.col3"
     * 
     * @return
     */
    private String allColumns(boolean bPrefix)
    {
        StringBuffer sb = new StringBuffer(200);
        int iNbCol = 0;

        //--- Data Columns
        for ( int c = 0 ; c < _dataColumns.length ; c++ )
        {
            if (iNbCol > 0)
            {
                sb.append(", ");
            }
            if (bPrefix)
            {
                sb.append(_table + ".");
            }
            sb.append(_dataColumns[c]);
            iNbCol++;
        }

        //--- Key Columns
        for ( int c = 0 ; c < _keyColumns.length ; c++ )
        {
            if (iNbCol > 0)
            {
                sb.append(", ");
            }
            if (bPrefix)
            {
                sb.append(_table + ".");
            }
            sb.append(_keyColumns[c]);
            iNbCol++;
        }
        return sb.toString();
    }

    /**
     * Build a list of coma separated "?" ( for ALL the columns ) ie : "?, ?, ?"
     * 
     * @return
     */
    private String values()
    {
        StringBuffer sb = new StringBuffer(200);
        int iNbCol = 0;

        //--- Data Columns
        for ( int c = 0 ; c < _dataColumns.length ; c++ )
        {
            if (iNbCol > 0)
            {
                sb.append(", ");
            }
            sb.append("?");
            iNbCol++;
        }

        //--- Key Columns
        for ( int c = 0 ; c < _keyColumns.length ; c++ )
        {
            if (iNbCol > 0)
            {
                sb.append(", ");
            }
            sb.append("?");
            iNbCol++;
        }
        return sb.toString();
    }

    /**
     * Build the where criteria ( for KEY columns only) <br>
     * ie : "tab.col1 = ? and tab.col2 = ?"
     * 
     * @return
     */
    private String whereCriteria(boolean bPrefix)
    {
        StringBuffer sb = new StringBuffer(200);
        //--- Key Columns
        for ( int c = 0 ; c < _keyColumns.length ; c++ )
        {
            if (c > 0)
            {
                sb.append(" and ");
            }
            if (bPrefix)
            {
                sb.append(_table + ".");
            }
            sb.append(_keyColumns[c] + " = ?");
        }
        return sb.toString();
    }

    /**
     * Build the set column value clause (for DATA columns only) ie : "tab.col1 = ?, tab.col2 = ?"
     * 
     * @return
     */
    private String setDataValues(boolean bPrefix)
    {
        StringBuffer sb = new StringBuffer(200);
        //--- Data Columns
        for ( int c = 0 ; c < _dataColumns.length ; c++ )
        {
            if (c > 0)
            {
                sb.append(", ");
            }
            if (bPrefix)
            {
                sb.append(_table + ".");
            }
            sb.append(_dataColumns[c] + " = ?");
        }
        return sb.toString();
    }

    /**
     * Build the SQL SELECT request
     * 
     * @return
     */
    private String buildSqlSelect()
    {
        //return "select " + allColumns(true) + " from " + _table + " where " + whereCriteria(true);
        return "select " + allColumns(false) + " from " + _table + " where " + whereCriteria(false);
    }

    /**
     * Build the SQL INSERT request
     * 
     * @return
     */
    private String buildSqlInsert()
    {
        return "insert into " + _table + " ( " + allColumns(false) + " ) values ( " + values() + " )";
    }

    /**
     * Build the SQL UPDATE request
     * 
     * @return
     */
    private String buildSqlUpdate()
    {
        return "update " + _table + " set " + setDataValues(false) + " where " + whereCriteria(false);
    }

    /**
     * Build the SQL DELETE request
     * 
     * @return
     */
    private String buildSqlDelete()
    {
        return "delete from " + _table + " where " + whereCriteria(false);
    }

    //============================================================================
    //============================================================================
    public ListQuery createQuery(int iKeySize, String sQueryCriteria, String sQueryEndOfSelect) //throws TelosysException
    {
        if (_bSelectWithJoin)
        {
            //--- The SELECT request has a JOIN => build the query with the JOIN
            return createQueryWithJoin(iKeySize, sQueryCriteria, sQueryEndOfSelect);
        }
        else
        {
            //--- No JOIN in the SELECT => build a normal query
            return createQueryWithoutJoin(iKeySize, sQueryCriteria, sQueryEndOfSelect);
        }
    }

    private ListQuery createQueryWithoutJoin(int iKeySize, String sQueryCriteria, String sQueryEndOfSelect)
    {
        String sWhere = SqlBuilder.buildSqlWhere(sQueryCriteria);
        String sEnd = SqlBuilder.buildSqlComplement(sQueryEndOfSelect);

        String sSelect = "select " + allColumns(true) + " from " + _table + sWhere + sEnd;
        String sCount = "select count(*) from " + _table + sWhere;
        String sDelete = "delete from " + _table + sWhere;
        return new StandardQuery(iKeySize, sSelect, sCount, sDelete);
    }

    private ListQuery createQueryWithJoin(int iKeySize, String sQueryCriteria, String sQueryEndOfSelect)
    {
        //--- Build the SQL fragments
        String sColumns = SqlBuilder.buildSqlColumns(allColumns(true), _sJoinColumns);
        String sTables = SqlBuilder.buildSqlTables(_table, _sJoinTables);
        String sWhere = SqlBuilder.buildSqlWhere(_sJoinCriteria, sQueryCriteria);
        String sEnd = SqlBuilder.buildSqlComplement(sQueryEndOfSelect);

        String sSelect = "select " + sColumns + " from " + sTables + sWhere + sEnd;
        String sCount = "select count(*) from " + sTables + sWhere;
        String sDelete = "delete from " + _table + SqlBuilder.buildSqlWhere(sQueryCriteria);
        return new StandardQuery(iKeySize, sSelect, sCount, sDelete);
    }

    /**
     * Change the SQL SELECT by adding a JOIN
     * 
     * @param sJoinTables
     * @param sJoinColumns
     * @param sJoinCriteria
     */
    public void alterSqlSelectWithJoin(String sJoinTables, String sJoinColumns, String sJoinCriteria)
    {
        //--- Build the SQL fragments
        String sColumns = SqlBuilder.buildSqlColumns(allColumns(true), sJoinColumns);
        String sTables = SqlBuilder.buildSqlTables(_table, sJoinTables);
        String sWhere = SqlBuilder.buildSqlWhere(sJoinCriteria, whereCriteria(true));

        //--- Set the new SELECT request
        _sqlSelect = "select " + sColumns + " from " + sTables + sWhere;

        //--- Keep the Join characteristics
        _bSelectWithJoin = true;
        _sJoinTables = sJoinTables;
        _sJoinColumns = sJoinColumns;
        _sJoinCriteria = sJoinCriteria;
    }

    //============================================================================
    //  GETTERS
    //============================================================================

    /**
     * Return the SQL SELECT request
     * 
     * @return
     */
    public String getSqlSelect()
    {
        return _sqlSelect;
    }

    /**
     * Return the SQL INSERT request
     * 
     * @return
     */
    public String getSqlInsert()
    {
        return _sqlInsert;
    }

    /**
     * Return the SQL UPDATE request
     * 
     * @return
     */
    public String getSqlUpdate()
    {
        return _sqlUpdate;
    }

    /**
     * Return the SQL DELETE request
     * 
     * @return
     */
    public String getSqlDelete()
    {
        return _sqlDelete;
    }

    /**
     * Return the TABLE NAME
     * 
     * @return
     */
    public String getTableName()
    {
        return _table;
    }

    /**
     * Return the list of all the columns (coma separated)
     * 
     * @return
     */
    public String getAllColumnsList()
    {
        return _allColumnsList;
    }

    /**
     * Return the where criteria of the requests
     * 
     * @return
     */
    public String getWhereCriteria()
    {
        return _whereCriteria;
    }

}