package org.objectweb.telosys.dal.dao;

import java.sql.Connection;
import java.sql.SQLException;
import java.util.List;

import org.objectweb.telosys.common.TelosysException;
import org.objectweb.telosys.common.TelosysObject;
import org.objectweb.telosys.dal.dao.ListQuery;
import org.objectweb.telosys.dal.sql.ConnectionManager;

/**
 * @author Laurent GUERIN
 *  
 */
public abstract class StandardDAO extends TelosysObject
{
    //============================================================================
    // Constants
    //============================================================================

    private static final int     ACTION_SQL_INSERT       = 1;

    private static final int     ACTION_SQL_UPDATE       = 2;

    private static final int     ACTION_SQL_DELETE       = 3;

    private static final boolean COMMIT                  = true;

    private static final boolean DO_NOT_COMMIT           = false;

    protected static final int   STORE_BOOLEAN_DEFAULT   = 0;

    protected static final int   STORE_BOOLEAN_AS_INT    = 1;

    protected static final int   STORE_BOOLEAN_AS_STRING = 2;

    //============================================================================
    // Instance Attributes
    //============================================================================

    private int                  _iBooleanStorage        = STORE_BOOLEAN_DEFAULT;

    private Class                _voClass                = null;

    private SqlRequests          _sqlRequests            = null;

    //============================================================================
    // Abstract methods (implemented in each DAO)
    //============================================================================

    protected abstract void setPrimaryKey(QueryContext context, Object entity) throws TelosysException;

    protected abstract int tableToBean(QueryContext context, Object entity) throws TelosysException;

    protected abstract int beanToTable(QueryContext context, Object entity) throws TelosysException;

    //============================================================================
    // CONSTRUCTOR
    //============================================================================

    public StandardDAO(Class voClass, SqlRequests sqlRequests)
    {
        super();
        _voClass = voClass;
        _sqlRequests = sqlRequests;
    }

    /**
     * Return the class of the Value Object managed by the current DAO
     * 
     * @return
     */
    public Class getEntityBeanClass()
    {
        return _voClass;
    }

    public SqlRequests getSqlRequests()
    {
        return _sqlRequests;
    }

    //----------------------------------------------------------------------------
    // Type of storage for Boolean
    //----------------------------------------------------------------------------
    /**
     * Set the boolean storage type to "int"
     */
    public void storeBooleanAsInt()
    {
        _iBooleanStorage = STORE_BOOLEAN_AS_INT;
    }

    //----------------------------------------------------------------------------
    /**
     * Set the boolean storage type to "String"
     */
    public void storeBooleanAsString()
    {
        _iBooleanStorage = STORE_BOOLEAN_AS_STRING;
    }

    //----------------------------------------------------------------------------
    /**
     * Return the boolean storage type
     * 
     * @return
     */
    public int getBooleanStorage()
    {
        return _iBooleanStorage;
    }

    //----------------------------------------------------------------------------
    /**
     * Create a query on a single table
     * 
     * @param iKeySize
     * @param sQueryCriteria
     * @param sQueryEndOfSelect
     * @return
     * @throws TelosysException
     */
    public ListQuery createQuery(int iKeySize, String sQueryCriteria, String sQueryEndOfSelect) throws TelosysException
    {
        return _sqlRequests.createQuery(iKeySize, sQueryCriteria, sQueryEndOfSelect);
    }

    //---------------------------------------------------------------------------
    /**
     * Change the SQL SELECT by adding a JOIN
     * 
     * @param sJoinTables
     * @param sJoinColumns
     * @param sJoinCriteria
     */
    protected void alterSqlSelectWithJoin(String sJoinTables, String sJoinColumns, String sJoinCriteria)
    {
        _sqlRequests.alterSqlSelectWithJoin(sJoinTables, sJoinColumns, sJoinCriteria);
    }

    protected void alterEntityBeanClass(Class newClass)
    {
        _voClass = newClass;
    }

    //---------------------------------------------------------------------------
    private Object createEntityBean() throws TelosysException
    {
        if (_voClass != null)
        {
            try
            {
                return _voClass.newInstance();
            } catch (Exception e)
            {
                String msg = "createEntityBean() : class = " + _voClass.getName() + " : Exception = " + e.getMessage();
                throw new TelosysException(msg, e);
            }
        }
        return null;
    }

    //---------------------------------------------------------------------------
    private Connection getConnection(int iDatabaseId) throws TelosysException
    {
        //--- Rcuprer une connexion dans le pool
        trace("getConnection(" + iDatabaseId + ")...");
        //--- Get a connection from the connection provider
        Connection dbCon = null;
        try
        {
            dbCon = ConnectionManager.getConnection(iDatabaseId);
        } catch (Exception e)
        {
            throw new TelosysException("EntityDAO.getConnection(" + iDatabaseId + ")", e);
        }
        return dbCon;
    }

    //---------------------------------------------------------------------------
    // Libration d'une connexion
    // Doesn't throw Exception (just trace the error)
    //---------------------------------------------------------------------------
    private void releaseConnection(Connection con)
    {
        trace("releaseConnection()...");
        if (con != null)
        {
            try
            {
                con.close();
            } catch (SQLException e)
            {
                error("Cannot close the connection ! SQLException : " + e.getMessage());
            }
        }
    }

    //---------------------------------------------------------------------------
    // Gestion du mode "AutoCommit" d'une connexion
    //---------------------------------------------------------------------------
    private boolean getAutoCommit(Connection con)
    {
        try
        {
            return con.getAutoCommit();
        } catch (SQLException ex)
        {
            error("Cannot get the AutoCommit mode : getAutoCommit() : Exception " + ex.toString());
            return false;
        }
    }

    //---------------------------------------------------------------------------
    private void setAutoCommit(Connection con, boolean bMode)
    {
        try
        {
            con.setAutoCommit(bMode);
        } catch (SQLException ex)
        {
            error("Cannot set AutoCommit mode : setAutoCommit() : Exception " + ex.toString());
        }
    }

    //---------------------------------------------------------------------------
    private void rollback(Connection con)
    {
        try
        {
            con.rollback();
        } catch (SQLException ex)
        {
            error("Cannot rollback : rollback() : Exception " + ex.toString());
        }
    }

    //============================================================================
    //                             ENTITY DAO
    //============================================================================

    //============================================================================
    // Abstract methods calls
    //============================================================================
    private void callSetPrimaryKey(QueryContext context, Object entity) throws TelosysException
    {
        if (context == null)
        {
            throw new TelosysException("DAO call setPrimaryKey : Query Context is null ");
        }
        if (entity == null)
        {
            throw new TelosysException("DAO call setPrimaryKey : Entity Object is null ");
        }
        setPrimaryKey(context, entity);
    }

    private void callTableToBean(QueryContext context, Object entity) throws TelosysException
    {
        if (context == null)
        {
            throw new TelosysException("DAO call tableToBean : Query Context is null ");
        }
        if (entity == null)
        {
            throw new TelosysException("DAO call tableToBean : Entity Object is null ");
        }
        tableToBean(context, entity);
    }

    private void callBeanToTable(QueryContext context, Object entity) throws TelosysException
    {
        if (context == null)
        {
            throw new TelosysException("DAO call beanToTable : Query Context is null ");
        }
        if (entity == null)
        {
            throw new TelosysException("DAO call beanToTable : Entity Object is null ");
        }
        beanToTable(context, entity);
    }

    //============================================================================
    //                             S E L E C T ( JDBC / SQL )
    // Code Retour :
    // . 1 : Trouv et charg
    // . 0 : Non trouv
    //============================================================================
    //---------------------------------------------------------------------------
    private int executeSqlSelect(String sSql, Object bean, int iDatabaseId) throws TelosysException
    {
        int iRet = 0;
        Connection con = getConnection(iDatabaseId);
        try
        {
            iRet = executeSqlSelect(sSql, bean, con);
        }
        // Don't catch exceptions (just force "releaseConnection" in "finally")
        finally
        {
            releaseConnection(con);
        }
        return iRet;
    }

    //---------------------------------------------------------------------------
    private int executeSqlSelect(String sSql, Object bean, Connection con) throws TelosysException
    {
        int iReturnCode = 0;
        QueryContext queryContext = null;

        if (con == null)
        {
            throw new TelosysException("executeSqlSelect(" + sSql + "): DB Connection is null");
        }
        try
        {
            //--- Cration du contexte de la requte SQL
            trace("executeSqlSelect: create QueryContext...");
            queryContext = new QueryContext(con, sSql, getBooleanStorage());

            //--- Valoriser dynamiquement les paramtres de la requte SQL
            callSetPrimaryKey(queryContext, bean); //-- in sub-class

            //--- Excuter la requte SQL
            trace("executeSqlSelect: executeQuery()...");
            queryContext.executeQuery();

            //--- Une seule ligne  traiter ...
            trace("executeSqlSelect: rs.next()...");
            if (!queryContext.nextResult())
            {
                //--- Fin normale, mais non trouv
                iReturnCode = 0;
                trace("executeSqlSelect: NOT FOUND ");
            }
            else
            {
                //--- Fin normale (trouv et charg)
                callTableToBean(queryContext, bean);
                iReturnCode = 1;
                trace("executeSqlSelect: FOUND ");
            }
        } catch (SQLException ex)
        {
            throw new TelosysException("EntityDAO.executeSqlSelect(" + sSql + ")", ex);
        } catch (NumberFormatException ex)
        {
            throw new TelosysException("EntityDAO.executeSqlSelect(" + sSql + ")", ex);
        } catch (Exception ex)
        {
            throw new TelosysException("EntityDAO.executeSqlSelect(" + sSql + ")", ex);
        } finally
        {
            trace("executeSqlSelect(): finally.");
            if (queryContext != null)
            {
                queryContext.close();
            }
        }
        return iReturnCode;
    }

    //============================================================================
    //  SQL for SAVE operation
    //============================================================================
    private int executeSqlSaveList(ListQuery query, List listData, boolean bCommitMode, int iDatabaseId) throws TelosysException
    {
        int iRet = 0;
        Connection con = getConnection(iDatabaseId);
        try
        {
            iRet = executeSqlSaveList(query, listData, bCommitMode, con);
        }
        // Don't catch exceptions (just force "releaseConnection" in "finally")
        finally
        {
            releaseConnection(con);
        }
        return iRet;
    }

    //============================================================================
    //   M I S E A J O U R : INSERT, UPDATE, DELETE
    // Code Retour :
    // Nombre de lignes insres, modifies ou supprimes ( 0  N )
    //============================================================================
    private int executeSql(String sSql, int iAction, Object bean, boolean bCommitMode, int iDatabaseId) throws TelosysException
    {
        int iRet = 0;
        Connection con = getConnection(iDatabaseId);
        try
        {
            iRet = executeSql(sSql, iAction, bean, bCommitMode, con);
        }
        // Don't catch exceptions (just force "releaseConnection" in "finally")
        finally
        {
            releaseConnection(con);
        }
        return iRet;
    }

    //---------------------------------------------------------------------------
    private int executeSql(String sSql, int iAction, Object bean, boolean bCommitMode, Connection con) throws TelosysException
    {
        boolean bMemoAutoCommitMode = false;
        boolean bError = false;
        QueryContext queryContext = null;
        String sAction = "???";
        switch (iAction)
        {
        case ACTION_SQL_INSERT :
            sAction = "Insert";
            break;
        case ACTION_SQL_UPDATE :
            sAction = "Update";
            break;
        case ACTION_SQL_DELETE :
            sAction = "Delete";
            break;
        }
        int iReturnCode = 0;

        if (con == null)
        {
            throw new TelosysException("EntityDAO.executeSql(" + sSql + "): DB Connection is null");
        }

        if (bCommitMode)
        {
            //--- Store the original AutoCommit mode
            bMemoAutoCommitMode = getAutoCommit(con);
        }

        try
        {
            if (bCommitMode)
            {
                //--- Programmatic Transaction control
                con.setAutoCommit(false);
            }
            //--- Cration de la requte SQL
            trace("executeSql(" + sAction + "): create QueryContext...");
            queryContext = new QueryContext(con, sSql, getBooleanStorage());

            //--- Valorisation des paramtres de la requte SQL
            switch (iAction)
            {
            case ACTION_SQL_INSERT :
                trace("executeSql(" + sAction + "): beanToTable()...");
                callBeanToTable(queryContext, bean); //--- in sub-class
                break;
            case ACTION_SQL_UPDATE :
                trace("executeSql(" + sAction + "): beanToTable()...");
                callBeanToTable(queryContext, bean); //--- in sub-class
                break;
            case ACTION_SQL_DELETE :
                trace("executeSql(" + sAction + "): setPrimaryKey()...");
                callSetPrimaryKey(queryContext, bean); //--- in sub-class
                break;
            default :
                error("executeSql(" + sAction + "): Unknown ACTION !");
                break;
            }
            //--- Excution de la requte SQL
            trace("executeSql(" + sAction + "): executeUpdate()...");
            iReturnCode = queryContext.executeUpdate();

            if (bCommitMode)
            {
                //--- COMMIT the operation
                trace("executeSql(" + sAction + "): con.commit()...");
                con.commit();
            }
        } catch (SQLException ex)
        {
            bError = true;
            throw new TelosysException("EntityDAO.executeSql(" + sSql + ") : ", ex);
        } catch (Exception ex)
        {
            bError = true;
            throw new TelosysException("EntityDAO.executeSql(" + sSql + ") : ", ex);
        } finally
        {
            trace("executeSql(" + sAction + "): finally.");
            if (bError)
            {
                if (bCommitMode)
                {
                    //--- CANCEL the operation
                    trace("executeSql(" + sAction + "): rollback()...");
                    rollback(con);
                }
            }
            if (queryContext != null)
            {
                queryContext.close();
            }
            if (bCommitMode)
            {
                //--- Restore the original AutoCommit mode
                setAutoCommit(con, bMemoAutoCommitMode);
            }
        }
        return iReturnCode;
    }

    //============================================================================
    // S A V E : DELETE + INSERT
    // Code Retour :
    // Nombre de lignes insres, modifies ou supprimes ( 0  N )
    //============================================================================
    private int executeSqlSave(String sSqlDelete, String sSqlInsert, Object bean, boolean bCommitMode, int iDatabaseId) throws TelosysException
    {
        int iRet = 0;
        Connection con = getConnection(iDatabaseId);
        try
        {
            iRet = executeSqlSave(sSqlDelete, sSqlInsert, bean, bCommitMode, con);
        }
        // Don't catch exceptions (just force "releaseConnection" in "finally")
        finally
        {
            releaseConnection(con);
        }
        return iRet;
    }

    //---------------------------------------------------------------------------
    private int executeSqlSave(String sSqlDelete, String sSqlInsert, Object bean, boolean bCommitMode, Connection con) throws TelosysException
    {
        boolean bMemoAutoCommitMode = false;
        boolean bError = false;
        QueryContext queryContextDelete = null;
        QueryContext queryContextInsert = null;
        int iReturnCode = 0;

        if (con == null)
        {
            throw new TelosysException("executeSqlSave(): DB Connection is null");
        }

        if (bCommitMode)
        {
            //--- Store the original AutoCommit mode
            bMemoAutoCommitMode = getAutoCommit(con);
        }

        try
        {
            if (bCommitMode)
            {
                //--- Programmatic Transaction control
                con.setAutoCommit(false);
            }

            //--- SQL DELETE
            trace("executeSqlSave() : DELETE - create QueryContext...");
            queryContextDelete = new QueryContext(con, sSqlDelete, getBooleanStorage());
            callSetPrimaryKey(queryContextDelete, bean); //--- in sub-class
            trace("executeSqlSave() : DELETE - executeUpdate()...");
            iReturnCode = queryContextDelete.executeUpdate();

            //--- SQL INSERT
            trace("executeSqlSave() : INSERT - create QueryContext...");
            queryContextInsert = new QueryContext(con, sSqlInsert, getBooleanStorage());
            callBeanToTable(queryContextInsert, bean); //--- in sub-class
            trace("executeSqlSave() : INSERT - executeUpdate()...");
            iReturnCode = queryContextInsert.executeUpdate();

            if (bCommitMode)
            {
                //--- COMMIT the operation
                trace("executeSqlSave(): con.commit()...");
                con.commit();
            }
        } catch (SQLException ex)
        {
            bError = true;
            throw new TelosysException("EntityDAO.executeSqlSave() : ", ex);
        } catch (Exception ex)
        {
            bError = true;
            throw new TelosysException("EntityDAO.executeSqlSave() : ", ex);
        } finally
        {
            trace("executeSqlSave(): finally.");
            if (bError)
            {
                if (bCommitMode)
                {
                    //--- CANCEL the operation
                    trace("executeSqlSave(): rollback()...");
                    rollback(con);
                }
            }
            if (queryContextDelete != null)
            {
                queryContextDelete.close();
            }
            if (queryContextInsert != null)
            {
                queryContextInsert.close();
            }
            if (bCommitMode)
            {
                //--- Restore the original AutoCommit mode
                setAutoCommit(con, bMemoAutoCommitMode);
            }
        }
        return iReturnCode;
    }

    //---------------------------------------------------------------------------
    private void checkParams(String sMethode, Object bean, int iBaseId) throws TelosysException
    {
        if (bean == null)
        {
            throw new TelosysException("DAO " + sMethode + " : bean is null ");
        }
        if (iBaseId < 0)
        {
            throw new TelosysException("DAO " + sMethode + " : Database ID < 0  (" + iBaseId + ")");
        }
    }

    //---------------------------------------------------------------------------
    private void checkParams(String sMethode, Object bean, DatabaseSession session) throws TelosysException
    {
        if (bean == null)
        {
            throw new TelosysException("DAO " + sMethode + " : bean is null ");
        }
        if (session == null)
        {
            throw new TelosysException("DAO " + sMethode + " : session is null ");
        }
        else
        {
            if (session.getConnection() == null)
            {
                throw new TelosysException("DAO " + sMethode + " : connection in session is null ");
            }
        }
    }

    //---------------------------------------------------------------------------
    private void checkParams(String sMethode, Object bean, Connection con) throws TelosysException
    {
        if (bean == null)
        {
            throw new TelosysException("DAO " + sMethode + " : bean is null ");
        }
        if (con == null)
        {
            throw new TelosysException("DAO " + sMethode + " : connection is null ");
        }
    }

    //---------------------------------------------------------------------------

    //============================================================================
    //     L O A D
    //============================================================================
    protected int doLoad(Object bean) throws TelosysException
    {
        return doLoad(bean, ConnectionManager.getDefaultDatabase());
    }

    //---------------------------------------------------------------------------
    protected int doLoad(Object bean, int iDatabaseId) throws TelosysException
    {
        checkParams("load", bean, iDatabaseId);
        return executeSqlSelect(_sqlRequests.getSqlSelect(), bean, iDatabaseId);
    }

    //---------------------------------------------------------------------------
    protected int doLoad(Object bean, DatabaseSession session) throws TelosysException
    {
        checkParams("load", bean, session);
        return executeSqlSelect(_sqlRequests.getSqlSelect(), bean, session.getConnection());
    }

    //---------------------------------------------------------------------------
    protected int doLoad(Object bean, Connection con) throws TelosysException
    {
        checkParams("load", bean, con);
        return executeSqlSelect(_sqlRequests.getSqlSelect(), bean, con);
    }

    //---------------------------------------------------------------------------

    //============================================================================
    //     S A V E ( DELETE + INSERT )
    //============================================================================
    protected int doSave(Object bean) throws TelosysException
    {
        return doSave(bean, ConnectionManager.getDefaultDatabase());
    }

    //---------------------------------------------------------------------------
    protected int doSave(Object bean, int iDatabaseId) throws TelosysException
    {
        checkParams("save", bean, iDatabaseId);
        return executeSqlSave(_sqlRequests.getSqlDelete(), _sqlRequests.getSqlInsert(), bean, COMMIT, iDatabaseId);
    }

    //---------------------------------------------------------------------------
    protected int doSave(Object bean, DatabaseSession session) throws TelosysException
    {
        checkParams("save", bean, session);
        return executeSqlSave(_sqlRequests.getSqlDelete(), _sqlRequests.getSqlInsert(), bean, DO_NOT_COMMIT, session.getConnection());
    }

    //---------------------------------------------------------------------------
    protected int doSave(Object bean, Connection con) throws TelosysException
    {
        checkParams("save", bean, con);
        return executeSqlSave(_sqlRequests.getSqlDelete(), _sqlRequests.getSqlInsert(), bean, DO_NOT_COMMIT, con);
    }

    //---------------------------------------------------------------------------

    //============================================================================
    //     I N S E R T
    //============================================================================
    protected int doInsert(Object bean) throws TelosysException
    {
        return doInsert(bean, ConnectionManager.getDefaultDatabase());
    }

    //---------------------------------------------------------------------------
    protected int doInsert(Object bean, int iDatabaseId) throws TelosysException
    {
        checkParams("insert", bean, iDatabaseId);
        return executeSql(_sqlRequests.getSqlInsert(), ACTION_SQL_INSERT, bean, COMMIT, iDatabaseId);
    }

    //---------------------------------------------------------------------------
    protected int doInsert(Object bean, DatabaseSession session) throws TelosysException
    {
        checkParams("insert", bean, session);
        return executeSql(_sqlRequests.getSqlInsert(), ACTION_SQL_INSERT, bean, DO_NOT_COMMIT, session.getConnection());
    }

    //---------------------------------------------------------------------------
    protected int doInsert(Object bean, Connection con) throws TelosysException
    {
        checkParams("insert", bean, con);
        return executeSql(_sqlRequests.getSqlInsert(), ACTION_SQL_INSERT, bean, DO_NOT_COMMIT, con);
    }

    //---------------------------------------------------------------------------

    //============================================================================
    //     U P D A T E
    //============================================================================
    protected int doUpdate(Object bean) throws TelosysException
    {
        return doUpdate(bean, ConnectionManager.getDefaultDatabase());
    }

    //---------------------------------------------------------------------------
    protected int doUpdate(Object bean, int iDatabaseId) throws TelosysException
    {
        checkParams("update", bean, iDatabaseId);
        return executeSql(_sqlRequests.getSqlUpdate(), ACTION_SQL_UPDATE, bean, COMMIT, iDatabaseId);
    }

    //---------------------------------------------------------------------------
    protected int doUpdate(Object bean, DatabaseSession session) throws TelosysException
    {
        checkParams("update", bean, session);
        return executeSql(_sqlRequests.getSqlUpdate(), ACTION_SQL_UPDATE, bean, DO_NOT_COMMIT, session.getConnection());
    }

    //---------------------------------------------------------------------------
    protected int doUpdate(Object bean, Connection con) throws TelosysException
    {
        checkParams("update", bean, con);
        return executeSql(_sqlRequests.getSqlUpdate(), ACTION_SQL_UPDATE, bean, DO_NOT_COMMIT, con);
    }

    //---------------------------------------------------------------------------

    //============================================================================
    //     D E L E T E
    //============================================================================
    protected int doDelete(Object bean) throws TelosysException
    {
        return doDelete(bean, ConnectionManager.getDefaultDatabase());
    }

    //---------------------------------------------------------------------------
    protected int doDelete(Object bean, int iDatabaseId) throws TelosysException
    {
        checkParams("delete", bean, iDatabaseId);
        return executeSql(_sqlRequests.getSqlDelete(), ACTION_SQL_DELETE, bean, COMMIT, iDatabaseId);
    }

    //---------------------------------------------------------------------------
    protected int doDelete(Object bean, DatabaseSession session) throws TelosysException
    {
        checkParams("delete", bean, session);
        return executeSql(_sqlRequests.getSqlDelete(), ACTION_SQL_DELETE, bean, DO_NOT_COMMIT, session.getConnection());
    }

    //---------------------------------------------------------------------------
    protected int doDelete(Object bean, Connection con) throws TelosysException
    {
        checkParams("delete", bean, con);
        return executeSql(_sqlRequests.getSqlDelete(), ACTION_SQL_DELETE, bean, DO_NOT_COMMIT, con);
    }

    //---------------------------------------------------------------------------

    //============================================================================
    //                              LIST DAO
    //============================================================================

    //============================================================================
    // KEY PARAMETERS ( FOR SQL WHERE CLAUSE )
    //============================================================================
    private void setKeyParameters(QueryContext context, Object[] key) throws TelosysException
    {
        if (key != null)
        {
            Object oKeyElement = null;
            int iNbKeyElements = key.length;
            int i = 0;
            int j = 0;
            for ( i = 0 ; i < iNbKeyElements ; i++ ) // indice : 0 to N-1
            {
                oKeyElement = key[i];
                j = i + 1; // PreparedStatement : 1 to N
                context.setParam(j, oKeyElement);
            }
        }
    }

    //============================================================================
    // LOAD : Chargement de la liste
    //============================================================================
    private int executeSqlSelectList(ListQuery query, List listData, int iDatabaseId) throws TelosysException
    {
        int iCount = 0;
        Connection con = getConnection(iDatabaseId);
        try
        {
            iCount = executeSqlSelectList(query, listData, con);
        }
        // Don't catch exceptions (just force "releaseConnection" in "finally")
        finally
        {
            releaseConnection(con);
        }
        return iCount;
    }

    //============================================================================
    private int executeSqlSelectList(ListQuery query, List listData, Connection con) throws TelosysException
    {
        int iCount = 0;
        String sSql = query.getSqlSelect();
        QueryContext queryContext = null;
        Object entityBean = null;

        if (con == null)
        {
            throw new TelosysException("ListDAO.executeLoad(" + sSql + "): DB Connection is null");
        }
        try
        {
            trace("executeSqlSelect: create QueryContext ...");
            queryContext = new QueryContext(con, sSql, getBooleanStorage());

            //--- Set the SQL request parameters (from the query object)
            setKeyParameters(queryContext, query.getParametersArray());

            //--- Execute the SQL request
            trace("executeSqlSelect: executeQuery()...");
            queryContext.executeQuery();

            //--- 0 .. N elements in the ResultSet
            while (queryContext.nextResult())
            {
                trace("executeSqlSelect: nextResult()...");
                //--- Create a new Entity Bean
                entityBean = this.createEntityBean();
                //--- Populate the Entity Bean with SQL Data
                this.tableToBean(queryContext, entityBean);
                //--- Add the new Entity Bean in the list
                listData.add(entityBean); // Idem addLast() for LinkedList
                iCount++;
            }
        } catch (SQLException ex)
        {
            throw new TelosysException("ListDAO.executeSqlSelect(" + sSql + ")", ex);
        } catch (NumberFormatException ex)
        {
            throw new TelosysException("ListDAO.executeSqlSelect(" + sSql + ")", ex);
        } catch (Exception ex)
        {
            throw new TelosysException("ListDAO.executeSqlSelect(" + sSql + ")", ex);
        } finally
        {
            trace("executeSqlSelect(): finally.");
            if (queryContext != null)
            {
                queryContext.close();
            }
        }
        return iCount;
    }

    //============================================================================
    // COUNT : Select count(*) from ... where
    //============================================================================
    private int executeSqlCount(ListQuery query, int iDatabaseId) throws TelosysException
    {
        int iCount = 0;
        Connection con = getConnection(iDatabaseId);
        try
        {
            iCount = executeSqlCount(query, con);
        }
        // Don't catch exceptions (just force "releaseConnection" in "finally")
        finally
        {
            releaseConnection(con);
        }
        return iCount;
    }

    //============================================================================
    private int executeSqlCount(ListQuery query, Connection con) throws TelosysException
    {
        int iCount = 0;
        String sSql = query.getSqlCount();
        QueryContext queryContext = null;

        if (con == null)
        {
            throw new TelosysException("ListDAO.executeSqlCount(" + sSql + "): DB Connection is null");
        }
        try
        {
            trace("executeSqlCount: create QueryContext ...");
            queryContext = new QueryContext(con, sSql);

            //--- Set the SQL request parameters (from the query object)
            setKeyParameters(queryContext, query.getParametersArray());

            //--- Execute the SQL request
            trace("executeSqlCount: executeQuery()...");
            queryContext.executeQuery();

            //--- Get the first and unique row of the ResultSet
            if (queryContext.nextResult())
            {
                trace("executeSqlCount: nextResult()...");
                iCount = queryContext.getResultInt(1);
            }
            else
            {
                error("executeSqlCount: nextResult() return false (ResultSet is void)");
                iCount = -1;
            }
        } catch (SQLException ex)
        {
            throw new TelosysException("ListDAO.executeSqlCount(" + sSql + ")", ex);
        } catch (NumberFormatException ex)
        {
            throw new TelosysException("ListDAO.executeSqlCount(" + sSql + ")", ex);
        } catch (Exception ex)
        {
            throw new TelosysException("ListDAO.executeSqlCount(" + sSql + ")", ex);
        } finally
        {
            trace("executeSqlCount(): finally.");
            if (queryContext != null)
            {
                queryContext.close();
            }
        }
        if (iCount < 0)
        {
            throw new TelosysException("ListDAO.executeSqlCount(" + sSql + ") : result set is void ( 0 row ) ");
        }
        return iCount;
    }

    //============================================================================
    //                         D E L E T E
    //============================================================================
    private int executeSqlDeleteList(ListQuery query, boolean bCommitMode, int iDatabaseId) throws TelosysException
    {
        int iRet = 0;
        Connection con = getConnection(iDatabaseId);
        try
        {
            iRet = executeSqlDeleteList(query, bCommitMode, con);
        }
        // Don't catch exceptions (just force "releaseConnection" in "finally")
        finally
        {
            releaseConnection(con);
        }
        return iRet;
    }

    //============================================================================
    private int executeSqlDeleteList(ListQuery query, boolean bCommitMode, Connection con) throws TelosysException
    {
        String sSql = query.getSqlDelete();
        boolean bMemoAutoCommitMode = false;
        boolean bError = false;
        int iReturnCode = 0;
        QueryContext queryContext = null;

        if (con == null)
        {
            throw new TelosysException("ListDAO.executeSqlDelete(" + sSql + "): DB Connection is null");
        }

        if (bCommitMode)
        {
            //--- Store the original AutoCommit mode
            bMemoAutoCommitMode = getAutoCommit(con);
        }

        try
        {
            if (bCommitMode)
            {
                //--- Programmatic Transaction control
                con.setAutoCommit(false);
            }

            //--- Prparation de la requte SQL
            trace("executeSqlDelete: create QueryContext ...");
            queryContext = new QueryContext(con, sSql, getBooleanStorage());

            //--- Valorisation des paramtres de la requte SQL
            trace("executeSqlDelete(): setSqlxxxxParameters()...");
            //---------------------------------------------------
            // Mthode de la classe DAO drive
            // Valorisation des paramtres de la clause WHERE
            //---------------------------------------------------
            setKeyParameters(queryContext, query.getParametersArray());

            //--- Excution de la requte SQL
            trace("executeSqlDelete(): executeUpdate()...");
            iReturnCode = queryContext.executeUpdate();

            if (bCommitMode)
            {
                //--- COMMIT the operation
                trace("executeSqlDelete(): con.commit()...");
                con.commit();
            }
        } catch (SQLException ex)
        {
            bError = true;
            //error(ex);
            throw new TelosysException("ListDAO.executeSqlDelete(" + sSql + ") : ", ex);
        } catch (Exception ex)
        {
            bError = true;
            //error(ex);
            throw new TelosysException("ListDAO.executeSqlDelete(" + sSql + ") : ", ex);
        } finally
        {
            trace("executeSqlDelete(): finally.");
            if (bError)
            {
                if (bCommitMode)
                {
                    //--- CANCEL the operation
                    trace("executeSqlDelete(): rollback()...");
                    rollback(con);
                }
            }
            if (queryContext != null)
            {
                queryContext.close();
            }
            if (bCommitMode)
            {
                //--- Restore the original AutoCommit mode
                setAutoCommit(con, bMemoAutoCommitMode);
            }
        }
        return iReturnCode;
    }

    //============================================================================
    // Return : number of lines saved
    private int executeSqlSaveList(ListQuery query, List listData, boolean bCommitMode, Connection con) throws TelosysException
    {
        boolean bMemoAutoCommitMode = false;
        boolean bError = false;
        int iReturnCode = 0;
        QueryContext queryContext = null;

        if (con == null)
        {
            throw new TelosysException("ListDAO.executeSqlSave(): DB Connection is null");
        }

        if (bCommitMode)
        {
            //--- Store the original AutoCommit mode
            bMemoAutoCommitMode = getAutoCommit(con);
        }

        try
        {
            if (bCommitMode)
            {
                //--- Programmatic Transaction control
                con.setAutoCommit(false);
            }
            //================================================================
            //   DELETE LINES
            //================================================================
            //--- Prparation de la requte SQL
            trace("executeSqlDelete: create QueryContext ...");
            queryContext = new QueryContext(con, query.getSqlDelete(), getBooleanStorage());
            //--- Valorisation des paramtres de la requte SQL
            trace("executeSqlSave [Delete] : setSqlxxxxParameters()...");
            //---------------------------------------------------
            setKeyParameters(queryContext, query.getParametersArray());

            //---------------------------------------------------
            //--- Excution de la requte SQL
            trace("executeSqlSave [Delete] : executeUpdate()...");
            queryContext.executeUpdate();

            //================================================================
            //   INSERT LINES
            //================================================================
            Object element = null;
            int i = 0;
            iReturnCode = 0;
            int iSize = listData.size();
            for ( i = 0 ; i < iSize ; i++ )
            {
                element = listData.get(i);
                //--- SQL INSERT (via the Entity DAO)
                this.doInsert(element, con);
                iReturnCode++;
            }
            //================================================================
            if (bCommitMode)
            {
                //--- COMMIT the operation
                trace("executeSqlSave : con.commit()...");
                con.commit();
            }
        } catch (SQLException ex)
        {
            bError = true;
            throw new TelosysException("ListDAO.executeSqlSave() : ", ex);
        } catch (Exception ex)
        {
            bError = true;
            throw new TelosysException("ListDAO.executeSqlSave() : ", ex);
        } finally
        {
            trace("executeSqlSave : finally.");
            if (bError)
            {
                if (bCommitMode)
                {
                    //--- CANCEL the operation
                    trace("executeSqlSave : rollback()...");
                    rollback(con);
                }
            }
            if (queryContext != null)
            {
                queryContext.close();
            }
            if (bCommitMode)
            {
                //--- Restore the original AutoCommit mode
                setAutoCommit(con, bMemoAutoCommitMode);
            }
        }
        return iReturnCode;
    }

    //============================================================================
    //     L O A D
    //============================================================================
    protected int doLoadList(ListQuery query, List listData) throws TelosysException
    {
        return doLoadList(query, listData, ConnectionManager.getDefaultDatabase());
    }

    //----------------------------------------------------------------------------
    protected int doLoadList(ListQuery query, List listData, int iDatabaseId) throws TelosysException
    {
        return executeSqlSelectList(query, listData, iDatabaseId);
    }

    //----------------------------------------------------------------------------
    protected int doLoadList(ListQuery query, List listData, DatabaseSession session) throws TelosysException
    {
        return executeSqlSelectList(query, listData, session.getConnection());
    }

    //----------------------------------------------------------------------------
    protected int doLoadList(ListQuery query, List listData, Connection con) throws TelosysException
    {
        return executeSqlSelectList(query, listData, con);
    }

    //----------------------------------------------------------------------------

    //============================================================================
    //     S A V E
    //============================================================================
    protected int doSaveList(ListQuery query, List listData) throws TelosysException
    {
        return doSaveList(query, listData, ConnectionManager.getDefaultDatabase());
    }

    //----------------------------------------------------------------------------
    protected int doSaveList(ListQuery query, List listData, int iDatabaseId) throws TelosysException
    {
        return executeSqlSaveList(query, listData, COMMIT, iDatabaseId);
    }

    //----------------------------------------------------------------------------
    protected int doSaveList(ListQuery query, List listData, DatabaseSession session) throws TelosysException
    {
        return executeSqlSaveList(query, listData, DO_NOT_COMMIT, session.getConnection());
    }

    //----------------------------------------------------------------------------
    protected int doSaveList(ListQuery query, List listData, Connection con) throws TelosysException
    {
        return executeSqlSaveList(query, listData, DO_NOT_COMMIT, con);
    }

    //----------------------------------------------------------------------------

    //============================================================================
    //     D E L E T E
    //============================================================================
    protected int doDeleteList(ListQuery query) throws TelosysException
    {
        return doDeleteList(query, ConnectionManager.getDefaultDatabase());
    }

    //----------------------------------------------------------------------------
    protected int doDeleteList(ListQuery query, int iDatabaseId) throws TelosysException
    {
        return executeSqlDeleteList(query, COMMIT, iDatabaseId);
    }

    //----------------------------------------------------------------------------
    protected int doDeleteList(ListQuery query, DatabaseSession session) throws TelosysException
    {
        return executeSqlDeleteList(query, DO_NOT_COMMIT, session.getConnection());
    }

    //----------------------------------------------------------------------------
    protected int doDeleteList(ListQuery query, Connection con) throws TelosysException
    {
        return executeSqlDeleteList(query, DO_NOT_COMMIT, con);
    }

    //----------------------------------------------------------------------------

    //============================================================================
    //     C O U N T
    //============================================================================
    protected int doCount(ListQuery query) throws TelosysException
    {
        return doCount(query, ConnectionManager.getDefaultDatabase());
    }

    //----------------------------------------------------------------------------
    protected int doCount(ListQuery query, int iDatabaseId) throws TelosysException
    {
        return executeSqlCount(query, iDatabaseId);
    }

    //----------------------------------------------------------------------------
    protected int doCount(ListQuery query, DatabaseSession session) throws TelosysException
    {
        return executeSqlCount(query, session.getConnection());
    }

    //----------------------------------------------------------------------------
    protected int doCount(ListQuery query, Connection con) throws TelosysException
    {
        return executeSqlCount(query, con);
    }

    //----------------------------------------------------------------------------

    //============================================================================
    //     D E S C R I P T I O N
    //============================================================================
    public String describe()
    {
        return "DAO description : " + "\n . Java class : " + this.getClass().getName() + "\n . SQL table  : " + _sqlRequests.getTableName() // this.getMainTable()
                + "\n . select : " + _sqlRequests.getSqlSelect() + "\n . insert : " + _sqlRequests.getSqlInsert() + "\n . update : " + _sqlRequests.getSqlUpdate() + "\n . delete : " + _sqlRequests.getSqlDelete() + "\n . boolean storage type : " + this._iBooleanStorage;
    }

}