package org.objectweb.telosys.dal.dataset;

import org.objectweb.telosys.common.TelosysObject;

/**
 * @author Laurent GUERIN
 *  
 */
public class SQLCriterias extends TelosysObject
{
    private SQLCriteria[] _criterias = null; // index : from 1 to N

    private int           _iCount    = 0;

    private String        _sOperator = "and";

    /**
     * Build a new criterias collection
     * 
     * @param criterias
     *            array of criterias to be used for dynamic request
     * @param sOperator
     *            where operator to use between each criteria ( "and" / "or" )
     */
    public SQLCriterias(SQLCriteria[] criterias, String sOperator)
    {
        // Duplicate the criterias in a N+1 array
        if (criterias != null)
        {
            _criterias = new SQLCriteria[criterias.length + 1];
            _criterias[0] = null;
            for ( int i = 0 ; i < criterias.length ; i++ )
            {
                _criterias[i + 1] = criterias[i];
            }
            _iCount = criterias.length;
        }
        else
        {
            _criterias = new SQLCriteria[1];
            _criterias[0] = null;
            _iCount = 0;
        }
        _sOperator = sOperator;
    }

    public String getOperator()
    {
        return _sOperator;
    }

    public int getCount()
    {
        return _iCount;
    }

    /**
     * Return the SQL criteria ( ie "thecolumn = ?" )
     * 
     * @param i
     *            criteria index ( from 1 to N )
     * @return
     */
    public String getCriteria(int i)
    {
        if (_criterias != null)
        {
            if (i > 0 && i < _criterias.length)
            {
                return _criterias[i].getCriteria();
            }
        }
        return null;
    }

    /**
     * Return the parameter type expected for this criteria
     * 
     * @param i
     *            criteria index ( from 1 to N )
     * @return
     */
    public int getCriteriaType(int i)
    {
        if (_criterias != null)
        {
            if (i > 0 && i < _criterias.length)
            {
                return _criterias[i].getCriteriaType();
            }
        }
        return 0;
    }

    /**
     * Fix the given criteria as "useful" and set its parameter value
     * 
     * @param i
     *            criteria index ( from 1 to N )
     * @param sParamVal
     *            criteria parameter value
     */
    public void useWithValue(int i, String sParamVal)
    {
        if (_criterias != null)
        {
            if (i > 0 && i < _criterias.length)
            {
                _criterias[i].useWithValue(sParamVal);
            }
        }
    }

    /**
     * Fix the given criteria as "not used"
     * 
     * @param i
     *            criteria index ( from 1 to N )
     */
    public void doNotUse(int i)
    {
        if (_criterias != null)
        {
            if (i > 0 && i < _criterias.length)
            {
                _criterias[i].doNotUse();
            }
        }
    }

    /**
     * Build the SQL where clause dynamicaly with the useful criterias
     * 
     * @return the SQL where clause built
     */
    public String getDynamicWhere()
    {
        int iAddCount = 0;
        StringBuffer sb = new StringBuffer(100);

        for ( int i = 1 ; i < _criterias.length ; i++ )
        {
            SQLCriteria crit = _criterias[i];
            if (crit != null)
            {
                if (crit.isUsed())
                {
                    if (iAddCount > 0)
                    {
                        sb.append(" " + _sOperator + " ");
                    }
                    sb.append(crit.getCriteria());
                    iAddCount++;
                }
            }
        }

        String sRet = "";
        if (iAddCount > 0)
        {
            sRet = sb.toString();
        }
        return sRet;
    }

    /**
     * Return the number of useful parameters
     * 
     * @return
     */
    public int getUsefulParametersCount()
    {
        int iSize = 0;
        for ( int i = 1 ; i < _criterias.length ; i++ )
        {
            SQLCriteria crit = _criterias[i];
            if (crit != null)
            {
                if (crit.isUsed())
                {
                    iSize++;
                }
            }
        }
        return iSize;
    }

    /**
     * Build a int array containing the useful parameters types
     * 
     * @return
     */
    public int[] getDynamicTypes()
    {
        //--- Determine the array size
        int iSize = getUsefulParametersCount();
        //--- Build the String array
        int[] types = new int[iSize];
        int x = 0;
        for ( int i = 1 ; i < _criterias.length ; i++ )
        {
            SQLCriteria crit = _criterias[i];
            if (crit != null)
            {
                if (crit.isUsed())
                {
                    if (x < types.length)
                    {
                        types[x++] = crit.getCriteriaType();
                    }
                }
            }
        }
        return types;
    }

    /**
     * Build a String array containing the useful parameters values
     * 
     * @return
     */
    public String[] getDynamicParameters()
    {
        //--- Determine the array size
        int iSize = getUsefulParametersCount();
        //--- Build the String array
        String[] strParams = new String[iSize];
        int x = 0;
        for ( int i = 1 ; i < _criterias.length ; i++ )
        {
            SQLCriteria crit = _criterias[i];
            if (crit != null)
            {
                if (crit.isUsed())
                {
                    if (x < strParams.length)
                    {
                        strParams[x++] = crit.getParamValue();
                    }
                }
            }
        }
        return strParams;
    }
}