package org.objectweb.telosys.dal.dataset;

import org.objectweb.telosys.common.TelosysException;
import org.objectweb.telosys.common.TelosysObject;
import org.objectweb.telosys.util.Cast;

/**
 * @author Laurent GUERIN
 *  
 */
public class SQLDataSetDefinition extends TelosysObject
{
    private String _sSqlSelect        = null;

    private String _sSqlFrom          = null;

    private String _sSqlWhere         = null;

    private String _sOriginalSqlWhere = null;

    private String _sSqlOption        = "";

    private int[]  _parametersTypes   = null;

    private int    _iMaxRows          = 1000;

    private int    _iLinesPerPage     = 20;  // 20 lines per page (by default)

    private int    _iDatabaseId       = 0;

    /**
     *  
     */
    private void init(String sSqlSelect, String sSqlFrom, String sSqlWhere, String sSqlOption, int[] paramTypes)
    {
        _sSqlSelect = (sSqlSelect != null ? sSqlSelect : "ERR_no_select");
        _sSqlFrom = (sSqlFrom != null ? sSqlFrom : "ERR_no_from");
        _sSqlWhere = (sSqlWhere != null ? sSqlWhere : "");
        _sOriginalSqlWhere = _sSqlWhere;
        _sSqlOption = (sSqlOption != null ? sSqlOption : "");
        if (paramTypes != null)
        {
            //--- Clone to preserve "private"
            _parametersTypes = (int[]) paramTypes.clone();
        }
        else
        {
            _parametersTypes = new int[0];
        }
    }

    //------------------------------------------------------------------------------------------
    // CONSTRUCTORS
    //------------------------------------------------------------------------------------------
    public SQLDataSetDefinition(String sSqlSelect, String sSqlFrom, String sSqlWhere, String sSqlOption)
    {
        init(sSqlSelect, sSqlFrom, sSqlWhere, sSqlOption, null);
    }

    public SQLDataSetDefinition(String sSqlSelect, String sSqlFrom, String sSqlWhere, String sSqlOption, int[] paramTypes)
    {
        init(sSqlSelect, sSqlFrom, sSqlWhere, sSqlOption, paramTypes);
    }

    /**
     * Change the initial SQL where clause using a criterias dynamic where
     * 
     * @param criterias
     */
    public void alterWhere(SQLCriterias criterias)
    {
        if (_sOriginalSqlWhere.trim().toLowerCase().startsWith("where"))
        {
            //--- Default = original static WHERE
            _sSqlWhere = _sOriginalSqlWhere.trim() ;
            //--- Static WHERE clause => add the dynamic WHERE clause if it exists
            String sDynamic = criterias.getDynamicWhere();
            if ( sDynamic != null )
            {
                if ( sDynamic.trim().length() > 0 )
                {
                    _sSqlWhere = _sOriginalSqlWhere.trim() + " and " + sDynamic ;                 
                }
            }
        }
        else
        {
            //--- No static WHERE clause => rebuild all the WHERE clause
            String sDynamic = criterias.getDynamicWhere();
            if ( sDynamic != null )
            {
                if ( sDynamic.trim().length() > 0 )
                {
                    _sSqlWhere = " where " + sDynamic ;
                }
            }
        }
        //--- Set the new parameters types
        _parametersTypes = criterias.getDynamicTypes();
        trace("alterWhere() : " + _sSqlWhere);

        //_criterias = criterias ;
    }

    //------------------------------------------------------------------------------------------
    // SETTERS & GETTERS
    //------------------------------------------------------------------------------------------
    //--- Setters
    public void setMaxRows(int v)
    {
        _iMaxRows = v;
    }

    public void setLinesPerPage(int v)
    {
        if (v <= _iMaxRows)
        {
            _iLinesPerPage = v;
        }
    }

    public void setDatabaseId(int v)
    {
        _iDatabaseId = v;
    }

    //--- Getters

    public String getSqlSelect()
    {
        return _sSqlSelect;
    }

    public String getSqlWhere()
    {
        return _sSqlWhere;
    }

    public String getSqlFrom()
    {
        return _sSqlFrom;
    }

    public String getSqlOption()
    {
        return _sSqlOption;
    }

    /**
     * Return the number of parameters expected to execute the SQL request
     * 
     * @return : number of parameters expected
     */
    public int getNbParams()
    {
        return _parametersTypes.length;
    }

    public int getLinesPerPage()
    {
        return _iLinesPerPage;
    }

    public int getMaxRows()
    {
        return _iMaxRows;
    }

    public int getDatabaseId()
    {
        return _iDatabaseId;
    }

    public String getParametersTypes()
    {
        StringBuffer sb = new StringBuffer(100);
        for ( int i = 0 ; i < _parametersTypes.length ; i++ )
        {
            if (i > 0)
            {
                sb.append("|");
            }
            sb.append("" + _parametersTypes[i]);
        }
        return sb.toString();
    }

    /**
     * Return the full SQL request for count(*)
     * 
     * @return : SQL request
     */
    public String getFullSqlCount()
    {
        String sSQL = "SELECT COUNT(1) " + _sSqlFrom + " " + _sSqlWhere;
        trace("getFullSqlSelect() : " + sSQL);
        return sSQL;
    }

    /**
     * Return the full SQL request for select data
     * 
     * @return : SQL request
     */
    public String getFullSqlSelect()
    {
        String sSQL = _sSqlSelect + " " + _sSqlFrom + " " + _sSqlWhere + " " + _sSqlOption;
        trace("getFullSqlSelect() : " + sSQL);
        return sSQL;
    }

    /**
     * Return the first row of the given page (calculated with "lines per page")
     * 
     * @param iPage :
     *            page number (from 1 to N )
     * @return : the row number ( from 1 to N )
     */
    public int getFirstRowForPage(int iPage)
    {
        return ((iPage - 1) * _iLinesPerPage) + 1;
    }

    /**
     * Return the last row of the given page (calculated with "lines per page")
     * 
     * @param iPage :
     *            page number (from 1 to N )
     * @return : the row number ( from 1 to N )
     */
    public int getLastRowForPage(int iPage)
    {
        return (iPage * _iLinesPerPage);
    }

    public Object[] castParameters(String strParams[]) throws TelosysException
    {
        if (strParams == null)
        {
            return new Object[0];
        }
        int iParamLength = _parametersTypes.length;
        if (strParams.length < iParamLength)
        {
            throw new TelosysException("Invalid parameters length (" + strParams.length + " < " + iParamLength + ")");
        }

        Object[] objParams = new Object[iParamLength];

        int i = 0;
        String sParam = "(null)";
        try
        {
            for ( i = 0 ; i < iParamLength ; i++ )
            {
                sParam = strParams[i];
                objParams[i] = Cast.toObject(strParams[i], _parametersTypes[i]);
            }
        } catch (Exception e)
        {
            throw new TelosysException("Cannot cast parameter " + (i + 1) + " : '" + sParam + "' ", e);
        } catch (Throwable t)
        {
            throw new TelosysException("Cannot cast parameter " + (i + 1) + " : '" + sParam + "' ", t);
        }

        return objParams;
    }

}