package org.objectweb.telosys.dal.sql;

/** 
 * @author Laurent GUERIN  
 * 
 * This class extends the standard "java.sql.Connection" class 
 * ( it encapsulate a standard connection en implements all the 
 * methods of the interface "Connection" )
 *  
 */
import java.sql.CallableStatement;
import java.sql.Connection;
import java.sql.DatabaseMetaData;
import java.sql.PreparedStatement;
import java.sql.SQLException;
import java.sql.SQLWarning;
import java.sql.Savepoint;
import java.sql.Statement;
import java.util.Map;

public class SqlConnection implements java.sql.Connection
{
  private java.sql.Connection _conn   = null ;
  private SqlConnectionPool   _pool   = null ; // Pool d'appartenance
  private boolean             _bInUse = false ;
  
  //-------------------------------------------------------------
  // Connection : construction without pool
  //-------------------------------------------------------------  
  public SqlConnection( Connection c )
  {
    _conn   = c;
    _pool   = null; // No pool 
    _bInUse = false;
  }
  //-------------------------------------------------------------
  // Connection : contruction with pool
  //-------------------------------------------------------------  
  public SqlConnection( Connection c, SqlConnectionPool pool )
  {
    _conn   = c;
    _pool   = pool; // From pool
    _bInUse = false;
  }

  //-------------------------------------------------------------
  // 
  //-------------------------------------------------------------  
  protected synchronized void inUse( boolean bFlag )
  {
    _bInUse = bFlag ;
  }
  public synchronized boolean isInUse()
  {
    return _bInUse;
  }

  //-------------------------------------------------------------
  // Close the real JDBC connection 
  //-------------------------------------------------------------  
  protected void closeJdbcConnection() throws SQLException
  {
    if ( _conn != null )
    {
      _conn.close();
    }
  }
  //-------------------------------------------------------------  
  protected void tryToCloseJdbcConnection()
  {
    if ( _conn != null )
    {
      try {
        _conn.close();
      }
      catch ( SQLException SqlEx) {}
    }
  }
  //-----------------------------------------------------------------------------
  public boolean isBad() 
  {
    boolean badConnection = false;
    try
    {
      if ( _conn.isClosed() ) 
      {
        return true ; // BAD CONNECTION
      } 
      if ( _conn.getWarnings() != null )
      {
        return true ; // BAD CONNECTION
      } 
        
    }
    catch (Exception e)
    {
      badConnection = true;
      e.printStackTrace();
    }
    return badConnection ;
  }
  
  //-------------------------------------------------------------
  // Overriding (specialisation)
  //-------------------------------------------------------------  
  public String toString()
  {
      // return _conn.toString() + " ( in use = " + _bInUse + " )" ;
      return _conn.toString() + " ( in use = " + isInUse() + " )" ;
  }
  //-------------------------------------------------------------
  // 
  //-------------------------------------------------------------  
  public synchronized void close() throws SQLException
  {
    if ( _pool != null ) // if the connection belongs to a pool
    {
      _bInUse = false ; // Release the connection
    }
    else
    {
      closeJdbcConnection();
    }
  }
  //-------------------------------------------------------------  
  public boolean isClosed() throws SQLException
  { 
    return _conn.isClosed();
  }

  //-------------------------------------------------------------  
  public boolean isReadOnly() throws SQLException
  {
    return _conn.isReadOnly();
  }
  //-------------------------------------------------------------  
  public void setReadOnly(boolean readOnly) throws SQLException
  {
    _conn.setReadOnly(readOnly);
  }

  //-------------------------------------------------------------
  // prepareStatement
  //-------------------------------------------------------------  
  public PreparedStatement prepareStatement(String sql) throws SQLException
  {
    return _conn.prepareStatement(sql);
  }
  //-------------------------------------------------------------
  public PreparedStatement prepareStatement(String sql, int autoGeneratedKeys) throws SQLException
  {
    return _conn.prepareStatement(sql,autoGeneratedKeys);
  }
  //-------------------------------------------------------------
  public PreparedStatement prepareStatement(String sql, int[] columnIndexes) throws SQLException
  {
    return _conn.prepareStatement(sql,columnIndexes);
  }
  //-------------------------------------------------------------
  public PreparedStatement prepareStatement(String sql, String[] columnNames) throws SQLException
  {
    return _conn.prepareStatement(sql,columnNames);
  }
  //-------------------------------------------------------------
  public PreparedStatement prepareStatement(String sql, int resultSetType, int resultSetConcurrency) throws SQLException
  {
    return _conn.prepareStatement(sql,resultSetType,resultSetConcurrency);
  }
  //-------------------------------------------------------------
  public PreparedStatement prepareStatement(String sql, int resultSetType, int resultSetConcurrency, int resultSetHoldability) throws SQLException
  {
    return _conn.prepareStatement(sql,resultSetType,resultSetConcurrency,resultSetHoldability);
  }

  //-------------------------------------------------------------
  // createStatement
  //-------------------------------------------------------------  
  public Statement createStatement() throws SQLException
  {
    return _conn.createStatement();
  }
  //-------------------------------------------------------------  
  public Statement createStatement(int resultSetType, int resultSetConcurrency) throws SQLException
  {
    return _conn.createStatement(resultSetType, resultSetConcurrency);
  }
  //-------------------------------------------------------------  
  public Statement createStatement(int resultSetType, int resultSetConcurrency, int resultSetHoldability) throws SQLException
  { 
    return _conn.createStatement(resultSetType, resultSetConcurrency, resultSetHoldability );
  }

  //-------------------------------------------------------------  
  //  TRANSACTIONS
  //-------------------------------------------------------------  
  public boolean getAutoCommit() throws SQLException
  { 
    return _conn.getAutoCommit();
  }
  //-------------------------------------------------------------  
  public void setAutoCommit(boolean autoCommit) throws SQLException
  { 
    _conn.setAutoCommit(autoCommit);
  }  
  //-------------------------------------------------------------  
  //-------------------------------------------------------------  
  public int getTransactionIsolation() throws SQLException
  {
    return _conn.getTransactionIsolation();
  }
  //-------------------------------------------------------------  
  public void setTransactionIsolation(int level) throws SQLException
  {
    _conn.setTransactionIsolation(level);
  }
  //-------------------------------------------------------------  
  //-------------------------------------------------------------  
  public Savepoint setSavepoint() throws SQLException
  {
    return _conn.setSavepoint();
  }
  //-------------------------------------------------------------  
  public Savepoint setSavepoint(String name) throws SQLException
  {
    return _conn.setSavepoint(name);
  }
  //-------------------------------------------------------------  
  public void releaseSavepoint(Savepoint savepoint) throws SQLException
  {
    _conn.releaseSavepoint(savepoint);
  }
  //-------------------------------------------------------------  
  //-------------------------------------------------------------  
  public void commit() throws SQLException
  { 
    _conn.commit();
  }
  //-------------------------------------------------------------  
  public void rollback() throws SQLException
  { 
    _conn.rollback();
  }
  //-------------------------------------------------------------  
  public void rollback(Savepoint savepoint) throws SQLException
  {
    _conn.rollback(savepoint);
  }
  //-------------------------------------------------------------  
  public void setTypeMap(Map map) throws SQLException
  {
    _conn.setTypeMap(map);
  }
  //-------------------------------------------------------------  
  public Map getTypeMap() throws SQLException
  {
    return _conn.getTypeMap();
  }

  //-------------------------------------------------------------  
  /**
   * @throws java.sql.SQLException
   */
  public void clearWarnings() throws SQLException
  {
      _conn.clearWarnings();
  }

  /* (non-Javadoc)
   * @see java.lang.Object#equals(java.lang.Object)
   */
//  public boolean equals(Object obj)
//  {
//      return _conn.equals(obj);
//  }
  /**
   * @return
   * @throws java.sql.SQLException
   */
  public String getCatalog() throws SQLException
  {
      return _conn.getCatalog();
  }
  /**
   * @return
   * @throws java.sql.SQLException
   */
  public int getHoldability() throws SQLException
  {
      return _conn.getHoldability();
  }
  /**
   * @return
   * @throws java.sql.SQLException
   */
  public DatabaseMetaData getMetaData() throws SQLException
  {
      return _conn.getMetaData();
  }
  /**
   * @return
   * @throws java.sql.SQLException
   */
  public SQLWarning getWarnings() throws SQLException
  {
      return _conn.getWarnings();
  }
  /* (non-Javadoc)
   * @see java.lang.Object#hashCode()
   */
//  public int hashCode()
//  {
//      return _conn.hashCode();
//  }
  /**
   * @param sql
   * @return
   * @throws java.sql.SQLException
   */
  public String nativeSQL(String sql) throws SQLException
  {
      return _conn.nativeSQL(sql);
  }
  /**
   * @param sql
   * @return
   * @throws java.sql.SQLException
   */
  public CallableStatement prepareCall(String sql) throws SQLException
  {
      return _conn.prepareCall(sql);
  }
  /**
   * @param sql
   * @param resultSetType
   * @param resultSetConcurrency
   * @return
   * @throws java.sql.SQLException
   */
  public CallableStatement prepareCall(String sql, int resultSetType, int resultSetConcurrency) throws SQLException
  {
      return _conn.prepareCall(sql, resultSetType, resultSetConcurrency);
  }
  /**
   * @param sql
   * @param resultSetType
   * @param resultSetConcurrency
   * @param resultSetHoldability
   * @return
   * @throws java.sql.SQLException
   */
  public CallableStatement prepareCall(String sql, int resultSetType, int resultSetConcurrency, int resultSetHoldability) throws SQLException
  {
      return _conn.prepareCall(sql, resultSetType, resultSetConcurrency, resultSetHoldability);
  }
  /**
   * @param catalog
   * @throws java.sql.SQLException
   */
  public void setCatalog(String catalog) throws SQLException
  {
      _conn.setCatalog(catalog);
  }
  /**
   * @param holdability
   * @throws java.sql.SQLException
   */
  public void setHoldability(int holdability) throws SQLException
  {
      _conn.setHoldability(holdability);
  }
    
}
