package org.objectweb.telosys.dal.sql;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.sql.Types;

/**
 * @author Laurent GUERIN
 * 
 */
public final class SqlTool
{
    //-------------------------------------------------------------
    // 
    //-------------------------------------------------------------
    public static String connectionStatus(Connection con)
    {
        String s1 = null;
        String s2 = null;
        String sStatus = null;
        if (con != null)
        {
            try
            {
                if (con.getAutoCommit())
                {
                    s1 = "true";
                }
                else
                {
                    s1 = "false";
                }

                switch (con.getTransactionIsolation())
                {
                case Connection.TRANSACTION_NONE :
                    s2 = "TRANSACTION_NONE";
                    break;
                case Connection.TRANSACTION_READ_COMMITTED :
                    s2 = "TRANSACTION_READ_COMMITTED";
                    break;
                case Connection.TRANSACTION_READ_UNCOMMITTED :
                    s2 = "TRANSACTION_READ_UNCOMMITTED";
                    break;
                case Connection.TRANSACTION_REPEATABLE_READ :
                    s2 = "TRANSACTION_REPEATABLE_READ";
                    break;
                case Connection.TRANSACTION_SERIALIZABLE :
                    s2 = "TRANSACTION_SERIALIZABLE";
                    break;
                default :
                    s2 = "???";
                    break;
                }
                sStatus = "AutoCommit = " + s1 + " / Transaction Isolation Level = " + s2;
            } catch (SQLException ex)
            {
                sStatus = "SQLException : " + ex.getMessage();
            }
        }
        else
        {
            sStatus = "Connection is null !";
        }
        return sStatus;
    }

    //-------------------------------------------------------------
    // 
    //-------------------------------------------------------------
    public static int getIsolationLevel(String sIsolationLevel, int iDefaultValue)
    {
        if (sIsolationLevel != null)
        {
            String s = sIsolationLevel.trim().toUpperCase();
            if (s.equals("TRANSACTION_NONE"))
            {
                return Connection.TRANSACTION_NONE;
            }
            else if (s.equals("TRANSACTION_READ_COMMITTED"))
            {
                return Connection.TRANSACTION_READ_COMMITTED;
            }
            else if (s.equals("TRANSACTION_READ_UNCOMMITTED"))
            {
                return Connection.TRANSACTION_READ_UNCOMMITTED;
            }
            else if (s.equals("TRANSACTION_REPEATABLE_READ"))
            {
                return Connection.TRANSACTION_REPEATABLE_READ;
            }
            else if (s.equals("TRANSACTION_SERIALIZABLE"))
            {
                return Connection.TRANSACTION_SERIALIZABLE;
            }
            else
            {
                return iDefaultValue;
            }
        }
        else
        {
            return iDefaultValue;
        }
    }

    /**
     * Execute a SQL "SELECT" with or without parameters and return a ResultSet
     * @param con the connection to use
     * @param sSql the SQL SELECT
     * @param aParameters
     * @return
     * @throws SQLException
     */
    public static ResultSet executeSelect(Connection con, String sSql, Object aParameters[]) throws SQLException
    {
        ResultSet rs = null;

        //trace("ExecuteSelect()...");
        if (aParameters != null) // if paramters => PreparedStatement
        {
            //--- Prepare the statement
            PreparedStatement pstmt = con.prepareStatement(sSql);

            //--- Set SQL statement parameters 
            if (setSqlParameters(pstmt, aParameters))
            {
                //--- Execute SQL request 
                rs = pstmt.executeQuery();
            }
        }
        else
        // no parameter => "normal" statement
        {
            Statement stmt = con.createStatement();
            if (stmt != null)
            {
                //--- Execute SQL request 
                rs = stmt.executeQuery(sSql);
            }
        }

        //--- Return the ResultSet 
        return rs;
    }

    //-------------------------------------------------------------
    // Fermeture et d-rfrencement des Statements
    //-------------------------------------------------------------
    public static boolean close(ResultSet rs)
    {
        //trace("CloseStatement()...");
        try
        {
            if (rs != null)
            {
                Statement stmt = rs.getStatement(); // Statement ou
                // PreparedStatement ou
                // CallableStatement
                rs.close();
                if (stmt != null)
                {
                    stmt.close(); // Provoque galement la fermeture du
                    // ResultSet s'il existe
                }
            }
        } catch (SQLException SqlEx)
        {
            //error("closeStatement: SQLException: "+ SqlEx.getMessage());
            return false;
        }
        return true;
    }

    //-------------------------------------------------------------
    // Affectation des paramtres de la requte SQL
    //-------------------------------------------------------------
    private static boolean setSqlParameters(PreparedStatement ps, Object aParameters[])
    {
        //trace("setSqlParameters() ...");
        //--- Pour chaque paramtre ...
        for ( int i = 0 ; i < aParameters.length ; i++ )
        {
            //--- Dtermination du type du paramtre
            try
            {
                if (aParameters[i] instanceof String)
                {
                    //trace("setSqlParameters: Type is String ");
                    ps.setString(i + 1, (String) aParameters[i]);
                }
                else if (aParameters[i] instanceof Integer)
                {
                    //trace("setSqlParameters: Type is Integer ");
                    ps.setInt(i + 1, ((Integer) aParameters[i]).intValue());
                }
                else if (aParameters[i] instanceof Double)
                {
                    //trace("setSqlParameters: Type is Double ");
                    ps.setDouble(i + 1, ((Double) aParameters[i]).doubleValue());
                }
                else if (aParameters[i] instanceof java.sql.Date)
                {
                    //trace("setSqlParameters: Type is Date ");
                    ps.setDate(i + 1, (java.sql.Date) aParameters[i]);
                }
                else if (aParameters[i] instanceof java.math.BigDecimal)
                {
                    //trace("setSqlParameters: Type is
                    // BigDecimal ");
                    ps.setBigDecimal(i + 1, (java.math.BigDecimal) aParameters[i]);
                }
                else
                {
                    //error("setSqlParameters:
                    // "+aParameters[i].getClass().getName()+"
                    // unknown type ! ");
                    break;
                }
            } catch (SQLException SqlEx)
            {
                //error("setSqlParameters: SQLException: "+
                // SqlEx.getMessage());
                SqlEx.printStackTrace();
                return false;
            } catch (Exception Ex)
            {
                //error("setSqlParameters: Exception: "+ Ex.getMessage() );
                Ex.printStackTrace();
                return false;
            }
        }
        return true;
    }

    //-------------------------------------------------------------
    // ScreenTitle the column value from a ResultSet ( value can be null )
    //-------------------------------------------------------------
    public static Object getColValue(ResultSet rs, int iCol, int iColType) throws SQLException
    {
        switch (iColType)
        {
        //--- Pour tous les types "NUMERIQUES" JDBC
        case Types.TINYINT :
        case Types.SMALLINT :
        case Types.INTEGER :
        case Types.BIGINT :
        case Types.REAL :
        case Types.FLOAT :
        case Types.DOUBLE :
        case Types.DECIMAL : // My SQL
        case Types.NUMERIC : // Oracle
            // Retour "NULL" possible
            return rs.getBigDecimal(iCol);
        // Remplacement de "NULL" par "ZERO" via "getDouble"
        // rec.setObject(iCol, new BigDecimal(rs.getDouble(iCol)));
        //break;

        //--- Pour tous les types "ALPHANUMERIQUES" JDBC
        case Types.CHAR :
        case Types.VARCHAR :
        case Types.LONGVARCHAR :
            // Retour "NULL" possible
            return rs.getString(iCol);
        // Remplacement de "NULL" par "Chaine Vide"
        // if (sVal == null) sVal = "";
        //break;

        //--- Pour tous les types "DATE" JDBC
        case Types.DATE :
        case Types.TIMESTAMP :
            return rs.getDate(iCol); // Objet java.sql.Date
        //break;

        //--- Pour tous les types "TIME" JDBC
        case Types.TIME :
            return rs.getTime(iCol); // Objet java.sql.Time
        //break;

        /***********************************************************************
         * //--- Pour tous les types "OBJETS BINAIRES" JDBC case Types.BINARY :
         * case Types.LONGVARBINARY : case Types.VARBINARY : rec.setObject(iCol,
         * rs.getBinaryStream(iCol)); break;
         **********************************************************************/
        }
        //--- Pour tous les autres types non grs : retour = null
        return null; // Rien !!
    }
}