package org.objectweb.telosys.rpl;

import java.io.InputStream;

import javax.servlet.http.HttpServletRequest;

import org.objectweb.telosys.common.Telosys;
import org.objectweb.telosys.common.TelosysClassLogger;
import org.objectweb.telosys.common.TelosysException;
import org.objectweb.telosys.common.TelosysFactory;
import org.objectweb.telosys.rpl.xml.IXmlWrapperProvider;
import org.objectweb.telosys.rpl.xml.XmlSaxParser;
import org.objectweb.telosys.rpl.xml.XmlToObjectSaxHandler;
import org.objectweb.telosys.rpl.xml.XmlWrapperManager;
import org.objectweb.telosys.screen.core.ScreenActions;
import org.objectweb.telosys.screen.core.ScreenContext;
import org.objectweb.telosys.screen.core.StandardScreenActions;
import org.objectweb.telosys.screen.env.ScreenContextManager;
import org.objectweb.telosys.screen.env.ScreenSession;
import org.objectweb.telosys.screen.env.ScreenSessionManager;

/**
 * @author Laurent GUERIN
 *  
 */
public final class ScreenRequestProcessor 
{
    private static final TelosysClassLogger log                   = new TelosysClassLogger(ScreenRequestProcessor.class);

    //--- Open/Close Screen
    public final static String              ACTION_OPEN           = "open";

    public final static String              ACTION_CLOSE          = "close";

    //--- Screen context Actions
    public final static String              ACTION_GET            = "get";

    public final static String              ACTION_SET            = "set";

    public final static String              ACTION_CLEAR          = "clear";

    //--- Data Access Layer Actions ( use DAO )
    public final static String              ACTION_LOAD           = "load";

    public final static String              ACTION_SAVE           = "save";

    public final static String              ACTION_SAVE_CURRENT   = "save_current";

    public final static String              ACTION_INSERT         = "insert";

    public final static String              ACTION_UPDATE         = "update";

    public final static String              ACTION_DELETE         = "delete";

    public final static String              ACTION_DELETE_CURRENT = "delete_current";

    //--- Call specific process
    public final static String              ACTION_EXEC           = "exec";

    //--- Test / Debug actions
    public final static String              ACTION_PING           = "ping";

    public final static String              ACTION_ECHO           = "echo";
    
    /**
     * No constructor for this class
     */
    private ScreenRequestProcessor()
    {
    }

    //---------------------------------------------------------------------------
    /**
     * Parse the XML input stream and return the resulting ScreenRequest instance
     * 
     * @param input
     * @return
     * @throws TelosysException
     */
    //---------------------------------------------------------------------------
    public static ScreenRequest parseInputStream(InputStream input) throws TelosysException
    {
        log.trace("getScreenRequest()...");
        if (input == null)
        {
            throw new TelosysException("ScreenRequestProcessor.getScreenRequest : InputStream parameter is null ! ");
        }
        //---------------------------------------------------------------------------
        // Prepare to process the XML input Stream 
        //---------------------------------------------------------------------------
        TelosysFactory factory = Telosys.getFactory();
        if (factory == null)
        {
            throw new TelosysException("ScreenRequestProcessor.getScreenRequest : Cannot get Telosys factory ! ");
        }
        
        IXmlWrapperProvider wrapperProvider = factory.getXmlWrapperProvider() ;
        if (wrapperProvider == null)
        {
            throw new TelosysException("ScreenRequestProcessor.getScreenRequest : XML wrapper provider is null ! ");
        }

        //--- Create the XML wrappers manager
        XmlWrapperManager wrapperManager = new XmlWrapperManager(wrapperProvider);

        XmlToObjectSaxHandler handler = new XmlToObjectSaxHandler(wrapperManager);
        XmlSaxParser parser = new XmlSaxParser(handler);

        //---------------------------------------------------------------------------
        // Parse the XML input stream ( load the ScreenData hold by the ScreenRequest )
        //---------------------------------------------------------------------------
        log.trace("start XML parsing()...");
        try
        {
            //--- Parsing the XML input stream
            parser.parse(input);
        } catch (Exception ex)
        {
            throw new TelosysException("ScreenRequestProcessor.getScreenRequest : Xml parsing error !", ex);
        }

        //---------------------------------------------------------------------------
        // Return the Screen Request
        //---------------------------------------------------------------------------
        log.trace("getScreenRequest() : get the screen request from the wrapper manager");
        ScreenRequest request = wrapperManager.getScreenRequest();

        return request;
    }

    //---------------------------------------------------------------------------
    /**
     * Return a ScreenResponse instance created from a given ScreenRequest.
     * 
     * @param request
     * @return
     */
    //---------------------------------------------------------------------------
    public static ScreenResponse getScreenResponse(ScreenRequest request) throws TelosysException
    {
        log.trace("getScreenResponse()...");

        if (request == null)
        {
            throw new TelosysException("ScreenRequestProcessor.getScreenResponse : ScreenRequest parameter is null ! ");
        }

        //---------------------------------------------------------------------------
        //--- Create a Screen Response
        //---------------------------------------------------------------------------
        log.trace("getScreenResponse() : create a screen response");
        String sScreenName = request.getScreenName();
        String sScreenId = request.getScreenId();
        String sAction = request.getAction();
        String sProcName = request.getProcName();

        ScreenResponse response = new ScreenResponse(sScreenName, sScreenId, sAction, sProcName);
        return response;
    }

    /**
     * Return the current ScreenSession for the HttpSession, throws a TelosysException if the session is not opened.
     * 
     * @param httpRequest
     * @return
     * @throws TelosysException
     */
    public static ScreenSession getScreenSession(HttpServletRequest httpRequest) throws TelosysException
    {
        log.trace("getScreenSession()...");
        if (httpRequest == null)
        {
            throw new TelosysException("ScreenRequestProcessor.getScreenSession : HttpServletRequest parameter is null ! ");
        }
        ScreenSession screenSession = ScreenSessionManager.getScreenSession(httpRequest);
        if (screenSession == null)
        {
            //--- No Screen Session ( no screen opened for this user http session )
            throw new TelosysException("ScreenRequestProcessor.getScreenSession : Screen session not found (session not opened) !");
        }
        //--- Screen Session found
        return screenSession;
    }

    /**
     * Return the integer value of Screen ID String
     * 
     * @param sScreenId
     * @return
     * @throws TelosysException
     */
    private static int getIntScreenId(String sScreenId) throws TelosysException
    {
        log.trace("getIntScreenId(" + sScreenId + ")...");

        //--- Get the Screen Id
        int iScreenId = 0;
        try
        {
            iScreenId = Integer.parseInt(sScreenId);
        } catch (Exception e)
        {
            throw new TelosysException("ScreenRequestProcessor.openNewScreen : Invalid screen Id ( " + sScreenId + " ) ! ");
        }
        if (iScreenId < 0)
        {
            throw new TelosysException("ScreenRequestProcessor.openNewScreen : Invalid screen Id ( " + sScreenId + " < 0 ) ! ");
        }
        if (iScreenId > ScreenSession.getMaxScreenId())
        {
            throw new TelosysException("ScreenRequestProcessor.openNewScreen : Invalid screen Id ( " + sScreenId + " > " + ScreenSession.getMaxScreenId() + " ) ! ");
        }
        return iScreenId;
    }

    /**
     * Return the current Screen Context corresponding to the Screen Id and Screen Name of the request
     * 
     * @param request
     * @param screenSession
     * @return
     * @throws TelosysException
     */
    public static ScreenContext getScreenContext(ScreenRequest request, ScreenSession screenSession) throws TelosysException
    {
        log.trace("getScreenContext()...");

        if (request == null)
        {
            throw new TelosysException("ScreenRequestProcessor.getScreenContext : ScreenRequest parameter is null ! ");
        }
        if (screenSession == null)
        {
            throw new TelosysException("ScreenRequestProcessor.getScreenContext : ScreenSession parameter is null ! ");
        }

        //--- Get the Screen Id ( Exception if invalid )
        int iScreenId = getIntScreenId(request.getScreenId());

        //--- Get the action
        String sAction = request.getAction();

        ScreenContext screenContext = screenSession.getScreen(iScreenId);
        if (screenContext == null)
        {
            //--- No current screen context for this ID
            if (sAction.equalsIgnoreCase(ACTION_OPEN))
            {
                //--- Ok, the screen will be created by the action
                return null;
            }
            else
            {
                throw new TelosysException("Screen context not found ( Id = " + request.getScreenId() + " ), Screen not initialized ( in ScreenRequestProcessor.getScreenContext() )");
            }
        }
        else
        {
            //--- There's a current screen context for this ID
            if (!sAction.equalsIgnoreCase(ACTION_OPEN))
            {
                //--- If NOT "open" action => check the context type
                //--- Get the Screen Name
                String sScreenName = request.getScreenName();
                if (sScreenName == null)
                {
                    throw new TelosysException("No Screen Name in the request !");
                }
                //--- Is it the expected Screen Context ?
                if (!sScreenName.equalsIgnoreCase(screenContext.getScreenName()))
                {
                    throw new TelosysException("The ScreenContext for id '" + iScreenId + "' is '" + screenContext.getScreenName() + "' ! ( '" + sScreenName + "' expected ) ");
                }
            }
        }

        return screenContext;
    }

    //---------------------------------------------------------------------------
    /**
     * Execute the ScreenRequest ACTION in a ScreenContext, and set the ScreenResponse.
     * 
     * @param screenContext
     * @param request
     * @param response
     */
    public static void executeSynchronizedAction(ScreenSession screenSession, ScreenContext screenContext, ScreenRequest request, ScreenResponse response) throws TelosysException
    {
        log.trace("executeSynchronizedAction()...");
        //--- Lock the Screen Session Object
        synchronized (screenSession)
        {
            executeAction(screenSession, screenContext, request, response);
        }
    }

    //---------------------------------------------------------------------------
    /**
     * Execute the ScreenRequest ACTION in a ScreenContext, and set the ScreenResponse.
     * 
     * @param screenContext
     * @param request
     * @param response
     */
    public static void executeAction(ScreenSession screenSession, ScreenContext screenContext, ScreenRequest request, ScreenResponse response) throws TelosysException
    {
        log.trace("executeAction()...");

        //--- Check the parameters (the ScreenContext can be null )
        if (request == null)
        {
            throw new TelosysException("ScreenRequestProcessor.executeAction : ScreenRequest parameter is null ! ");
        }
        if (response == null)
        {
            throw new TelosysException("ScreenRequestProcessor.executeAction : ScreenResponse parameter is null ! ");
        }

        String sAction = request.getAction();
        log.trace("  Action = " + sAction);
        if (sAction == null)
        {
            throw new TelosysException("ScreenRequestProcessor.executeAction : Request action is null ! ");
        }
        sAction = sAction.toLowerCase();
        if (screenContext == null)
        {
            if (!sAction.equals(ACTION_OPEN))
            {
                //--- No screen context and action != OPEN => ERROR
                throw new TelosysException("ScreenRequestProcessor.executeAction : ScreenContext parameter is null ! ");
            }
        }

        //--- Execute the action
        try
        {
            //----------------------------------------------------------
            if (sAction.equals(ACTION_OPEN))
            {
                openScreen(screenSession, request);
                return;
            }
            //----------------------------------------------------------
            if (sAction.equals(ACTION_CLOSE))
            {
                closeScreen(screenSession, request);
                return;
            }
            //----------------------------------------------------------
            if (sAction.equals(ACTION_PING))
            {
                ping(screenSession, request, response);
                return;
            }
            //----------------------------------------------------------
            if (sAction.equals(ACTION_ECHO))
            {
                echo(screenSession, request, response);
                return;
            }
            //----------------------------------------------------------

            ScreenActions actions = screenContext.getScreenActions();
            if (actions == null)
            {
                actions = new StandardScreenActions();
            }

            //----------------------------------------------------------
            if (sAction.equals(ACTION_GET))
            {
                actions.actionGet(screenContext, request, response);
            }
            //----------------------------------------------------------
            else if (sAction.equals(ACTION_SET))
            {
                actions.actionSet(screenContext, request, response);
            }
            //----------------------------------------------------------
            else if (sAction.equals(ACTION_CLEAR))
            {
                actions.actionClear(screenContext, request, response);
            }
            //----------------------------------------------------------
            //----------------------------------------------------------
            else if (sAction.equals(ACTION_LOAD))
            {
                actions.actionLoad(screenContext, request, response);
            }
            //----------------------------------------------------------
            else if (sAction.equals(ACTION_SAVE))
            {
                actions.actionSave(screenContext, request, response);
            }
            //----------------------------------------------------------
            else if (sAction.equals(ACTION_SAVE_CURRENT))
            {
                actions.actionSaveCurrent(screenContext, request, response);
            }
            //----------------------------------------------------------
            else if (sAction.equals(ACTION_DELETE))
            {
                actions.actionDelete(screenContext, request, response);
            }
            //----------------------------------------------------------
            else if (sAction.equals(ACTION_DELETE_CURRENT))
            {
                actions.actionDeleteCurrent(screenContext, request, response);
            }
            //----------------------------------------------------------
            else if (sAction.equals(ACTION_INSERT))
            {
                actions.actionInsert(screenContext, request, response);
            }
            //----------------------------------------------------------
            else if (sAction.equals(ACTION_UPDATE))
            {
                actions.actionUpdate(screenContext, request, response);
            }
            //----------------------------------------------------------
            //----------------------------------------------------------
            else if (sAction.equals(ACTION_EXEC))
            {
                actions.actionExec(screenContext, request, response);
            }
            //----------------------------------------------------------
            else
            {
                String sMsg = "Invalid action : " + sAction;
                Telosys.error(sMsg);
                response.setReturnCode(ScreenResponse.INVALID_ACTION);
                response.setReturnMessage(sMsg);
            }
        } catch (TelosysException tex)
        {
            Telosys.error(tex);
            response.setActionReturn(ScreenResponse.ACTION_ERROR);
            response.addError(tex);
        } catch (Exception ex)
        {
            Telosys.error(ex);
            response.setActionReturn(ScreenResponse.ACTION_ERROR);
            response.addError(ex);
        } catch (Throwable th)
        {
            Telosys.error(th);
            response.setActionReturn(ScreenResponse.ACTION_ERROR);
            response.addError(th);
        }
    }

    //---------------------------------------------------------------------------
    /**
     * Open the screen specified in the request (create a new ScreenContext)
     * 
     * @param screenSession
     * @param screenRequest
     * @throws TelosysException
     */
    private static void openScreen(ScreenSession screenSession, ScreenRequest screenRequest) throws TelosysException
    {
        log.trace("openScreen()...");
        //--- Get the screen NAME
        String sScreenName = screenRequest.getScreenName();
        //--- Get the screen ID
        int iScreenId = getIntScreenId(screenRequest.getScreenId());
        //--- Open (create) the screen context
        ScreenContextManager.openScreenContext(screenSession, sScreenName, iScreenId);
    }

    //---------------------------------------------------------------------------
    /**
     * Close the screen specified in the request (remove the ScreenContext)
     * 
     * @param screenSession
     * @param screenRequest
     * @throws TelosysException
     */
    private static void closeScreen(ScreenSession screenSession, ScreenRequest screenRequest) throws TelosysException
    {
        log.trace("closeScreen()...");
        //--- Get the screen ID
        int iScreenId = getIntScreenId(screenRequest.getScreenId());
        //--- Close = remove the screen context from the current session
        ScreenContextManager.closeScreenContext(screenSession, iScreenId);
    }

    //---------------------------------------------------------------------------
    private static void ping(ScreenSession screenSession, ScreenRequest screenRequest, ScreenResponse response) throws TelosysException
    {
        log.trace("ping()...");
    }
    //---------------------------------------------------------------------------
    private static void echo(ScreenSession screenSession, ScreenRequest screenRequest, ScreenResponse response) throws TelosysException
    {
        log.trace("echo()...");
        /**
        ScreenData screenData = screenRequest.getData();        
        response.setDataElements(screenData);
        **/
        response.setDataElements( screenRequest.getDataElements() );
    }
    //---------------------------------------------------------------------------
}