package org.objectweb.telosys.rpl.xml;

import java.util.LinkedList;

import org.objectweb.telosys.common.Telosys;
import org.objectweb.telosys.common.TelosysException;
import org.objectweb.telosys.common.TelosysFactory;
import org.objectweb.telosys.common.TelosysObject;
import org.objectweb.telosys.common.TelosysRuntimeException;
import org.objectweb.telosys.rpl.ElementClassResolver;
import org.objectweb.telosys.rpl.ScreenRequest;

/**
 * This class provides methods called by the XML SAX parser.<br>
 * It holds a ScreenRequest instance that is built from the XML input stream, tag by tag.
 * @author Laurent GUERIN
 * 
 */
public class XmlWrapperManager extends TelosysObject
{
    private final static int        DATA_LEVEL            = 2;

    private final static int        PARAMS_LEVEL          = 2;

    private int                     _iNbTagRequest        = 0;

    //--- ScreenRequest Instance
    private ScreenRequest           _screenRequest        = null;

    private ScreenRequestXmlWrapper _screenRequestWrapper = null;

    private boolean                 _bInTagParams         = false;

    private boolean                 _bInTagData           = false;

    //--- Application Wrappers Provider
    private IXmlWrapperProvider     _wrapperProvider      = null;

    //--- TREE STACK : STORE WRAPPERS ( NOT DATA OBJECTS ) ( index 0 to size-1 ) 
    private LinkedList              _stack                = new LinkedList();

    //-----------------------------------------------------------------------------------------------
    public static IXmlWrapperProvider getXmlWrapperProvider() throws TelosysException
    {
        //--- Get the Telosys factory
        TelosysFactory factory = Telosys.getFactory();
        if (factory == null)
        {
            throw new TelosysException("ScreenRequestProcessor.getScreenRequest : Cannot get Telosys factory ! ");
        }
        
        //--- Get the XML wrapper provder from the Telosys factory
        IXmlWrapperProvider wrapperProvider = factory.getXmlWrapperProvider() ;
        if (wrapperProvider == null)
        {
            throw new TelosysException("ScreenRequestProcessor.getScreenRequest : XML wrapper provider is null ! ");
        }
        
        return wrapperProvider ;
    }
    
    //-----------------------------------------------------------------------------------------------
    /**
     * Constructor 
     */
    public XmlWrapperManager() throws TelosysException
    {
        super();        
        _wrapperProvider = getXmlWrapperProvider() ;
    }
    
    //-----------------------------------------------------------------------------------------------
    /**
     * Constructor 
     * @param wrapperProvider : the XML wrapper provider to use 
     */
    public XmlWrapperManager(IXmlWrapperProvider wrapperProvider)
    {
        super();
        _wrapperProvider = wrapperProvider;
    }

    //-----------------------------------------------------------------------------------------------
    private void register(String sTagName, IXmlWrapper wrapper, int iLevel)
    {
        trace("register (" + wrapper.getClass().getName() + ", " + iLevel + ")");
        //--- Storage in the stack
        while (_stack.size() <= iLevel) //--- Check stack size (OK if size > iLevel)
        {
            _stack.add(null); //--- Void level
        }
        _stack.set(iLevel, wrapper); //--- register the wrapper

        //--- Parent notification
        int iParentLevel = iLevel - 1; // Parent level
        IXmlWrapper parent = null;
        if (iParentLevel >= 0)
        {
            if (iParentLevel < _stack.size())
            {
                parent = (IXmlWrapper) _stack.get(iParentLevel);
            }
            else
            {
                error("register() : iParentLevel = " + iParentLevel + " stack size = " + _stack.size());
            }
        }
        if (parent != null)
        {
            // Notify the parent : there's a new child !
            parent.newXmlChild(sTagName, wrapper);
        }
        else
        // No Parent
        {
            if (_bInTagData)
            {
                if (iLevel == DATA_LEVEL + 1)
                {
                    // It is a ScreenRequest DATA (top level)
                    _screenRequest.storeDataElement(sTagName, wrapper.getObject());
                }
                else
                {
                    // It is an ERROR
                    error ( "parent not found for this wrapper (class=" + wrapper.getClass().getName() + ", level=" + iLevel + ")");
                }
            }
        }
    }
    //-----------------------------------------------------------------------------------------------
    private void unregister(int iLevel)
    {
        trace("unregister (" + iLevel + ")");
        if ( iLevel < _stack.size() )
        {
            _stack.set(iLevel, null); //--- unregister the wrapper            
        }
    }

    //-----------------------------------------------------------------------------------------------
    /**
     * Return the screen request instance built from the XML input stream
     * @return
     */
    public ScreenRequest getScreenRequest()
    {
        return _screenRequest;
    }

    //-----------------------------------------------------------------------------------------------
    /**
     * Return the XML Wrapper for the given tag name <br>
     * ( method called by the XML SAX parser for each new tag )<br>
     * ( SAX "startElement" event )
     * @param sTagName
     * @param iLevel
     * @return
     */
    public IXmlWrapper openTag(String sTagName, int iLevel)
    {
        trace(" openTag(" + sTagName + ", " + iLevel + ")");
        //        _iCurrentLevel = iLevel;

        //--- TAGS STANDARDS
        //-----------------------------------------------------------------------------------------------
        //  <request ... >
        //-----------------------------------------------------------------------------------------------
        if (sTagName.equals("request"))
        {
            if (_screenRequest == null)
            {
                trace(" new ScreenRequest() ");
                _screenRequest = new ScreenRequest();
            }
            if (_screenRequestWrapper == null)
            {
                _screenRequestWrapper = new ScreenRequestXmlWrapper(_screenRequest);
            }
            if (_iNbTagRequest != 0)
            {
                error("Multiple tag <request> in XML stream !! ");
            }
            _iNbTagRequest++;
            return _screenRequestWrapper;
        }

        //-----------------------------------------------------------------------------------------------
        //  Opening "data" section
        //  <data ... > : no attribute => no wrapper
        //-----------------------------------------------------------------------------------------------
        else if (sTagName.equals("data")) // Tag <data ..../>
        {
            if (iLevel == DATA_LEVEL)
            {
                _bInTagData = true;
                trace(" ScreenRequest createData() ");
                //--- NEW
                //_screenRequest.createData();
                ElementClassResolver.initScreenDataDefinition(_screenRequest.getScreenName());
                
                return null;
            }
        }

        //-----------------------------------------------------------------------------------------------
        //  <params> : beginning of parameters list (no attribute => no wrapper)
        //-----------------------------------------------------------------------------------------------
        else if (sTagName.equals("params"))
        {
            if (iLevel == PARAMS_LEVEL)
            {
                _bInTagParams = true;
                return null;
            }
        }

        //-----------------------------------------------------------------------------------------------
        //  <param ... > : parameter name and value
        //-----------------------------------------------------------------------------------------------
        else if (sTagName.equals("param"))
        {
            if (_bInTagParams)
            {
                RequestParamXmlWrapper wrapper = new RequestParamXmlWrapper(_screenRequest);
                register(sTagName, wrapper, iLevel);
                return wrapper;
            }
        }

        //-----------------------------------------------------------------------------------------------
        //--- Application Beans TAGS 
        //-----------------------------------------------------------------------------------------------
        else
        {
            // OLD (get by TagName) : IXmlWrapper wrapper = _wrapperProvider.getWrapper(sTagName);
            IXmlWrapper wrapper = null ;
         /***
            //--- At this level the ScreenData is supposed to be created in the ScreenRequest 
            ScreenData screenData = _screenRequest.getData();
            if ( screenData != null )
            {
                Class elementClass = screenData.getElementClass(sTagName);
                ElementClassResolver.getElementClass(_screenRequest.getScreenName(), sTagName);
                if ( elementClass != null )
                {
                    wrapper = _wrapperProvider.getWrapper(elementClass);
                }
            }
         ***/
            
            //--- NEW 
            Class elementClass = ElementClassResolver.getElementClass(_screenRequest.getScreenName(), sTagName);
            if ( elementClass != null )
            {
                wrapper = _wrapperProvider.getWrapper(elementClass);
            }
            
            //--- Is there a wrapper for this tag ?
            if (wrapper != null)
            {
                register(sTagName, wrapper, iLevel);
            }
            else
            {
                if ( _bInTagData)
                {
                    String sErrMsg = "No wrapper for the tag <"+sTagName+"> (in DATA tree)" ;
                    //error ( sErrMsg );
                    throw new TelosysRuntimeException(sErrMsg);
                }
            }
            return wrapper;
        }
        return null;
    }

    //-----------------------------------------------------------------------------------------------
    /**
     * Method called by the XML SAX parser for each end of tag<br>
     * ( SAX "endElement" event )
     * @param sTagName
     * @param iLevel
     */
    public void closeTag(String sTagName, int iLevel)
    {
        trace(" closeTag(" + sTagName + ", " + iLevel + ")");

        //--- TAGS STANDARDS
        //-----------------------------------------------------------------------------------------------
        //  </request ... >
        //-----------------------------------------------------------------------------------------------
//        if (sTagName.equals("request"))
//        {
//            _screenRequest.endOfInitialization();
//        }
//        else if (sTagName.equals("params"))
        //-----------------------------------------------------------------------------------------------
        //  </params>
        //-----------------------------------------------------------------------------------------------
        if (sTagName.equals("params"))
        {
            if (iLevel == PARAMS_LEVEL)
            {
                _bInTagParams = false;
            }
        }
        //-----------------------------------------------------------------------------------------------
        //  </data>
        //-----------------------------------------------------------------------------------------------
        else if (sTagName.equals("data"))
        {
            if (iLevel == DATA_LEVEL)
            {
                _bInTagData = false;
            }
        }
        else
        {
            unregister(iLevel);
        }

    }
}