package org.objectweb.telosys.screen.core;

import javax.servlet.http.HttpSession;

import org.objectweb.telosys.screen.env.ScreenApplication;
import org.objectweb.telosys.screen.env.ScreenSession;

/**
 * @author Laurent GUERIN
 *  
 */
public interface ScreenContext
{
    //---------------------------------------------------------------------------
    // get the basic screen elements ( name, session, ... )
    //---------------------------------------------------------------------------
    /**
     * Returns the symbolic screen name of the screen context
     * @return the screen context name
     */
    public String getScreenName();

    /**
     * Returns the ID of the screen context 
     * @return : screen context ID ( 0 to N )
     */
    public int getScreenId();

    // public Class getScreenClass(); // SUPPR v 0.9.4

    // public String getScreenClassName(); // SUPPR v 0.9.4

    /**
     * Returns the ScreenApplication instance 
     * @return the ScreenApplication
     */
    public ScreenApplication getScreenApplication();
    
    /**
     * Returns the ScreenSession to which the ScreenContext belongs  
     * @return the ScreenSession
     */
    public ScreenSession getScreenSession();

    /**
     * Returns the standard HttpSession to which the ScreenContext belongs   
     * @return
     */
    public HttpSession getHttpSession();

    //---------------------------------------------------------------------------
    // ScreenData management 
    //---------------------------------------------------------------------------
    /**
     * Returns the current ScreenData of the context (can be null)
     * @return : the current context data or null
     */
    public ScreenData getCurrentData();

    /**
     * Creates a new ScreenData instance and returns it.<br>
     * NB: the ScreenData created is not set as the current ScreenData of the context 
     * @return : the new ScreenData created
     */
    public ScreenData createNewScreenData();

    /**
     * Returns the current ScreenData of the context if it's not null, <br>
     * if the current ScreenData is null : it is created, set as current and returned
     * @return : the current data (never null)
     */
    public ScreenData getOrCreateCurrentData();

    /**
     * Sets the given ScreenData as the new current ScreenData of the context 
     * @param : the new ScreenData to set
     */
    public void setCurrentData(ScreenData data);

    /**
     * Remove the current ScreenData of the context <br>
     * After this operation, getCurrentData() returns 'null' 
     */
    public void clearCurrentData();
    
    /**
     * Notify the screen context that an action is about to begin<br>
     * The action needs a ScreenData instance to work with<br>
     * The screen context keep an image of the current ScreenData<br>
     * to do a "cancel" if needed
     * @return the instance to use
     */
    public ScreenData beginAction() ;
    
    /**
     * Notify the screen context that the current action must be canceled<br>
     * The screen context restore the ScreenData image <br>
     * 
     * @return the instance to use
     */
    public void cancelAction() ;

    /**
     * Returns a temporary ScreenData instance dedicated to 'delete' operations
     * @return : ScreenData instance
     */
    public ScreenData getDataForDelete() ;

    /**
     * Creates and returns a new ScreenData instance for 'load' operations 
     * @return : ScreenData instance
     */
    public ScreenData getDataForLoad() ;
    
    //---------------------------------------------------------------------------
    // put/get/remove specific data in the Screen Context
    //---------------------------------------------------------------------------
    /**
     * Puts an object in the 'screen context storage space', associated with a unique object key 
     * If key or value is null : do nothing
     * @param key : the key
     * @param value : the value  
     * @return : the previous value of the specified key in this hashtable, or null if it did not 
	 have one
     */
    public Object put(Object key, Object value);

    /**
     * Gets an object from the 'screen context storage space' with the given key 
     * If key is null : return null
     * @param key : the key
     * @return : value instance ( or null if key is null or not found ) 
     */
    public Object get(Object key);

    /**
     * Removes the key (and its corresponding value) from the 'screen context storage space'
     * If key is null : do nothing and return null
     * @param key : the key
     * @return : the value removed, or null if not found
     */
    public Object remove(Object key);

    //---------------------------------------------------------------------------
//    /**
//     * Returns the ScreenDAO associated with the ScreenContext
//     * @return : the ScreenDAO (or null)
//     */
//    public ScreenDAO getScreenDAO();

    /**
     * Returns the ScreenDataAccessor associated with the ScreenContext
     * @return
     */
    public ScreenDataAccessor getScreenDataAccessor();

    /**
     * Returns the specific ScreenActions associated with the ScreenContext
     * @return : the ScreenActions (or null)
     */
    public ScreenActions getScreenActions();

    /**
     * Returns the ScreenTriggers associated with the ScreenContext
     * @return : the ScreenTriggers (or null)
     */
    public ScreenTriggers getScreenTriggers();

    /**
     * Returns the ScreenProcedures associated with the ScreenContext
     * @return : the ScreenProcedures (or null)
     */
    public ScreenProcedures getScreenProcedures();

    // public DatabaseSession getDatabaseSession() throws TelosysException; // SUPPR v 0.9.4

}