package org.objectweb.telosys.screen.env;

import javax.servlet.http.HttpServletRequest;

import org.objectweb.telosys.common.Telosys;
import org.objectweb.telosys.common.TelosysException;
import org.objectweb.telosys.common.TelosysFactory;
import org.objectweb.telosys.screen.core.ScreenContext;

/**
 * @author Laurent GUERIN
 *  
 */
public final class ScreenContextManager
{
    /**
     * No constructor for this class
     */
    private ScreenContextManager()
    {
    }

    //---------------------------------------------------------------------------
    // OPEN
    //---------------------------------------------------------------------------
    /**
     * Open (create) a new Screen Context 
     * @param request
     * @param sScreenName
     * @param iScreenId
     * @return
     * @throws TelosysException
     */
    public static ScreenContext openScreenContext(HttpServletRequest request, String sScreenName, int iScreenId) throws TelosysException
    {
        if (request == null)
        {
            throw new TelosysException("openScreenContext : request is null !");
        }

        //--- Get the screen session
        ScreenSession screenSession = ScreenSessionManager.getScreenSession(request);
        if (screenSession == null)
        {
            throw new TelosysException("openScreenContext : cannot get Screen Session !");
        }
        
        return openScreenContext(screenSession, sScreenName, iScreenId);
    }

    /**
     * Open (create) a new Screen Context
     * @param screenSession
     * @param sScreenName
     * @param iScreenId
     * @return
     * @throws TelosysException
     */
    public static ScreenContext openScreenContext(ScreenSession screenSession, String sScreenName, int iScreenId) throws TelosysException
    {
        if (screenSession == null)
        {
            throw new TelosysException("openScreenContext : screen session is null !");
        }

        //--- Get the screen factory
        //TelosysFactory factory = TelosysScreen.getFactory();
        TelosysFactory factory = Telosys.getFactory();
        if (factory == null)
        {
            throw new TelosysException("openScreenContext : Cannot get the TelosysFactory !");
        }

        //--- Create the new screen
        ScreenContext screenContext = factory.createScreen(screenSession, sScreenName, iScreenId );
        if (screenContext == null)
        {
            throw new TelosysException("openScreenContext : Cannot create the screen (factory returns null) !");
        }

        //--- Set the new screen in the current session
        screenSession.setScreen(iScreenId, screenContext);

        return screenContext;
    }

    //---------------------------------------------------------------------------
    // CLOSE
    //---------------------------------------------------------------------------
    /**
     * Close screen context
     * @param request
     * @param sScreenName
     * @param iScreenId
     * @throws TelosysException
     */
    public static void closeScreenContext(HttpServletRequest request, int iScreenId) throws TelosysException
    {
        if (request == null)
        {
            throw new TelosysException("closeScreenContext : request is null !");
        }

        //--- Get the screen session
        //ScreenSession screenSession = ScreenSessionManager.getScreenSession(request);
        ScreenSession screenSession = ScreenSessionManager.findScreenSession(request);
        if (screenSession == null)
        {
            throw new TelosysException("closeScreenContext : cannot get Screen Session !");
        }
        
        closeScreenContext(screenSession, iScreenId);
    }
    
    /**
     * Close screen context
     * @param screenSession
     * @param sScreenName
     * @param iScreenId
     * @throws TelosysException
     */
    public static void closeScreenContext(ScreenSession screenSession, int iScreenId) throws TelosysException
    {
        if (screenSession == null)
        {
            throw new TelosysException("closeScreenContext : screen session is null !");
        }

        //--- Remove the screen from the screen session
        screenSession.removeScreen(iScreenId);
    }
    
    //---------------------------------------------------------------------------
    // USE  ( RETRIEVE AN EXISTING SCREEN or CREATE A NEW ONE IF IT DOESN'T EXIST )
    //---------------------------------------------------------------------------
    /**
     * Retrieve an existing screen context or create a new one
     * @param request
     * @param sScreenName
     * @param iScreenId
     * @return
     * @throws TelosysException
     */
    public static ScreenContext useScreenContext(HttpServletRequest request, String sScreenName, int iScreenId) throws TelosysException
    {
        if (request == null)
        {
            throw new TelosysException("useScreenContext : request is null !");
        }

        //--- Get the screen session
        ScreenSession screenSession = ScreenSessionManager.getScreenSession(request);
        if (screenSession == null)
        {
            throw new TelosysException("useScreenContext : cannot get Screen Session !");
        }
        return useScreenContext(screenSession, sScreenName, iScreenId );
    }
    /**
     * Retrieve an existing screen context or create a new screen context
     * @param screenSession
     * @param sScreenName
     * @param iScreenId
     * @return
     * @throws TelosysException
     */
    public static ScreenContext useScreenContext(ScreenSession screenSession, String sScreenName, int iScreenId) throws TelosysException
    {
        if (screenSession == null)
        {
            throw new TelosysException("useScreenContext : screen session is null !");
        }

        //--- Get the screen context
        ScreenContext screenContext = screenSession.getScreen(iScreenId);
        if ( screenContext != null )
        {
            //--- Is it the expected screen name ? 
            if ( sScreenName.equalsIgnoreCase( screenContext.getScreenName() ) )
            {
                return screenContext ;                
            }
            else
            {
                //--- It's not the expected screen => cannot use it => open the expected screen context 
                return openScreenContext(screenSession, sScreenName, iScreenId);
            }
        }
        else
        {
            return openScreenContext(screenSession, sScreenName, iScreenId);
        }
    }

}