package org.objectweb.telosys.screen.env;

import java.util.Hashtable;

import javax.servlet.ServletContext;
import javax.servlet.http.HttpSession;

import org.objectweb.telosys.auth.IAppUser;
import org.objectweb.telosys.auth.LoginUser;
import org.objectweb.telosys.common.TelosysException;
import org.objectweb.telosys.common.TelosysObject;
import org.objectweb.telosys.screen.core.ScreenContext;
import org.objectweb.telosys.service.ServiceResultHolder;
import org.objectweb.telosys.uil.ScreenPostback;
import org.objectweb.telosys.uil.SessionPostback;

/**
 * This class is a context holder for a user session <br>
 * It is attached to the standard http session, it contains the user informations, <br>
 * the user screen contexts, the services results, etc...
 * 
 * @author Laurent GUERIN
 *  
 */
public class ScreenSession extends TelosysObject
{
    //--- Maximum Screens in a User Session (Screen Id : 0 to 20)
    private static final int  NB_MAX_SCREENS     = 21;

    //---------------------------------------------------------------------------
    //--- The "HTTP Session" object linked with this "Screen Session"
    private HttpSession       _httpSession       = null;

    private ScreenApplication _screenApplication = null;

    //--- The technical "Login User" of this session
    private LoginUser         _loginUser         = null;

    //--- The "Application User" of this session
    private IAppUser          _appUser           = null;

    //--- The Screens of this session
    private ScreenContext     _screens[]         = new ScreenContext[NB_MAX_SCREENS];

    //--- The "postback" values
    private SessionPostback   _postback          = null;

    //--- Open storage space (to store anything)
    private Hashtable         _htData            = new Hashtable();

    //---------------------------------------------------------------------------
    /**
     *  
     */
    public ScreenSession(HttpSession httpSession, LoginUser loginUser, IAppUser appUser)
    {
        _httpSession = httpSession;
        _loginUser = loginUser;
        _appUser = appUser;
        //--- Retrieve the Screen Application instance
        _screenApplication = ScreenApplicationManager.getScreenApplication(_httpSession);
        if (_screenApplication == null)
        {
            //--- If not yet created => creation
            _screenApplication = ScreenApplicationManager.openScreenApplication(_httpSession);
        }
    }

    /**
     * Returns the standard HttpSession object
     * 
     * @return
     */
    public HttpSession getHttpSession()
    {
        return _httpSession;
    }

    /**
     * Returns the standard HttpSession id
     * 
     * @return
     */
    public String getHttpSessionId()
    {
        if (_httpSession != null)
        {
            return _httpSession.getId();
        }
        return null;
    }

    /**
     * Returns the standard ServletContext
     * 
     * @return
     */
    public ServletContext getServletContext()
    {
        if (_httpSession != null)
        {
            return _httpSession.getServletContext();
        }
        return null;
    }

    /**
     * Returns the ScreenApplication object
     * 
     * @return
     */
    public ScreenApplication getScreenApplication()
    {
        return _screenApplication;
    }

    /**
     * Returns the SessionPostback object
     * 
     * @return the postback object (or null if not yet created)
     */
    public SessionPostback getSessionPostback()
    {
        return _postback;
    }

    /**
     * Returns the ScreenPostback object for the given Screen key (if not found, the ScreenPostback is created)
     * 
     * @param sScreenKey
     *            the "screen key" ( Screen Name + Screen Context Id )
     * @return
     */
    public ScreenPostback getScreenPostback(String sScreenKey) throws TelosysException
    {
        //--- Create the SessionPostack (if not yet done)
        if (_postback == null)
        {
            _postback = new SessionPostback();
        }
        //--- Get (or create if not found) the ScreenPostback for the given Screen Key
        ScreenPostback screenPostback = _postback.getScreenPostback(sScreenKey);
        if (screenPostback == null)
        {
            throw new TelosysException("Cannot get ScreenPostback for screen key '" + sScreenKey + "' ");
        }
        return screenPostback;
    }

    /**
     * Returns the ScreenPostback for the given Screen Name and Screen Context Id 
     * @param sScreenName
     * @param iScreenContextId
     * @return the ScreenPostback (or null if not found)
     */
    public ScreenPostback getScreenPostback(String sScreenName, int iScreenContextId)
    {
        trace("getScreenPostback("+ sScreenName+"," + iScreenContextId+")...");
        String sScreenKey = SessionPostback.composeScreenKey(sScreenName, iScreenContextId);
        ScreenPostback screenPostback = null;
        try
        {
            screenPostback = this.getScreenPostback(sScreenKey);
        } catch (TelosysException e)
        {
            error("Cannot get ScreenPostback instance from the SessionPostback");
        }
        return screenPostback;
    }

    //---------------------------------------------------------------------------
    /**
     * Returns the maximum Screen ID ( 0 to N )
     * 
     * @return
     */
    public static int getMaxScreenId()
    {
        return NB_MAX_SCREENS - 1;
    }

    //---------------------------------------------------------------------------
    /**
     * Set a screen context in the screen session
     * 
     * @param iScreenId
     * @param screen
     */
    public void setScreen(int iScreenId, ScreenContext screen)
    {
        trace("setScreen(" + iScreenId + ", ... )");
        if (iScreenId >= 0 && iScreenId < NB_MAX_SCREENS)
        {
            _screens[iScreenId] = screen;
        }
    }

    //---------------------------------------------------------------------------
    /**
     * Returns the screen context bound with the specified id
     * 
     * @param iScreenId
     * @return
     */
    public ScreenContext getScreen(int iScreenId)
    {
        trace("getScreen(" + iScreenId + ")");
        if (iScreenId >= 0 && iScreenId < NB_MAX_SCREENS)
        {
            return _screens[iScreenId];
        }
        else
        {
            return null;
        }
    }

    //---------------------------------------------------------------------------
    /**
     * Removes the screen context bound with the specified id
     * 
     * @param iScreenId
     */
    public void removeScreen(int iScreenId)
    {
        trace("removeScreen(" + iScreenId + ", ... )");
        if (iScreenId >= 0 && iScreenId < NB_MAX_SCREENS)
        {
            _screens[iScreenId] = null;
        }
    }

    //---------------------------------------------------------------------------
    /**
     * Store a service result holder in screen session with the given key
     * 
     * @param sKey
     * @param srh
     *            the service result holder
     */
    public void setServiceResultHolder(String sKey, ServiceResultHolder srh)
    {
        _htData.put(sKey, srh);
        trace("setServiceResultHolder(" + sKey + ", ..) : stored.");
    }

    //---------------------------------------------------------------------------
    /**
     * Return the service result holder stored in screen session with the given key
     * 
     * @param sKey
     * @return the service result holder found (or null if not found)
     */
    public ServiceResultHolder getServiceResultHolder(String sKey)
    {
        Object object = _htData.get(sKey);
        if (object != null)
        {
            if (object instanceof ServiceResultHolder)
            {
                trace("getServiceResultHolder(" + sKey + ") : found.");
                return (ServiceResultHolder) object;
            }
            else
            {
                error("getServiceResultHolder(" + sKey + ") : found, but it's not an instance of ServiceResultHolder.");
            }
        }
        else
        {
            trace("getServiceResultHolder(" + sKey + ") : not found.");
        }
        return null;
    }

    //---------------------------------------------------------------------------
    /**
     * Return the Service Result object from the ResultHolder stored in session with the given key
     * 
     * @param sKey
     * @return object instance or null if not found
     */
    public Object getServiceResult(String sKey)
    {
        ServiceResultHolder srh = getServiceResultHolder(sKey);
        if (srh != null)
        {
            trace("getServiceResult(" + sKey + ") : holder found.");
            //--- Return the service result object
            return srh.getResult();
        }
        else
        {
            trace("getServiceResult(" + sKey + ") : holder not found.");
            return null;
        }
    }

    //---------------------------------------------------------------------------
    /**
     * Retun the login user (technical user)
     * 
     * @return
     */
    public LoginUser getLoginUser()
    {
        return _loginUser;
    }

    //---------------------------------------------------------------------------
    /**
     * Return the application user
     * 
     * @return
     */
    public IAppUser getAppUser()
    {
        return _appUser;
    }

    //---------------------------------------------------------------------------
    // get/set/clear every types of data
    //---------------------------------------------------------------------------
    /**
     * Stores the object value, maped with the specified key
     * 
     * @param key
     * @param value
     * @return
     */
    public Object put(Object key, Object value)
    {
        return _htData.put(key, value);
    }

    /**
     * Returns the object value stored with the specified key
     * 
     * @param key
     * @return
     */
    public Object get(Object key)
    {
        return _htData.get(key);
    }

    /**
     * Removes the object stored with the given key
     * 
     * @param key
     * @return
     */
    public Object remove(Object key)
    {
        return _htData.remove(key);
    }

    //---------------------------------------------------------------------------
    /**
     * Close the session (clean all the references held by the session)
     */
    public void close()
    {
        _httpSession = null;
        _loginUser = null;
        _appUser = null;
        _screens = null;
        _htData = null;
    }
    //---------------------------------------------------------------------------
}