package org.objectweb.telosys.screen.env;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpSession;

import org.objectweb.telosys.auth.DefaultAppUser;
import org.objectweb.telosys.auth.IAppUser;
import org.objectweb.telosys.auth.LoginUser;
import org.objectweb.telosys.common.Telosys;

/**
 * @author Laurent GUERIN
 *  
 */
public final class ScreenSessionManager // extends TelosysObject
{
    private static final String TELOSYS_SESSION = "TELOSYS_SESSION";

    /**
     * No constructor for this class
     */
    private ScreenSessionManager()
    {
    }

    /**
     * Try to find the Screen Session associated with the current HTTP Session <br>
     * Return null if the ScreenSession doesn't exist (no current HttpSession, <br>
     * or no ScreenSession in the HttpSession, <br>
     * 
     * @param request
     * @return ScreenSession instance or null
     */
    public static ScreenSession findScreenSession(HttpServletRequest request)
    {
        if (request != null)
        {
            //--- Try to get the HTTP Session (without creation if not found)
            HttpSession httpSession = request.getSession(false);
            if (httpSession != null)
            {
                //--- Try to find the Screen Session in the Http Session
                Object obj = httpSession.getAttribute(TELOSYS_SESSION);
                if (obj != null)
                {
                    if (obj instanceof ScreenSession)
                    {
                        return (ScreenSession) obj;
                    }
                    else
                    {
                        Telosys.error("findScreenSession: Http Session corrupted ! Attribute is not a ScreenSession instance !");
                    }
                }
            }
        }
        return null;
    }

    //---------------------------------------------------------------------------
    /**
     * Return the LoginUser instance associated with the current ScreenSession. <br>
     * Return null if the ScreenSession doesn't exist or if there's no LoginUser in the ScreenSession <br>
     * 
     * @param request
     * @return LoginUser instance or null
     */
    public static LoginUser findScreenSessionUser(HttpServletRequest request)
    {
        ScreenSession screenSession = findScreenSession(request);
        if (screenSession != null)
        {
            return screenSession.getLoginUser();
        }
        return null;
    }

    //---------------------------------------------------------------------------
    private static ScreenSession getScreenSession(HttpServletRequest request, boolean bCreate)
    {
        //--- Try to get the HTTP Session (without creation if not found)
        HttpSession httpSession = request.getSession(false);
        if (httpSession == null)
        {
            //--- No HTTP session
            if (!bCreate)
            {
                //--- No creation
                return null;
            }
            else
            {
                //--- create a "dummy user" session
                return createDummyScreenSession(request);
            }
        }
        else
        {
            Object obj = httpSession.getAttribute(TELOSYS_SESSION);
            if (obj == null)
            {
                //--- No Screen Session in the Http Session
                if (!bCreate)
                {
                    //--- No creation
                    return null;
                }
                else
                {
                    //--- create a "dummy user" session
                    return createDummyScreenSession(request);
                }
            }
            else
            {
                if (obj instanceof ScreenSession)
                {
                    return (ScreenSession) obj;
                }
                else
                {
                    Telosys.error("getScreenSession: Http Session corrupted ! Attribute is not a ScreenSession instance !");
                    return null;
                }
            }
        }
    }

    //---------------------------------------------------------------------------

    /**
     * Return the current Screen Session from the HttpRequest <br>
     * create a "dummy session" if not found and authentication not required
     * 
     * @param request
     * @return ScreenSession instance or null if not found
     */
    public static ScreenSession getScreenSession(HttpServletRequest request)
    {
        if (Telosys.isAuthenticationRequired())
        {
            //--- Authentication Requiered => cannot create a "dummy user" session
            return getScreenSession(request, false);
        }
        else
        {
            //--- Authentication Not Requiered => can create a "dummy user" session
            return getScreenSession(request, true);
        }
    }

    //---------------------------------------------------------------------------
    private static ScreenSession createScreenSession(HttpServletRequest request, LoginUser loginUser, IAppUser appUser)
    {
        //--- Get the HTTP Session (create it if necessary)
        HttpSession httpSession = request.getSession();
        if (httpSession == null)
        {
            Telosys.error("createScreenSession : cannot create the HTTP session ");
            //--- No HTTP session => cannot create the session !
            return null;
        }

        //--- Create a screen session object
        ScreenSession screenSession = new ScreenSession(httpSession, loginUser, appUser);

        //--- Store the screen session in the Http Session
        httpSession.setAttribute(TELOSYS_SESSION, screenSession);

        Telosys.info("ScreenSession created ( User = " + loginUser.getLogin() + " HTTP session Id = " + httpSession.getId() + ")");

        //--- Return the screen session created
        return screenSession;
    }

    //---------------------------------------------------------------------------
    private static ScreenSession createDummyScreenSession(HttpServletRequest request)
    {
        //--- Create a dummy user
        LoginUser loginUser = new LoginUser("NonAuthenticatedUser", "NoPassword");
        IAppUser appUser = new DefaultAppUser(loginUser.getLogin());
        return createScreenSession(request, loginUser, appUser);
    }

    //---------------------------------------------------------------------------
    /**
     * Open (create) a new Screen Session for a given user <br>( create a new HttpSession if it doesn't exist )
     * 
     * @param request
     * @param loginUser
     * @param appUser
     * @return
     */
    public static boolean openScreenSession(HttpServletRequest request, LoginUser loginUser, IAppUser appUser)
    {
        if (request == null)
        {
            Telosys.error("openScreenSession : request is null !");
            return false;
        }
        if (loginUser == null)
        {
            Telosys.error("openScreenSession : login user is null !");
            return false;
        }
        if (appUser == null)
        {
            Telosys.error("openScreenSession : application user is null !");
            return false;
        }

        Telosys.info("openScreenSession : create the session ( login = " + loginUser.getLogin() + " )");
        ScreenSession screenSession = createScreenSession(request, loginUser, appUser);
        return (screenSession != null ? true : false);
    }

    /**
     * Close the Screen Session associated with a request <br>( close the screen session and remove it from HttpSession )
     * 
     * @param request
     * @return
     */
    public static boolean closeScreenSession(HttpServletRequest request)
    {
        //--- Try to get the HTTP Session (without creation if not found)
        HttpSession httpSession = request.getSession(false);
        if (httpSession == null)
        {
            Telosys.error("closeScreenSession : cannot get the HTTP session ");
            return false;
        }
        else
        {
            ScreenSession screenSession = findScreenSession(request);
            if (screenSession != null)
            {
                LoginUser user = screenSession.getLoginUser();
                screenSession.close();
                httpSession.removeAttribute(TELOSYS_SESSION);
                Telosys.info("closeScreenSession ( login = " + user.getLogin() + " - Http Session Id = " + httpSession.getId() + " )");
                return true;
            }
            else
            {
                Telosys.error("closeScreenSession : cannot get the Screen Session ");
                return false;
            }
        }
    }
}