package org.objectweb.telosys.service;

import java.util.HashMap;

import org.objectweb.telosys.common.TelosysException;
import org.objectweb.telosys.common.TelosysObject;

/**
 * @author Laurent GUERIN
 *  
 */
public class StandardServiceRegistry extends TelosysObject implements IServiceProvider
{
    //=========================================================================================
    // INDEPENDENT (static) INNER CLASS 
    //=========================================================================================
    private static class ServiceComponents
    {
        private final static String ERR      = "ServiceProvider : cannot create service ";
        
        private String              _serviceName;

        private Class               _serviceClass;

        private String              _serviceRenderer ;

        private String              _serviceSessionKey ; 

        public ServiceComponents(String serviceName, Class serviceClass, String sRenderer, String sSessionKey ) throws TelosysException
        {
            final String sErrMsg = "Register Service '" + serviceName + "' : Invalid class ";
          
            try
            {
                //--- Service Name ------------------------------------
                _serviceName = serviceName;
                if (_serviceName == null)
                {
                    throw new TelosysException("Register Service : Service name is NULL ");
                }

                //--- Service class ------------------------------------
                _serviceClass = serviceClass;
                if (_serviceClass != null)
                {
                    if ( ScreenService.class.isAssignableFrom(_serviceClass) != true)
                    {
                        throw new TelosysException(sErrMsg);
                    }
                }
                
                //--- Service renderer ------------------------------------
                _serviceRenderer = sRenderer ; // can be null

                //--- Service key to store result in session---------------
                _serviceSessionKey = sSessionKey ; // can be null
                
            } catch (Throwable e)
            {
                throw new TelosysException("Register Service '"+ _serviceName + "' : Exception", e);
            }
        }

        /**
         * Create an instance of ScreenService
         * 
         * @return
         * @throws TelosysException
         */
        private ScreenService createService() throws TelosysException
        {
            try
            {
                //--- Create the screen instance with the class
                ScreenService service = (ScreenService) _serviceClass.newInstance();
                if (service != null)
                {
                    //--- Init the service
                    service.init( _serviceName, _serviceRenderer, _serviceSessionKey );
                    return service;
                }
                else
                {
                    throw new TelosysException(ERR + _serviceName + " (class.newInstance return null)");
                }
            } catch (InstantiationException e)
            {
                throw new TelosysException(ERR + _serviceName, e);
            } catch (IllegalAccessException e)
            {
                throw new TelosysException(ERR + _serviceName, e);
            } catch (Throwable e)
            {
                throw new TelosysException(ERR + _serviceName, e);
            }
        }

        private String getSessionKey() 
        {
            return _serviceSessionKey ;
        }
    }

    //=========================================================================================

    private final HashMap hmRegistry = new HashMap(40);

    /**
     * Register the service for future use.
     * @param serviceName the logical name of the service 
     * @param serviceClass the class where the service is implemented
     * @param sRenderer the renderer used to build the "result view" (no view if null) 
     * @param sSessionKey the key used to store the result in session (not stored if null)
     */
    protected final void register(String serviceName, Class serviceClass, String sRenderer, String sSessionKey  )
    {
        trace("register service " + serviceName);
        try
        {
            ServiceComponents sc = new ServiceComponents(serviceName, serviceClass, sRenderer, sSessionKey );
            hmRegistry.put(serviceName, sc);
        } catch (TelosysException e)
        {
            error("Cannot register screen " + serviceName);
            error(e.getMessage());
        }
    }
//    protected final void register(String serviceName, Class serviceClass, String sRenderer  )
//    {
//        register(serviceName, serviceClass, sRenderer, null);
//    }
    
    private final ServiceComponents lookup(String sServiceName) throws TelosysException
    {
        ServiceComponents servComp = (ServiceComponents) hmRegistry.get(sServiceName);
        if (servComp != null)
        {
            return servComp;
        }
        else
        {
            throw new TelosysException("Service " + sServiceName + " not found in the registy ");
        }
    }


    /* (non-Javadoc)
     * @see org.objectweb.telosys.service.IServiceProvider#getService(java.lang.String)
     */
    public ScreenService getService(String sServiceName) throws TelosysException
    {
        trace(" getService(" + sServiceName + ")");

        ServiceComponents def = lookup(sServiceName);
        if (def != null)
        {
            return def.createService();
        }
        return null;
    }
    
    public String getSessionKey(String sServiceName) throws TelosysException
    {
        trace(" getSessionKey(" + sServiceName + ")");

        ServiceComponents def = lookup(sServiceName);
        if (def != null)
        {
            return def.getSessionKey();
        }
        return null;
    }
    
}