package org.objectweb.telosys.uil.taglib;

import java.util.LinkedList;

import javax.servlet.jsp.PageContext;

import org.objectweb.telosys.common.TelosysClassLogger;

/**
 * Static methods to managed a given JSP "page context"
 * 
 * @author Laurent GUERIN
 *  
 */
public class Page
{
    private final static String             DEFAULT_DATA_ELEMENT    = "TELOSYS_DEFAULT_DATA_ELEMENT";

    private final static String             JS_LINES                = "TELOSYS_JS_LINES";

    private final static String             TABBOX_JS_LINES         = "TELOSYS_TABBOX_JS_LINES";

    private final static String             RADIO_GROUP_ITEMS       = "TELOSYS_RADIO_GROUP_ITEMS";

    private final static String             LAST_WIDGET_ID          = "TELOSYS_LAST_WIDGET_ID";

    private final static String             CURRENT_TAB_INDEX       = "TELOSYS_CURRENT_TAB_INDEX";

    private final static String             TAB_ID_SELECTED         = "TELOSYS_TAB_ID_SELECTED";

    public final static String              ITERATOR_COUNT          = "TELOSYS_ITERATOR_COUNT";

    public final static String              ITERATOR_INDEX          = "TELOSYS_ITERATOR_INDEX";

    public final static String              CURRENT_BEAN            = "TELOSYS_CURRENT_BEAN";

    private final static String             PREFIX_FOR_GENERATED_ID = "fwk_id";

    private static final TelosysClassLogger log                     = new TelosysClassLogger(Page.class);

    /**
     * Generate a tag ID (unique for the given page context)
     * 
     * @param pageContext
     * @return
     */
    public static String generateId(PageContext pageContext)
    {
        int iLastId = 0;
        //--- Try to find the last widget id
        Object oAttribute = pageContext.getAttribute(LAST_WIDGET_ID);
        if (oAttribute != null)
        {
            if (oAttribute instanceof Integer)
            {
                iLastId = ((Integer) oAttribute).intValue();
            }
            else
            {
                log.error("generateId(..) '" + LAST_WIDGET_ID + "' is not an instance of Integer !");
                iLastId = 0;
            }
        }
        else
        {
            //--- Never used before => init
            iLastId = 0;
        }
        //--- Next ID
        iLastId++;
        //--- Store it in the page context ..
        pageContext.setAttribute(LAST_WIDGET_ID, new Integer(iLastId));
        //--- Return the generated string ID
        return PREFIX_FOR_GENERATED_ID + iLastId;
    }

    /**
     * Data binding :<br>
     * Set (or clear) the default data element name for the given page context
     * 
     * @param pageContext
     * @param sElementName
     */
    public static void setDefaultDataElement(PageContext pageContext, String sElementName)
    {
        if (sElementName != null)
        {
            pageContext.setAttribute(DEFAULT_DATA_ELEMENT, sElementName);
        }
        else
        {
            pageContext.removeAttribute(DEFAULT_DATA_ELEMENT);
        }
    }

    /**
     * Data binding :<br>
     * Returns the default data element name from the given page context
     * 
     * @param pageContext
     *            the current page context to use
     * @return the data element name (or null if not found)
     */
    public static String getDefaultDataElement(PageContext pageContext)
    {
        Object oElement = pageContext.getAttribute(DEFAULT_DATA_ELEMENT);
        if (oElement != null)
        {
            if (oElement instanceof String)
            {
                String sElement = (String) oElement;
                return sElement.trim();
            }
        }
        return null;
    }

    //==============================================================================================
    // Register fields Javascript lines ( for <script> tag generation at the end )
    //==============================================================================================
    /**
     * Javascript generation :<br>
     * Returns the dynamic Javascript lines of the given page context
     * 
     * @param pageContext :
     *            the current page context to use
     * @param bCreate :
     *            if true the list is created if not found
     * @return lines list (or null if not found and bCreate is set to false)
     */
    public static LinkedList getJavascriptLines(PageContext pageContext, boolean bCreate)
    {
        LinkedList jsLines = null;
        //--- Try to found the list in the given page context
        Object oJavascriptLines = pageContext.getAttribute(JS_LINES);
        if (oJavascriptLines != null)
        {
            if (oJavascriptLines instanceof LinkedList)
            {
                jsLines = (LinkedList) oJavascriptLines;
            }
            else
            {
                log.error("Page contex attribute '" + JS_LINES + "' is not an instance of LinkedList ! ( => Create a new LinkedList instance ) ");
                jsLines = null; // Consider as not found
            }
        }
        //--- Create the list if not found
        if (jsLines == null && bCreate)
        {
            jsLines = new LinkedList();
            pageContext.setAttribute(JS_LINES, jsLines);
        }
        //--- Return the list
        return jsLines;
    }

    /**
     * Javascript generation :<br>
     * Returns the dynamic Javascript lines of the given page context
     * 
     * @param pageContext
     * @return lines list (or null if not found)
     */
    public static LinkedList getJavascriptLines(PageContext pageContext)
    {
        return getJavascriptLines(pageContext, false);
    }

    /**
     * Javascript generation :<br>
     * Clear the Javascript lines list (clear the list and remove it from the page context)
     * 
     * @param pageContext
     */
    public static void clearJavascriptLines(PageContext pageContext)
    {
        LinkedList jsLines = getJavascriptLines(pageContext, false);
        if (jsLines != null)
        {
            jsLines.clear();
            pageContext.removeAttribute(JS_LINES);
        }
    }

    //==============================================================================================
    // Radio items IDs management
    //==============================================================================================
    private static String getRadioItemsListName(String sRadioGroupId)
    {
        if (sRadioGroupId != null)
        {
            return RADIO_GROUP_ITEMS + "_" + sRadioGroupId.trim();
        }
        else
        {
            return RADIO_GROUP_ITEMS;
        }
    }

    private static LinkedList getRadioItems(PageContext pageContext, String sRadioGroupId, boolean bCreate)
    {
        LinkedList list = null;
        String sListName = getRadioItemsListName(sRadioGroupId);
        //--- Try to found the list in the given page context
        Object oJavascriptLines = pageContext.getAttribute(sListName);
        if (oJavascriptLines != null)
        {
            if (oJavascriptLines instanceof LinkedList)
            {
                list = (LinkedList) oJavascriptLines;
            }
            else
            {
                log.error("Page contex attribute '" + sListName + "' is not an instance of LinkedList ! ( => Create a new LinkedList instance ) ");
                list = null; // Consider as not found
            }
        }
        //--- Create the list if not found
        if (list == null && bCreate)
        {
            list = new LinkedList();
            pageContext.setAttribute(sListName, list);
        }
        //--- Return the list
        return list;
    }

    public static void registerRadioItem(PageContext pageContext, String sRadioGroupId, String sRadioItemId)
    {
        //--- Get the list from the page context (create it if not found)
        LinkedList list = getRadioItems(pageContext, sRadioGroupId, true);
        if (list != null)
        {
            list.add(sRadioItemId);
        }
        else
        {
            log.error("registerRadioItem(..) : Cannot get the list from page context ");
        }

    }

    public static LinkedList getRadioItems(PageContext pageContext, String sRadioGroupId)
    {
        return getRadioItems(pageContext, sRadioGroupId, false);
    }

    public static void clearRadioItems(PageContext pageContext, String sRadioGroupId)
    {
        //--- Get the list from the page context (create it if not found)
        LinkedList list = getRadioItems(pageContext, sRadioGroupId, false);
        if (list != null)
        {
            list.clear();
            String sListName = getRadioItemsListName(sRadioGroupId);
            pageContext.removeAttribute(sListName);
        }
    }

    //==============================================================================================
    // Tab Index management
    //==============================================================================================
    private static Integer getTabIndex(PageContext pageContext)
    {
        //--- Try to found the list in the given page context
        Object oTabIndex = pageContext.getAttribute(CURRENT_TAB_INDEX);
        Integer intTabIndex = null;
        if (oTabIndex != null)
        {
            if (oTabIndex instanceof Integer)
            {
                intTabIndex = ((Integer) oTabIndex);
            }
            else
            {
                log.error("Page contex attribute '" + CURRENT_TAB_INDEX + "' is not an instance of Integer ! ( => Create a new instance ) ");
                intTabIndex = null; // Consider as not found
            }
        }
        //--- Create the list if not found
        if (intTabIndex == null)
        {
            intTabIndex = new Integer(-1); // Not SET
            pageContext.setAttribute(CURRENT_TAB_INDEX, intTabIndex);
        }
        return intTabIndex;
    }

    //    public static void initTabIndex(PageContext pageContext, int iValue)
    //    {
    //        pageContext.setAttribute(CURRENT_TAB_INDEX, new Integer(iValue));
    //    }
    public static void initTabIndex(PageContext pageContext)
    {
        pageContext.setAttribute(CURRENT_TAB_INDEX, new Integer(-1));
    }

    /**
     * Set the next tab index and return it
     * 
     * @param pageContext
     * @return : the new tab index ( from 0 to N )
     */
    public static int getNextTabIndex(PageContext pageContext)
    {
        Integer intTabIndex = getTabIndex(pageContext);
        if (intTabIndex != null)
        {
            int i = intTabIndex.intValue();
            if (i < 0)
            {
                i = 0; // FIRST
            }
            else
            {
                i++; // NEXT
            }
            pageContext.setAttribute(CURRENT_TAB_INDEX, new Integer(i));
            return i;
        }
        return -1;
    }

    /**
     * Clear the current tab index ( reset for future use )
     * 
     * @param pageContext
     * @return
     */
    public static void clearTabIndex(PageContext pageContext)
    {
        pageContext.removeAttribute(CURRENT_TAB_INDEX);
    }

    //==============================================================================================
    // Selected Tab ID management
    //==============================================================================================
    /**
     * Set the ID of the selected tab in the current tabbox
     * 
     * @param pageContext
     * @param sTabId
     */
    public static void setSelectedTabId(PageContext pageContext, String sTabId)
    {
        pageContext.setAttribute(TAB_ID_SELECTED, sTabId);
    }

    /**
     * Get the ID of the selected tab in the current tabbox
     * 
     * @param pageContext
     * @return
     */
    public static String getSelectedTabId(PageContext pageContext)
    {
        Object obj = pageContext.getAttribute(TAB_ID_SELECTED);
        Integer intTabIndex = null;
        if (obj != null)
        {
            if (obj instanceof String)
            {
                return (String) obj;
            }
            else
            {
                log.error("Page contex attribute '" + TAB_ID_SELECTED + "' is not an instance of String !");
            }
        }
        return null;
    }

    /**
     * Clear the ID of the selected tab in the current tabbox
     * 
     * @param pageContext
     */
    public static void clearSelectedTabId(PageContext pageContext)
    {
        pageContext.removeAttribute(TAB_ID_SELECTED);
    }

    //==============================================================================================
    // Generic Javascript lines ( for <script> tag generation at the end )
    //==============================================================================================
    /**
     * Javascript generation :<br>
     * Returns the dynamic Javascript lines of the given page context
     * 
     * @param pageContext :
     *            the current page context to use
     * @param bCreate :
     *            if true the list is created if not found
     * @return lines list (or null if not found and bCreate is set to false)
     */
    public static LinkedList getJSLines(PageContext pageContext, String sId, boolean bCreate)
    {
        LinkedList jsLines = null;
        //--- Try to found the list in the given page context
        Object oJavascriptLines = pageContext.getAttribute(JS_LINES);
        if (oJavascriptLines != null)
        {
            if (oJavascriptLines instanceof LinkedList)
            {
                jsLines = (LinkedList) oJavascriptLines;
            }
            else
            {
                log.error("Page contex attribute '" + JS_LINES + "' is not an instance of LinkedList ! ( => Create a new LinkedList instance ) ");
                jsLines = null; // Consider as not found
            }
        }
        //--- Create the list if not found
        if (jsLines == null && bCreate)
        {
            jsLines = new LinkedList();
            pageContext.setAttribute(JS_LINES, jsLines);
        }
        //--- Return the list
        return jsLines;
    }

    /**
     * Javascript generation :<br>
     * Returns the dynamic Javascript lines of the given page context
     * 
     * @param pageContext
     * @return lines list (or null if not found)
     */
    public static LinkedList getJSLines(PageContext pageContext, String sId)
    {
        return getJSLines(pageContext, sId, false);
    }

    /**
     * Javascript generation :<br>
     * Clear the Javascript lines list (clear the list and remove it from the page context)
     * 
     * @param pageContext
     */
    public static void clearJSLines(PageContext pageContext, String sId)
    {
        LinkedList jsLines = getJSLines(pageContext, sId, false);
        if (jsLines != null)
        {
            jsLines.clear();
            pageContext.removeAttribute(JS_LINES);
        }
    }

    //==============================================================================================
    // Iteration count management
    //==============================================================================================
    /**
     * Set the iterator count
     * 
     * @param pageContext
     * @param iCount
     */
    public static void setIterationCount(PageContext pageContext, int iCount)
    {
        pageContext.setAttribute(ITERATOR_COUNT, new Integer(iCount));
    }

    /**
     * Get the iterator count
     * 
     * @param pageContext
     * @return
     */
    public static int getIterationCount(PageContext pageContext)
    {
        Object obj = pageContext.getAttribute(ITERATOR_COUNT);
        if (obj != null)
        {
            if (obj instanceof Integer)
            {
                return ((Integer) obj).intValue();
            }
            else
            {
                log.error("Page contex attribute '" + ITERATOR_COUNT + "' is not an instance of Integer !");
            }
        }
        return 0;
    }

    //==============================================================================================
    // Iteration index management
    //==============================================================================================
    /**
     * Set the iterator index
     * 
     * @param pageContext
     * @param iIndex
     */
    public static void setIterationIndex(PageContext pageContext, int iIndex)
    {
        pageContext.setAttribute(ITERATOR_INDEX, new Integer(iIndex));
    }

    /**
     * Get the iterator index
     * 
     * @param pageContext
     * @return
     */
    public static int getIterationIndex(PageContext pageContext)
    {
        Object obj = pageContext.getAttribute(ITERATOR_INDEX);
        if (obj != null)
        {
            if (obj instanceof Integer)
            {
                return ((Integer) obj).intValue();
            }
            else
            {
                log.error("Page contex attribute '" + ITERATOR_COUNT + "' is not an instance of Integer !");
            }
        }
        return 0;
    }

    //==============================================================================================
    // Current bean management
    //==============================================================================================
    /**
     * Set the current bean
     * 
     * @param pageContext
     * @param oBean :
     *            current bean to set
     */
    public static void setCurrentBean(PageContext pageContext, Object oBean)
    {
        pageContext.setAttribute(CURRENT_BEAN, oBean);
    }

    /**
     * Get the current bean
     * 
     * @param pageContext
     * @return
     */
    public static Object getCurrentBean(PageContext pageContext)
    {
        return pageContext.getAttribute(CURRENT_BEAN);
    }
    
    /**
     * Clear the current bean ( reset for future use )
     * 
     * @param pageContext
     * @return
     */
    public static void clearCurrentBean(PageContext pageContext)
    {
        pageContext.removeAttribute(CURRENT_BEAN);
    }
}