package org.objectweb.telosys.uil.taglib.widget;

import javax.servlet.ServletRequest;

import org.objectweb.telosys.common.TelosysConst;
import org.objectweb.telosys.screen.env.ScreenSession;
import org.objectweb.telosys.uil.ScreenPostback;
import org.objectweb.telosys.uil.taglib.TagCommons;

/**
 * Ancestor for tags with postback values management <br>
 * 
 * @author Laurent GUERIN
 *  
 */
public class GenericPostbackTag extends GenericTag
{
    //----------------------------------------------------------------
    // Nested Item tags common attributes
    //----------------------------------------------------------------

    private String _sValue    = null;

    private String _sPostback = null;

    /**
     * Constructor
     * 
     * @param sTagName
     * @param htmlGen
     * @param xulGen
     */
    public GenericPostbackTag(String sTagName, IWidget htmlGen, IWidget xulGen)
    {
        super(sTagName, htmlGen, xulGen);
    }

    //----------------------------------------------------------------
    public void setValue(String v)
    {
        _sValue = v;
    }

    public String getValue()
    {
        return _sValue;
    }

    //----------------------------------------------------------------
    /**
     * Search a postback value in request or session scope 
     * @return the value found (null if not found or if the widget is not postback managed)
     */
    public String getPostbackValue()
    {
        trace("getPostbackValue()");
        String sFormName = this.getCurrentFormName();
        String sFieldName = this.getName();

        if (this.isSessionPostback())
        {
            trace("Search in Session Postback ...");
            String sPostbackValue = getSessionPostbackValue(sFormName, sFieldName);
            trace("Session Postback Value = " + sPostbackValue);
            return sPostbackValue ;
        }
        else if (this.isRequestPostback())
        {
            trace("Search in Request Postback ...");
            String sPostbackValue = getRequestPostbackValue(sFormName, sFieldName);
            trace("Request Postback Value = " + sPostbackValue);
            return sPostbackValue ;
        }
        else
        {
            return null;
        }        
    }
    //----------------------------------------------------------------
    /**
     * Returns the value or the postback value if any
     * @return
     */
    public String getValueWithPostback()
    {
        trace("getValueWithPostback()");
        if ( isNoPostback() ) // postback="none"
        {
            trace("Postback disabled => use the current value = '" + _sValue + "' )");
            return _sValue;
        }
        else
        {
            trace("Postback enabled => get Postback ...");
            //--- Postback is not disabled => try to get a pre-stored value in the postback container
            String sPostbackValue = getPostbackValue();
            return (sPostbackValue != null ? sPostbackValue : _sValue);            
        }
    }

    //----------------------------------------------------------------
    public void setPostback(String v)
    {
        _sPostback = v;
    }

    //    public String getPostback()
    //    {
    //        return _sPostBack;
    //    }
    //----------------------------------------------------------------
    /**
     * Returns true if the widget is explicitly declared with no postback  
     * @return
     */
    public boolean isNoPostback()
    {
        return TagCommons.isNoPostback(_sPostback) ;
    }
    //----------------------------------------------------------------
    /**
     * Returns true if the widget postack is enabled ( session or request )
     * @return
     */
    protected boolean isPostbackEnabled()
    {
        if ( isSessionPostback() || isRequestPostback() )
        {
            return true ;
        }
        else
        {
            return false ;
        }
    }
    //----------------------------------------------------------------
    public boolean isSessionPostback()
    {
        if (TagCommons.isNoPostback(_sPostback))
        {
            //--- A specific postback attribute was set to "none" for this tag
            return false;
        }
        else
        {
            //--- The postback type is inherited from the parent "form" tag
            return TagCommons.isSessionPostback(this.getCurrentFormPostback());
        }
    }

    //----------------------------------------------------------------
    public boolean isRequestPostback()
    {
        if (TagCommons.isNoPostback(_sPostback))
        {
            //--- A specific postback attribute was set to "none" for this tag
            return false;
        }
        else
        {
            //--- The postback type is inherited from the parent "form" tag
            return TagCommons.isRequestPostback(this.getCurrentFormPostback());
        }
    }

    //----------------------------------------------------------------
    /**
     * Returns the ScreenPostback for the current screen<br>
     * The ScreenPostback is searched in the inherited postback scope (request or session) 
     * @return the ScreenPostback (or null if not found)
     */
    public ScreenPostback getScreenPostback()
    {
        if (this.isSessionPostback())
        {
            return getSessionScreenPostback();
        }
        else if (this.isRequestPostback())
        {
            return getRequestScreenPostback();
        }
        else
        {
            return null;
        }        
    }
    //----------------------------------------------------------------
    /**
     * Returns the ScreenPostback from the REQUEST scope for the current screen
     * @return the ScreenPostback (or null if not found)
     */
    private ScreenPostback getRequestScreenPostback()
    {
        trace("getRequestScreenPostback() ...");
        //--- Retrieve the http servlet request
        ServletRequest httpRequest = this.getHttpServletRequest();
        if (httpRequest == null)
        {
            error("Cannot get HttpServletRequest ! ");
        }
        else
        {
            //--- Try to find a "postback entry" in the request
            Object oScreenPostback = httpRequest.getAttribute(TelosysConst.POSTBACK_KEY);
            if (oScreenPostback != null)
            {
                if (oScreenPostback instanceof ScreenPostback)
                {
                    return (ScreenPostback) oScreenPostback;
                }
                else
                {
                    error("Request attribute '" + TelosysConst.POSTBACK_KEY + "' is not a ScreenPostback instance !");
                }
            }
        }
        return null ; // ScreenPostback not found
    }
    //----------------------------------------------------------------
    /**
     * Returns the ScreenPostback from the SESSION scope for the current screen
     * @param sFormName
     * @return the ScreenPostback (or null if not found)
     */
    private ScreenPostback getSessionScreenPostback()
    {
        trace("getSessionScreenPostback() ...");
        ScreenSession screenSession = this.getScreenSession();
        if (screenSession != null)
        {
            String sScreenName = this.getScreenMarkerName();
            int iScreenContextId = this.getScreenContextId();
            //--- Retrieve the ScreenPostback object in the ScreenSession for this screen
            return screenSession.getScreenPostback(sScreenName, iScreenContextId);
        }
        else
        {
            error("Cannot get ScreenSession ! ");
        }
        return null; // ScreenPostback not found
    }
    //----------------------------------------------------------------
    private String getRequestPostbackValue(String sFormName, String sFieldName)
    {
        String sValue = null ;
        ScreenPostback screenPostback = getRequestScreenPostback();
        if (screenPostback != null)
        {
            sValue =  screenPostback.getFieldValue(sFormName, sFieldName);
        }
        trace("getRequestPostbackValue(" + sFormName + "," + sFieldName + ") --> '" + sValue + "' ");
        return sValue;
        
//        //--- Retrieve the http servlet request
//        ServletRequest httpRequest = this.getHttpServletRequest();
//        if (httpRequest == null)
//        {
//            // Error
//            return null;
//        }
//        //--- Try to find a "postback entry" in the request
//        Object oScreenPostback = httpRequest.getAttribute(TelosysConst.POSTBACK_KEY);
//        if (oScreenPostback != null)
//        {
//            if (oScreenPostback instanceof ScreenPostback)
//            {
//                ScreenPostback screenPostback = (ScreenPostback) oScreenPostback;
//                //--- Get the field value pre-stored in the ScreenPostback
//                String sValue = screenPostback.getFieldValue(sFormName, sFieldName);
//                //trace("Field " + sFieldName + " : Postback value = " + sValue );
//                return sValue;
//            }
//            else
//            {
//                // Error
//            }
//        }
//        return null; // No postback value
    }

    //----------------------------------------------------------------
    private String getSessionPostbackValue(String sFormName, String sFieldName)
    {
        String sValue = null ;
        ScreenPostback screenPostback = getSessionScreenPostback();
        if (screenPostback != null)
        {
            sValue =  screenPostback.getFieldValue(sFormName, sFieldName);
        }
        trace("getSessionPostbackValue(" + sFormName + "," + sFieldName + ") --> '" + sValue + "' ");
        return sValue;

//        trace("getSessionPostbackValue(" + sFormName + "," + sFieldName + " ) ...");
//        //--- Retrieve the ScreenSession object
//        ScreenSession screenSession = this.getScreenSession();
//        if (screenSession != null)
//        {
//            String sScreenName = this.getScreenName();
//            int iScreenContextId = this.getScreenContextId();
//            //--- Retrieve the ScreenPostback object in the ScreenSession for this screen
//            ScreenPostback screenPostback = screenSession.getScreenPostback(sScreenName, iScreenContextId);
//            if (screenPostback != null)
//            {
//                //--- Get the field value pre-stored in the ScreenPostback
//                String sValue = screenPostback.getFieldValue(sFormName, sFieldName);
//                //trace("Field " + sFieldName + " : Postback value = " + sValue );
//                return sValue;
//            }
//            else
//            {
//                // Error
//            }
//        }
//        else
//        {
//            // Error
//        }
//        return null; // No postback value
    }

}