package org.objectweb.telosys.util;

import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.jar.JarEntry;
import java.util.jar.JarFile;

import org.objectweb.telosys.common.TelosysClassLogger;
import org.objectweb.telosys.common.TelosysException;

/**
 * @author Laurent GUERIN
 *
 */
public class JarUtil
{
    private static final TelosysClassLogger $log = new TelosysClassLogger(JarUtil.class);

    /**
     * Open a JAR file
     * @param sJarFile
     * @return JarFile instance
     * @throws TelosysException
     */
    private static JarFile openJarFile(String sJarFile) throws TelosysException
    {
        JarFile jarFile = null;
        try
        {
            jarFile = new JarFile(sJarFile);
        } catch (IOException e)
        {
            throw new TelosysException("Cannot open JAR file '" + sJarFile + "'.", e);
        }
        return jarFile;
    }

    /**
     * Copy an input stream in an output stream
     * @param inStream
     * @param outStream
     * @throws IOException
     */
    private static void copyStream(InputStream inStream, OutputStream outStream) throws IOException
    {
        byte[] buffer = new byte[1024];
        int bytesRead;

        //--- Read input data and write it to the output stream
        while ((bytesRead = inStream.read(buffer)) != -1)
        {
            outStream.write(buffer, 0, bytesRead);
        }
    }
    

    /**
     * Extract a file from a JAR file (replace the extracted file if it exists)
     * @param sJarFile jar file name
     * @param sJarEntry file name to extract
     * @param sExtractedFile file name of the extracted file 
     * @throws TelosysException
     */
    public static void extract(String sJarFile, String sJarEntry, String sExtractedFile) throws TelosysException
    {
        JarFile jarFile = null;
        InputStream inputStream = null;
        FileOutputStream outputStream = null;

        if ( sJarFile == null )
        {
            throw new TelosysException("Jar file name is null");
        }
        if ( sJarEntry == null )
        {
            throw new TelosysException("Jar entry name is null");
        }
        if ( sExtractedFile == null )
        {
            throw new TelosysException("Extracted file name is null");
        }
        
        try
        {
            jarFile = openJarFile(sJarFile);
            //System.out.println("JAR file " + sJarFile + " opened.");

            JarEntry jarEntry = jarFile.getJarEntry(sJarEntry);

            if (jarEntry != null)
            {
                //System.out.println("JAR entry " + sJarEntry + " found.");

                //--- Get an input stream for the entry.
                try
                {
                    inputStream = jarFile.getInputStream(jarEntry);
                } catch (IOException e)
                {
                    throw new TelosysException("Cannot get Input Stream for entry '" + sJarEntry + "' not found in JAR file '" + sJarFile + "'.");
                }

                //--- Create the output file (clobbering the file if it exists).
                try
                {
                    outputStream = new FileOutputStream(sExtractedFile);
                } catch (FileNotFoundException e1)
                {
                    throw new TelosysException("Cannot create output file '" + sExtractedFile + "'.");
                }

                //--- Create output file content (copy) 
                try
                {
                    copyStream(inputStream, outputStream);
                } catch (IOException e2)
                {
                    throw new TelosysException("Cannot copy stream.", e2);
                }
            }
            else
            {
                throw new TelosysException("Entry '" + sJarEntry + "' not found in JAR file '" + sJarFile + "'.");
            }
        } finally
        {
            if (outputStream != null)
            {
                //System.out.println("Close outputStream.");
                try
                {
                    outputStream.close();
                } catch (IOException e)
                {
                    // log and continue
                    $log.error("Cannot close output stream ('" + sExtractedFile + "').");
                }
            }
            if (inputStream != null)
            {
                //System.out.println("Close inputStream.");
                try
                {
                    inputStream.close();
                } catch (IOException e)
                {
                    // log and continue
                    $log.error("Cannot close input stream ('" + sJarEntry + "').");
                }
            }
            if (jarFile != null)
            {
                //System.out.println("Close jar file.");
                try
                {
                    jarFile.close();
                } catch (IOException e)
                {
                    // log and continue
                    $log.error("Cannot close jar file ('" + sJarFile + "').");
                }
            }
        }
    }

}
